@@ -1,7 +1,7 @@
 /* capture.c
  * Routines for packet capture windows
  *
- * $Id: capture.c,v 1.71 1999/09/23 07:57:16 guy Exp $
+ * $Id: capture.c,v 1.83 1999/11/29 01:54:00 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -38,11 +38,16 @@
 # include <sys/stat.h>
 #endif
 
+#ifdef HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
+
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
+#include <fcntl.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -74,7 +79,12 @@
 # include "snprintf.h"
 #endif
 
+#ifndef lib_pcap_h
+#include <pcap.h>
+#endif
+
 #include "gtk/main.h"
+#include "gtk/gtkglobals.h"
 #include "packet.h"
 #include "file.h"
 #include "gtk/menu.h"
@@ -83,6 +93,13 @@
 #include "prefs.h"
 #include "globals.h"
 
+int sync_mode;	/* fork a child to do the capture, and sync between them */
+static int sync_pipe[2]; /* used to sync father */
+int quit_after_cap; /* Makes a "capture only mode". Implies -k */
+gboolean capture_child;	/* if this is the child for "-S" */
+static guint cap_input_id;
+
+static void cap_file_input_cb(gpointer, gint, GdkInputCondition);
 static void capture_stop_cb(GtkWidget *, gpointer);
 static void capture_pcap_cb(u_char *, const struct pcap_pkthdr *,
   const u_char *);
@@ -97,21 +114,34 @@ typedef struct _loop_data {
   wtap_dumper   *pdh;
 } loop_data;
 
+/* Open a specified file, or create a temporary file, and start a capture
+   to the file in question. */
 void
-do_capture(void)
+do_capture(char *capfile_name)
 {
   char tmpname[128+1];
-  char c;
+  gboolean is_temp_file;
+  u_char c;
   int i;
   guint byte_count;
   char *msg;
+  int err;
+  int capture_succeeded;
 
-  /* Choose a random name for the capture buffer */
-  cf.save_file_fd = create_tempfile(tmpname, sizeof tmpname, "ether");
+  if (capfile_name != NULL) {
+    /* Try to open/create the specified file for use as a capture buffer. */
+    cf.save_file_fd = open(capfile_name, O_RDWR|O_TRUNC|O_CREAT, 0600);
+    is_temp_file = FALSE;
+  } else {
+    /* Choose a random name for the capture buffer */
+    cf.save_file_fd = create_tempfile(tmpname, sizeof tmpname, "ether");
+    capfile_name = g_strdup(tmpname);
+    is_temp_file = TRUE;
+  }
   if (cf.save_file_fd == -1) {
     simple_dialog(ESD_TYPE_WARN, NULL,
 	"The file to which the capture would be saved (\"%s\")"
-	"could not be opened: %s.", tmpname, strerror(errno));
+	"could not be opened: %s.", capfile_name, strerror(errno));
     return;
   }
   close_cap_file(&cf, info_bar, file_ctx);
@@ -121,120 +151,327 @@ do_capture(void)
       unlink(cf.save_file); /* silently ignore error */
     g_free(cf.save_file);
   }
-  cf.save_file = g_strdup(tmpname);
-  cf.user_saved = 0;
-  
-  if (sync_mode || fork_mode) {	/*  use fork() for capture */
+  cf.save_file = capfile_name;
+  cf.user_saved = !is_temp_file;
+
+  if (sync_mode) {	/*  use fork() for capture */
     int  fork_child;
     char ssnap[24];
     char scount[24];	/* need a constant for len of numbers */
     char save_file_fd[24];
-    int err;
 
     sprintf(ssnap,"%d",cf.snap); /* in lieu of itoa */
     sprintf(scount,"%d",cf.count);
     sprintf(save_file_fd,"%d",cf.save_file_fd);
     signal(SIGCHLD, SIG_IGN);
-    if (sync_mode)
-      pipe(sync_pipe);
+    pipe(sync_pipe);
     if ((fork_child = fork()) == 0) {
-      /* args: -k -- capture
-       * -i interface specification
+      /*
+       * Child process - run Ethereal with the right arguments to make
+       * it just pop up the live capture dialog box and capture with
+       * the specified capture parameters, writing to the specified file.
+       *
+       * args: -i interface specification
        * -w file to write
        * -W file descriptor to write
        * -c count to capture
-       * -Q quit after capture (forces -k)
        * -s snaplen
-       * -S sync mode
        * -m / -b fonts
        * -f "filter expression"
        */
-       if (sync_mode) {
-	 close(1);
-	 dup(sync_pipe[1]);
-	 close(sync_pipe[0]);
-	 execlp(ethereal_path, "ethereal", "-k", "-Q", "-i", cf.iface,
+      close(1);
+      dup(sync_pipe[1]);
+      close(sync_pipe[0]);
+      execlp(ethereal_path, CHILD_NAME, "-i", cf.iface,
 		"-w", cf.save_file, "-W", save_file_fd,
-		"-c", scount, "-s", ssnap, "-S", 
+		"-c", scount, "-s", ssnap, 
 		"-m", medium_font, "-b", bold_font,
 		(cf.cfilter == NULL)? 0 : "-f",
 		(cf.cfilter == NULL)? 0 : cf.cfilter,
 		(const char *)NULL);	
-       }
-       else {
-	 execlp(ethereal_path, "ethereal", "-k", "-Q", "-i", cf.iface,
-		"-w", cf.save_file, "-W", save_file_fd,
-		"-c", scount, "-s", ssnap,
-		"-m", medium_font, "-b", bold_font,
-		(cf.cfilter == NULL)? 0 : "-f",
-		(cf.cfilter == NULL)? 0 : cf.cfilter,
-		(const char *)NULL);
-       }
-    }
-    else {
-       cf.filename = cf.save_file;
-       if (sync_mode) {
-	 close(sync_pipe[1]);
+    } else {
+      /* Parent process - read messages from the child process over the
+         sync pipe. */
+      cf.filename = cf.save_file;
+      close(sync_pipe[1]);
 
-	 /* Read a byte count from "sync_pipe[0]", terminated with a
-	    colon; if the count is 0, the child process created the
-	    capture file and we should start reading from it, otherwise
-	    the capture couldn't start and the count is a count of bytes
-	    of error message, and we should display the message. */
-	 byte_count = 0;
-	 for (;;) {
-	   i = read(sync_pipe[0], &c, 1);
-	   if (i == 0) {
-	     /* EOF - the child process died.
-	        XXX - reap it and report the status. */
-	     simple_dialog(ESD_TYPE_WARN, NULL, "Capture child process died");
-	     return;
-	   }
-	   if (c == ';')
-	     break;
-	   if (!isdigit(c)) {
-	     /* Child process handed us crap. */
-	     simple_dialog(ESD_TYPE_WARN, NULL,
-	        "Capture child process sent us a bad message");
-	     return;
-	   }
-	   byte_count = byte_count*10 + c - '0';
-	 }
-	 if (byte_count == 0) {
-	   /* Success. */
-	   err = tail_cap_file(cf.save_file, &cf);
-	   if (err != 0) {
-	     simple_dialog(ESD_TYPE_WARN, NULL,
+      /* Read a byte count from "sync_pipe[0]", terminated with a
+	 colon; if the count is 0, the child process created the
+	 capture file and we should start reading from it, otherwise
+	 the capture couldn't start and the count is a count of bytes
+	 of error message, and we should display the message. */
+      byte_count = 0;
+      for (;;) {
+	i = read(sync_pipe[0], &c, 1);
+	if (i == 0) {
+	  /* EOF - the child process died.
+	     Close the read side of the sync pipe, remove the capture file,
+	     and report the failure.
+	     XXX - reap the child process and report the status in detail. */
+	  close(sync_pipe[0]);
+	  unlink(cf.save_file);
+	  cf.save_file = NULL;
+	  simple_dialog(ESD_TYPE_WARN, NULL, "Capture child process died");
+	  return;
+	}
+	if (c == ';')
+	  break;
+	if (!isdigit(c)) {
+	  /* Child process handed us crap.
+	     Close the read side of the sync pipe, remove the capture file,
+	     and report the failure. */
+	  close(sync_pipe[0]);
+	  unlink(cf.save_file);
+	  cf.save_file = NULL;
+	  simple_dialog(ESD_TYPE_WARN, NULL,
+	     "Capture child process sent us a bad message");
+	  return;
+	}
+	byte_count = byte_count*10 + c - '0';
+      }
+      if (byte_count == 0) {
+	/* Success.  Open the capture file, and set up to read it. */
+	err = start_tail_cap_file(cf.save_file, &cf);
+	if (err == 0) {
+	  /* We were able to open and set up to read the capture file;
+	     arrange that our callback be called whenever it's possible
+	     to read from the sync pipe, so that it's called when
+	     the child process wants to tell us something. */
+	  cap_input_id = gtk_input_add_full(sync_pipe[0],
+				       GDK_INPUT_READ,
+				       cap_file_input_cb,
+				       NULL,
+				       (gpointer) &cf,
+				       NULL);
+	} else {
+	  /* We weren't able to open the capture file; complain, and
+	     close the sync pipe. */
+	  simple_dialog(ESD_TYPE_WARN, NULL,
 			file_open_error_message(err, FALSE), cf.save_file);
-	   }
-	 } else {
-	   /* Failure. */
-	   msg = g_malloc(byte_count + 1);
-	   if (msg == NULL) {
-	     simple_dialog(ESD_TYPE_WARN, NULL,
-	        "Capture child process failed, but its error message was too big.");
-	   } else {
-	     i = read(sync_pipe[0], msg, byte_count);
-	     if (i < 0) {
-		simple_dialog(ESD_TYPE_WARN, NULL,
+
+	  /* Close the sync pipe. */
+	  close(sync_pipe[0]);
+
+	  /* Don't unlink the save file - leave it around, for debugging
+	     purposes. */
+	  cf.save_file = NULL;
+	}
+      } else {
+	/* Failure - the child process sent us a message indicating
+	   what the problem was. */
+	msg = g_malloc(byte_count + 1);
+	if (msg == NULL) {
+	  simple_dialog(ESD_TYPE_WARN, NULL,
+		"Capture child process failed, but its error message was too big.");
+	} else {
+	  i = read(sync_pipe[0], msg, byte_count);
+	  if (i < 0) {
+	    simple_dialog(ESD_TYPE_WARN, NULL,
 		  "Capture child process failed: Error %s reading its error message.",
 		  strerror(errno));
-	     } else if (i == 0) {
-		simple_dialog(ESD_TYPE_WARN, NULL,
+	  } else if (i == 0) {
+	    simple_dialog(ESD_TYPE_WARN, NULL,
 		  "Capture child process failed: EOF reading its error message.");
-	     } else
-		simple_dialog(ESD_TYPE_WARN, NULL, msg);
-	     g_free(msg);
-	   }
-	 }
-       }
+	  } else
+	    simple_dialog(ESD_TYPE_WARN, NULL, msg);
+	  g_free(msg);
+
+	  /* Close the sync pipe. */
+	  close(sync_pipe[0]);
+
+	  /* Get rid of the save file - the capture never started. */
+	  unlink(cf.save_file);
+	  cf.save_file = NULL;
+	}
+      }
+    }
+  } else {
+    /* Not sync mode. */
+    capture_succeeded = capture();
+    if (quit_after_cap) {
+      /* DON'T unlink the save file.  Presumably someone wants it. */
+      gtk_exit(0);
+    }
+    if (capture_succeeded) {
+      /* Capture succeeded; read in the capture file. */
+      if ((err = open_cap_file(cf.save_file, &cf)) == 0) {
+        /* Set the read filter to NULL. */
+        cf.rfcode = NULL;
+        err = read_cap_file(&cf);
+        set_menu_sensitivity("/File/Save", TRUE);
+        set_menu_sensitivity("/File/Save As...", FALSE);
+      }
     }
   }
-  else
-    capture();
 }
 
-void
+/* There's stuff to read from the sync pipe, meaning the child has sent
+   us a message, or the sync pipe has closed, meaning the child has
+   closed it (perhaps because it exited). */
+static void 
+cap_file_input_cb(gpointer data, gint source, GdkInputCondition condition)
+{
+  capture_file *cf = (capture_file *)data;
+  char buffer[256+1], *p = buffer, *q = buffer;
+  int  nread;
+  int  to_read = 0;
+  gboolean exit_loop = FALSE;
+  int  err;
+  int  wstatus;
+  int  wsignal;
+  char *msg;
+  char *sigmsg;
+  char sigmsg_buf[6+1+3+1];
+  char *coredumped;
+
+  /* avoid reentrancy problems and stack overflow */
+  gtk_input_remove(cap_input_id);
+
+  if ((nread = read(sync_pipe[0], buffer, 256)) <= 0) {
+    /* The child has closed the sync pipe, meaning it's not going to be
+       capturing any more packets.  Pick up its exit status, and
+       complain if it died of a signal. */
+    if (wait(&wstatus) != -1) {
+      /* XXX - are there any platforms on which we can run that *don't*
+         support POSIX.1's <sys/wait.h> and macros therein? */
+      wsignal = wstatus & 0177;
+      coredumped = "";
+      if (wstatus == 0177) {
+      	/* It stopped, rather than exiting.  "Should not happen." */
+      	msg = "stopped";
+      	wsignal = (wstatus >> 8) & 0xFF;
+      } else {
+        msg = "terminated";
+        if (wstatus & 0200)
+          coredumped = " - core dumped";
+      }
+      if (wsignal != 0) {
+        switch (wsignal) {
+
+        case SIGHUP:
+          sigmsg = "Hangup";
+          break;
+
+        case SIGINT:
+          sigmsg = "Interrupted";
+          break;
+
+        case SIGQUIT:
+          sigmsg = "Quit";
+          break;
+
+        case SIGILL:
+          sigmsg = "Illegal instruction";
+          break;
+
+        case SIGTRAP:
+          sigmsg = "Trace trap";
+          break;
+
+        case SIGABRT:
+          sigmsg = "Abort";
+          break;
+
+        case SIGFPE:
+          sigmsg = "Arithmetic exception";
+          break;
+
+        case SIGKILL:
+          sigmsg = "Killed";
+          break;
+
+        case SIGBUS:
+          sigmsg = "Bus error";
+          break;
+
+        case SIGSEGV:
+          sigmsg = "Segmentation violation";
+          break;
+
+	/* http://metalab.unc.edu/pub/Linux/docs/HOWTO/GCC-HOWTO 
+		Linux is POSIX compliant.  These are not POSIX-defined signals ---
+		  ISO/IEC 9945-1:1990 (IEEE Std 1003.1-1990), paragraph B.3.3.1.1 sez:
+
+	       ``The signals SIGBUS, SIGEMT, SIGIOT, SIGTRAP, and SIGSYS
+		were omitted from POSIX.1 because their behavior is
+		implementation dependent and could not be adequately catego-
+		rized.  Conforming implementations may deliver these sig-
+		nals, but must document the circumstances under which they
+		are delivered and note any restrictions concerning their
+		delivery.''
+	*/
+
+	#ifdef SIGSYS
+        case SIGSYS:
+          sigmsg = "Bad system call";
+          break;
+	#endif
+
+        case SIGPIPE:
+          sigmsg = "Broken pipe";
+          break;
+
+        case SIGALRM:
+          sigmsg = "Alarm clock";
+          break;
+
+        case SIGTERM:
+          sigmsg = "Terminated";
+          break;
+
+        default:
+          sprintf(sigmsg_buf, "Signal %d", wsignal);
+          sigmsg = sigmsg_buf;
+          break;
+        }
+	simple_dialog(ESD_TYPE_WARN, NULL,
+		"Child capture process %s: %s%s", msg, sigmsg, coredumped);
+      }
+    }
+      
+    /* Read what remains of the capture file, and finish the capture.
+       XXX - do something if this fails? */
+    err = finish_tail_cap_file(cf);
+    return;
+  }
+
+  buffer[nread] = '\0';
+
+  while(!exit_loop) {
+    /* look for (possibly multiple) '*' */
+    switch (*q) {
+    case '*' :
+      to_read += atoi(p);
+      p = q + 1; 
+      q++;
+      break;
+    case '\0' :
+      /* XXX should handle the case of a pipe full (i.e. no star found) */
+      exit_loop = TRUE;
+      break;
+    default :
+      q++;
+      break;
+    } 
+  }
+
+  /* Read from the capture file the number of records the child told us
+     it added.
+     XXX - do something if this fails? */
+  err = continue_tail_cap_file(cf, to_read);
+
+  /* restore pipe handler */
+  cap_input_id = gtk_input_add_full (sync_pipe[0],
+				     GDK_INPUT_READ,
+				     cap_file_input_cb,
+				     NULL,
+				     (gpointer) cf,
+				     NULL);
+}
+
+/* Do the low-level work of a capture.
+   Returns TRUE if it succeeds, FALSE otherwise. */
+int
 capture(void)
 {
   GtkWidget  *cap_w, *main_vb, *count_lb, *tcp_lb, *udp_lb, *icmp_lb,
@@ -265,11 +502,12 @@ capture(void)
   pch = pcap_open_live(cf.iface, cf.snap, 1, 250, err_str);
 
   if (pch == NULL) {
-    /* Well, we couldn't start the capture. */
-    if (!sync_mode && !fork_mode) {
-      /* In fork mode, we shouldn't do any UI stuff until we pop up the
-         capture-progress window, and, since we couldn't start the
-	 capture, we haven't popped it up. */
+    /* Well, we couldn't start the capture.
+       If this is a child process that does the capturing in sync
+       mode or fork mode, it shouldn't do any UI stuff until we pop up the
+       capture-progress window, and, since we couldn't start the
+       capture, we haven't popped it up. */
+    if (!capture_child) {
       while (gtk_events_pending()) gtk_main_iteration();
     }
     snprintf(errmsg, sizeof errmsg,
@@ -337,14 +575,15 @@ capture(void)
     goto error;
   }
 
-  if (sync_mode) {
+  if (capture_child) {
     /* Well, we should be able to start capturing.
 
-       Sync out the capture file, so the header makes it to the file
-       system, and send a "capture started successfully and capture
-       file created" message to our parent so that they'll open the
-       capture file and update its windows to indicate that we have
-       a live capture in progress. */
+       This is the child process for a sync mode capture, so sync out
+       the capture file, so the header makes it to the file system,
+       and send a "capture started successfully and capture file created"
+       message to our parent so that they'll open the capture file and
+       update its windows to indicate that we have a live capture in
+       progress. */
     fflush(wtap_dump_file(ld.pdh));
     write(1, "0;", 2);
   }
@@ -447,7 +686,10 @@ capture(void)
 
       /* do sync here, too */
       fflush(wtap_dump_file(ld.pdh));
-      if (sync_mode && ld.sync_packets) {
+      if (capture_child && ld.sync_packets) {
+	/* This is the child process for a sync mode capture, so send
+	   our parent a message saying we've written out "ld.sync_packets"
+	   packets to the capture file. */
 	char tmp[20];
 	sprintf(tmp, "%d*", ld.sync_packets);
 	write(1, tmp, strlen(tmp));
@@ -478,17 +720,10 @@ capture(void)
       break;
 
     default:
-      if (err < 0) {
-        simple_dialog(ESD_TYPE_WARN, NULL,
-		"The file to which the capture was being"
-		" saved (\"%s\") could not be closed: Error %d.",
-		cf.save_file, err);
-      } else {
-        simple_dialog(ESD_TYPE_WARN, NULL,
+      simple_dialog(ESD_TYPE_WARN, NULL,
 		"The file to which the capture was being"
 		" saved (\"%s\") could not be closed: %s.",
-		cf.save_file, strerror(err));
-      }
+		cf.save_file, wtap_strerror(err));
       break;
     }
   }
@@ -497,26 +732,15 @@ capture(void)
   gtk_grab_remove(GTK_WIDGET(cap_w));
   gtk_widget_destroy(GTK_WIDGET(cap_w));
 
-  if (quit_after_cap) {
-    /* DON'T unlink the save file.  Presumably someone wants it. */
-    gtk_exit(0);
-  }
-
-  if ((err = open_cap_file(cf.save_file, &cf)) == 0) {
-    /* Set the read filter to NULL. */
-    cf.rfcode = NULL;
-    err = read_cap_file(&cf);
-    set_menu_sensitivity("/File/Save", TRUE);
-    set_menu_sensitivity("/File/Save As...", FALSE);
-  }
-  return;
+  return TRUE;
 
 error:
   /* We couldn't even start the capture, so get rid of the capture
      file. */
   unlink(cf.save_file); /* silently ignore error */
-  if (sync_mode) {
-    /* Send the error message to our parent, so they can display a
+  if (capture_child) {
+    /* This is the child process for a sync mode capture.
+       Send the error message to our parent, so they can display a
        dialog box containing it. */
     int msglen = strlen(errmsg);
     char lenbuf[10+1+1];
@@ -530,8 +754,7 @@ error:
   if (pch != NULL)
     pcap_close(pch);
 
-  if (quit_after_cap)
-    gtk_exit(0);
+  return FALSE;
 }
 
 static float
