@@ -1,7 +1,7 @@
 /* colors.c
  * Definitions for color structures and routines
  *
- * $Id: colors.c,v 1.8 1999/09/12 14:10:00 deniel Exp $
+ * $Id: colors.c,v 1.15 1999/11/10 19:58:32 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -46,14 +46,15 @@
 extern capture_file cf;
 
 static gboolean read_filters(capture_file *cf);
+static GtkWidget* create_color_sel_win(cap_file  *cf, GdkColor *);
 
 GdkColor 	proto_colors[MAXCOLORS];
 GdkColormap*	sys_cmap;
+GdkColormap*	our_cmap = NULL;
 
 static gchar *titles[2] = { "Name", "Filter String" };
-GdkColor	color_light_gray = { 0, 45000, 45000, 45000 };
-GdkColor	WHITE = { 0,65535, 65535, 65535};
-GdkColor	BLACK = { 0, 0, 0, 0};
+GdkColor	WHITE = { 0, 65535, 65535, 65535 };
+GdkColor	BLACK = { 0, 0, 0, 0 };
 
 /* This structure is used to allow you to compile in default colors if
  * you wish.  They can be later changed by a user.
@@ -72,11 +73,12 @@ struct _default_colors {
 void
 colors_init(capture_file *cf)
 {
-
+  gboolean got_white, got_black;
 #ifdef READ_DEFAULT_COLOR_LIST
   gint i;
   GdkColor color;
 #endif
+
   cf->colors = (colfilter *)g_malloc(sizeof(colfilter));
   cf->colors->num_of_filters = 0;
   cf->colors->color_filters = gtk_clist_new_with_titles(2, titles);
@@ -86,14 +88,18 @@ colors_init(capture_file *cf)
   sys_cmap = gdk_colormap_get_system();
 
   /* Allocate "constant" colors. */
-  if( !gdk_colormap_alloc_color(sys_cmap, &WHITE, TRUE, TRUE)){
-	/* oops */
-	simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color white.");
-  }
+  got_white = get_color(&WHITE);
+  got_black = get_color(&BLACK);
 
-  if( !gdk_colormap_alloc_color(sys_cmap, &BLACK, TRUE, TRUE)){
-	/* oops */
-	simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color black.");
+  /* Got milk? */
+  if (!got_white) {
+    if (!got_black)
+      simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate colors black or white.");
+    else
+      simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color white.");
+  } else {
+    if (!got_black)
+      simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color black.");
   }
 
 #ifdef READ_DEFAULT_COLOR_LIST
@@ -101,7 +107,7 @@ colors_init(capture_file *cf)
   for (i = 0 ; i < sizeof default_colors/sizeof (struct _default_colors); i++){
 	gdk_color_parse(default_colors[i].color, &color);
 	
-	if( !gdk_colormap_alloc_color(sys_cmap, &color, TRUE, TRUE)){
+	if( !get_color(&color)){
 		/* oops */
 		simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color %s.",
 		    default_colors[i].color);
@@ -110,11 +116,10 @@ colors_init(capture_file *cf)
 	new_color_filter(cf->colors, default_colors[i].proto, default_colors[i].proto);
 	color_filter(cf,i)->bg_color = color;
 
-	color_filter(cf,i)->c_colorfilter = dfilter_new();
-	if(dfilter_compile((color_filter(cf,i)->c_colorfilter),
-	  default_colors[i].proto) != 0){
+	if (dfilter_compile(default_colors[i].proto,
+	    &color_filter(cf,i)->c_colorfilter) != 0) {
 		simple_dialog(ESD_TYPE_WARN, NULL,
-		  "Cannot compile default filter %s.\n%s",
+		  "Cannot compile default color filter %s.\n%s",
 		  default_colors[i].proto, dfilter_error_msg);
 		/* should reject this filter */
 	}
@@ -189,6 +194,7 @@ read_filters(capture_file *cf)
 {
 	/* TODO: Lots more syntax checking on the file */
 	/* I hate these fixed length names! TODO: make more dynamic */
+	/* XXX - buffer overflow possibility here */
 	gchar name[256],filter[256], buf[1024];
 	guint16 fg_r, fg_g, fg_b, bg_r, bg_g, bg_b;
 	GdkColor fg_color, bg_color;
@@ -197,6 +203,8 @@ read_filters(capture_file *cf)
 	FILE *f;
 	gchar *path;
 	gchar *fname = PF_DIR "/colorfilters";
+	dfilter *temp_dfilter;
+
 	/* decide what file to open (from dfilter code) */
 
 	/* should only be called by colors_init.
@@ -218,6 +226,7 @@ read_filters(capture_file *cf)
 	  g_free(path);
 	  return FALSE;
 	}
+	g_free(path);
 
 	i = 0;
 
@@ -235,13 +244,14 @@ read_filters(capture_file *cf)
 		name, filter, &bg_r, &bg_g, &bg_b, &fg_r, &fg_g, &fg_b) == 8){
 		/* we got a filter */
 
-            new_color_filter(cf->colors, name, filter);
-	    color_filter(cf,i)->c_colorfilter = dfilter_new();
-	    if(dfilter_compile((color_filter(cf,i)->c_colorfilter),filter) != 0){
+	    if(dfilter_compile(filter, &temp_dfilter) != 0){
 		simple_dialog(ESD_TYPE_WARN, NULL,
-		 "Could not compile filter %s from saved filters because\n%s",
+		 "Could not compile color filter %s from saved filters.\n%s",
 		 name, dfilter_error_msg);
+		continue;
 	    }
+            new_color_filter(cf->colors, name, filter);
+	    color_filter(cf,i)->c_colorfilter = temp_dfilter;
 	    cf->colors->num_of_filters++;
 	    fg_color.red = fg_r;
 	    fg_color.green = fg_g;
@@ -249,32 +259,32 @@ read_filters(capture_file *cf)
 	    bg_color.red = bg_r;
 	    bg_color.green = bg_g;
 	    bg_color.blue = bg_b;
-	    if( !gdk_colormap_alloc_color(sys_cmap, &fg_color, TRUE, TRUE)){
-			/* oops */
-			simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate fg color specified"
-			    "in input file for %s.", name);
+	    if( !get_color(&fg_color)){
+		/* oops */
+		simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate fg color specified"
+		  "in input file for %s.", name);
 
-			i++;
-			continue;
+		i++;
+		continue;
 	    }
-	    	if( !gdk_colormap_alloc_color(sys_cmap, &bg_color, TRUE, TRUE)){
-			/* oops */
-			simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate bg color specified"
-			    "in input file for %s.", name);
-			i++;
-			continue;
+	    if( !get_color(&bg_color)){
+		/* oops */
+		simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate bg color specified"
+		  "in input file for %s.", name);
+		i++;
+		continue;
 	    }
 
-        color_filter(cf,i)->bg_color = bg_color;
-        color_filter(cf,i)->fg_color = fg_color;
-        gtk_clist_set_foreground(GTK_CLIST(cf->colors->color_filters),
+            color_filter(cf,i)->bg_color = bg_color;
+            color_filter(cf,i)->fg_color = fg_color;
+            gtk_clist_set_foreground(GTK_CLIST(cf->colors->color_filters),
 			i,&fg_color);
-        gtk_clist_set_background(GTK_CLIST(cf->colors->color_filters),
+            gtk_clist_set_background(GTK_CLIST(cf->colors->color_filters),
 			i,&bg_color);
 
 	    i++;
 	  }    /* if sscanf */
-	}   while( !feof(f));
+	} while( !feof(f));
 	return TRUE;
 }
 
@@ -479,7 +489,7 @@ color_ok_cb                            (GtkButton       *button,
 
   /* colorize list */
 
-  filter_packets(&cf);
+  colorize_packets(&cf);
   gtk_container_remove(GTK_CONTAINER(GTK_WIDGET(cf.colors->color_filters)->parent),
 	cf.colors->color_filters);
   gtk_widget_destroy(dialog);
@@ -493,7 +503,7 @@ color_apply_cb                         (GtkButton       *button,
 {
   capture_file *cf;
   cf = (capture_file *)user_data;
-  filter_packets(cf);
+  colorize_packets(cf);
 }
 
 /* Exit dialog and do not process list */
@@ -515,8 +525,11 @@ colorize_fg_cb                         (GtkButton       *button,
                                         gpointer         user_data)
 {
   colfilter *filter;
+  color_filter_t * cof;
+
   filter = (colfilter *)user_data; /* UNUSED */
-  create_color_sel_win(&cf);
+  cof = color_filter(&cf, filter->row_selected);
+  create_color_sel_win(&cf, &cof->fg_color);
   bg_set_flag = 0;
 
 }
@@ -527,8 +540,11 @@ colorize_bg_cb                         (GtkButton       *button,
                                         gpointer         user_data)
 {
   colfilter *filter;
+  color_filter_t * cof;
+
   filter = (colfilter *)user_data; /* UNUSED */
-  create_color_sel_win(&cf);
+  cof = color_filter(&cf, filter->row_selected);
+  create_color_sel_win(&cf, &cof->bg_color);
   bg_set_flag = 1;
 
 }
@@ -573,12 +589,9 @@ colorize_ok_cb                         (GtkButton       *button,
 
 
 
-  compiled_filter = dfilter_new();
-  
-  if( dfilter_compile( compiled_filter, filter_text) != 0 ){
+  if(dfilter_compile(filter_text, &compiled_filter) != 0 ){
 	simple_dialog(ESD_TYPE_WARN, NULL, "Filter \"%s\" did not compile correctly.\n"
 		" Please try again. Filter unchanged.\n%s\n", filter_name,dfilter_error_msg);
-	dfilter_destroy(compiled_filter);
   } else {
 
 	if( color_filter(&cf, cf.colors->row_selected)->c_colorfilter != NULL)
@@ -625,7 +638,7 @@ color_sel_ok_cb                        (GtkButton       *button,
   new_color.green = (guint16)(new_colors[1]*65535.0);
   new_color.blue  = (guint16)(new_colors[2]*65535.0);
 
-  if ( ! gdk_colormap_alloc_color(sys_cmap, &new_color, TRUE, TRUE) ){
+  if ( ! get_color(&new_color) ){
 	simple_dialog(ESD_TYPE_WARN, NULL, "Could not allocate color.  Try again.");
   } else {
 	gtk_widget_destroy(color_dialog);
@@ -1038,8 +1051,8 @@ create_colorize_win (capture_file *cf,
   return colorize_win;
 }
 
-GtkWidget*
-create_color_sel_win (capture_file *cf)
+static GtkWidget*
+create_color_sel_win (capture_file *cf, GdkColor * color)
 {
   GtkWidget *color_sel_win;
   GtkWidget *color_sel_ok;
@@ -1050,6 +1063,18 @@ create_color_sel_win (capture_file *cf)
   gtk_object_set_data (GTK_OBJECT (color_sel_win), "color_sel_win", color_sel_win);
   gtk_container_set_border_width (GTK_CONTAINER (color_sel_win), 10);
 
+  if (color != NULL) {
+    gdouble cols[3];
+
+    cols[0] = (gdouble)color->red / 65536.0;
+    cols[1] = (gdouble)color->green / 65536.0;
+    cols[2] = (gdouble)color->blue / 65536.0;
+
+    gtk_color_selection_set_color(
+		    GTK_COLOR_SELECTION(
+			    GTK_COLOR_SELECTION_DIALOG(color_sel_win)->colorsel), cols);
+  }
+
   color_sel_ok = GTK_COLOR_SELECTION_DIALOG (color_sel_win)->ok_button;
   gtk_object_set_data (GTK_OBJECT (color_sel_win), "color_sel_ok", color_sel_ok);
   gtk_widget_show (color_sel_ok);
@@ -1084,3 +1109,18 @@ create_color_sel_win (capture_file *cf)
   return color_sel_win;
 }
 
+gboolean
+get_color ( GdkColor *new_color) {
+
+    GdkVisual *pv;
+
+    if (!our_cmap) {
+	if ( !gdk_colormap_alloc_color (sys_cmap, new_color, FALSE, TRUE)) {
+	    pv = gdk_visual_get_best();
+	    if ( !(our_cmap = gdk_colormap_new(pv, TRUE)))
+		simple_dialog(ESD_TYPE_WARN, NULL, "Could not create new colormap");
+	} else
+	    return (TRUE);
+    }
+    return ( gdk_colormap_alloc_color ( our_cmap, new_color, FALSE, TRUE) );
+}
