@@ -1,7 +1,7 @@
 /* file.c
  * File I/O routines
  *
- * $Id: file.c,v 1.98 1999/09/24 05:22:28 gram Exp $
+ * $Id: file.c,v 1.125 1999/11/29 01:54:00 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -46,6 +46,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <signal.h>
 
 #ifdef NEED_SNPRINTF_H
 # ifdef HAVE_STDARG_H
@@ -78,14 +79,24 @@
 #include "gtk/proto_draw.h"
 #include "dfilter.h"
 #include "timestamp.h"
+#include "conversation.h"
+#include "globals.h"
 
-#include "packet-ncp.h"
+#ifndef __RESOLV_H__
+#include "resolv.h"
+#endif
+
+#include "packet-atalk.h"
+
+#include "packet-ipv6.h"
+
+#include "packet-sna.h"
+
+#include "packet-vines.h"
 
 extern GtkWidget *packet_list, *prog_bar, *info_bar, *byte_view, *tree_view;
 extern guint      file_ctx;
-extern int        sync_pipe[];
 
-guint cap_input_id;
 gboolean auto_scroll_live = FALSE;
 
 static guint32 firstsec, firstusec;
@@ -125,8 +136,11 @@ open_cap_file(char *fname, capture_file *cf) {
      and fill in the information for this file. */
   close_cap_file(cf, info_bar, file_ctx);
 
+  /* Initialize the table of conversations. */
+  conversation_init();
+
   /* Initialize protocol-specific variables */
-  ncp_init_protocol();
+  init_all_protocols();
 
   cf->wth = wth;
   cf->fh = fh;
@@ -194,13 +208,48 @@ close_cap_file(capture_file *cf, void *w, guint context) {
   set_menu_sensitivity("/File/Reload", FALSE);
   set_menu_sensitivity("/File/Print...", FALSE);
   set_menu_sensitivity("/Display/Options...", FALSE);
+  set_menu_sensitivity("/Display/Match Selected", FALSE);
+  set_menu_sensitivity("/Display/Colorize Display...", FALSE);
+  set_menu_sensitivity("/Display/Find Frame...", FALSE);
+  set_menu_sensitivity("/Display/Go To Frame...", FALSE);
+  set_menu_sensitivity("/Display/Collapse All", FALSE);
+  set_menu_sensitivity("/Display/Expand All", FALSE);
+  set_menu_sensitivity("/Tools/Follow TCP Stream", FALSE);
+  set_menu_sensitivity("/Tools/Graph", FALSE);
   set_menu_sensitivity("/Tools/Summary", FALSE);
 }
 
+static void
+set_statusbar_filename(capture_file *cf)
+{
+  gchar  *name_ptr;
+  size_t  msg_len;
+  gchar  *done_fmt = " File: %s  Drops: %u";
+  gchar  *done_msg;
+
+  if (cf->user_saved || !cf->save_file) {
+    /* Get the last component of the file name, and put that in the
+       status bar. */
+    if ((name_ptr = (gchar *) strrchr(cf->filename, '/')) == NULL)
+      name_ptr = cf->filename;
+    else
+      name_ptr++;
+  } else {
+    /* The file we read is a temporary file from a live capture;
+       we don't mention its name in the status bar. */
+    name_ptr = "<none>";
+  }
+
+  msg_len = strlen(name_ptr) + strlen(done_fmt) + 64;
+  done_msg = g_malloc(msg_len);
+  snprintf(done_msg, msg_len, done_fmt, name_ptr, cf->drops);
+  gtk_statusbar_push(GTK_STATUSBAR(info_bar), file_ctx, done_msg);
+  g_free(done_msg);
+}
+
 int
 read_cap_file(capture_file *cf) {
   gchar  *name_ptr, *load_msg, *load_fmt = " Loading: %s...";
-  gchar  *done_fmt = " File: %s  Drops: %d";
   int     success;
   int     err;
   size_t  msg_len;
@@ -213,9 +262,11 @@ read_cap_file(capture_file *cf) {
   else
     name_ptr++;
 
-  load_msg = g_malloc(strlen(name_ptr) + strlen(load_fmt) + 2);
-  sprintf(load_msg, load_fmt, name_ptr);
+  msg_len = strlen(name_ptr) + strlen(load_fmt) + 2;
+  load_msg = g_malloc(msg_len);
+  snprintf(load_msg, msg_len, load_fmt, name_ptr);
   gtk_statusbar_push(GTK_STATUSBAR(info_bar), file_ctx, load_msg);
+  g_free(load_msg);
 
   cf->update_progbar = TRUE;
   /* Update the progress bar when it gets to this value. */
@@ -232,6 +283,9 @@ read_cap_file(capture_file *cf) {
   cf->filed = open(cf->filename, O_RDONLY);
   cf->fh = filed_open(cf->filed, "r");
   cf->unfiltered_count = cf->count;
+  cf->current_frame = cf->first_displayed;
+  /* Make the first row the selected row. */
+  gtk_clist_select_row(GTK_CLIST(packet_list), 0, -1);
   thaw_clist(cf);
 
   gtk_progress_set_activity_mode(GTK_PROGRESS(prog_bar), FALSE);
@@ -239,22 +293,19 @@ read_cap_file(capture_file *cf) {
 
   gtk_statusbar_pop(GTK_STATUSBAR(info_bar), file_ctx);
 
-  msg_len = strlen(name_ptr) + strlen(done_fmt) + 64;
-  load_msg = g_realloc(load_msg, msg_len);
-
-  if (cf->user_saved || !cf->save_file)
-    snprintf(load_msg, msg_len, done_fmt, name_ptr, cf->drops);
-  else
-    snprintf(load_msg, msg_len, done_fmt, "<none>", cf->drops);
-
-  gtk_statusbar_push(GTK_STATUSBAR(info_bar), file_ctx, load_msg);
-  g_free(load_msg);
+  set_statusbar_filename(cf);
 
   /* Enable menu items that make sense if you have a capture. */
   set_menu_sensitivity("/File/Close", TRUE);
   set_menu_sensitivity("/File/Reload", TRUE);
   set_menu_sensitivity("/File/Print...", TRUE);
   set_menu_sensitivity("/Display/Options...", TRUE);
+  set_menu_sensitivity("/Display/Match Selected", TRUE);
+  set_menu_sensitivity("/Display/Colorize Display...", TRUE);
+  set_menu_sensitivity("/Display/Find Frame...", TRUE);
+  set_menu_sensitivity("/Display/Go To Frame...", TRUE);
+  set_menu_sensitivity("/Tools/Follow TCP Stream", TRUE);
+  set_menu_sensitivity("/Tools/Graph", TRUE);
   set_menu_sensitivity("/Tools/Summary", TRUE);
 
   if (!success) {
@@ -291,93 +342,22 @@ read_cap_file(capture_file *cf) {
 }
 
 #ifdef HAVE_LIBPCAP
-void 
-cap_file_input_cb (gpointer data, gint source, GdkInputCondition condition) {
-  
-  capture_file *cf = (capture_file *)data;
-  char buffer[256+1], *p = buffer, *q = buffer;
-  int  nread;
-  int  to_read = 0;
-  gboolean exit_loop = FALSE;
-  int  err;
-
-  /* avoid reentrancy problems and stack overflow */
-  gtk_input_remove(cap_input_id);
-
-  if ((nread = read(sync_pipe[0], buffer, 256)) <= 0) {
-
-    /* The child has closed the sync pipe, meaning it's not going to be
-       capturing any more packets.  Read what remains of the capture file,
-       and stop capture (restore menu items) */
-    gtk_clist_freeze(GTK_CLIST(packet_list));
-
-    /* XXX - do something if this fails? */
-    wtap_loop(cf->wth, 0, wtap_dispatch_cb, (u_char *) cf, &err);
-
-    thaw_clist(cf);
-    if (auto_scroll_live)
-      gtk_clist_moveto(GTK_CLIST(packet_list), 
-		       cf->plist_end->row, -1, 1.0, 1.0);
-
-    wtap_close(cf->wth);
-    cf->wth = NULL;
-    set_menu_sensitivity("/File/Open...", TRUE);
-    set_menu_sensitivity("/File/Close", TRUE);
-    set_menu_sensitivity("/File/Save As...", TRUE);
-    set_menu_sensitivity("/File/Print...", TRUE);
-    set_menu_sensitivity("/File/Reload", TRUE);
-    set_menu_sensitivity("/Capture/Start...", TRUE);
-    set_menu_sensitivity("/Tools/Summary", TRUE);
-    gtk_statusbar_push(GTK_STATUSBAR(info_bar), file_ctx, " File: <none>");
-    return;
-  }
-
-  buffer[nread] = '\0';
-
-  while(!exit_loop) {
-    /* look for (possibly multiple) '*' */
-    switch (*q) {
-    case '*' :
-      to_read += atoi(p);
-      p = q + 1; 
-      q++;
-      break;
-    case '\0' :
-      /* XXX should handle the case of a pipe full (i.e. no star found) */
-      exit_loop = TRUE;
-      break;
-    default :
-      q++;
-      break;
-    } 
-  }
-
-  gtk_clist_freeze(GTK_CLIST(packet_list));
-  /* XXX - do something if this fails? */
-  wtap_loop(cf->wth, to_read, wtap_dispatch_cb, (u_char *) cf, &err);
-  gtk_clist_thaw(GTK_CLIST(packet_list));
-  if (auto_scroll_live)
-    gtk_clist_moveto(GTK_CLIST(packet_list), cf->plist_end->row, -1, 1.0, 1.0);
-
-  /* restore pipe handler */
-  cap_input_id = gtk_input_add_full (sync_pipe[0],
-				     GDK_INPUT_READ,
-				     cap_file_input_cb,
-				     NULL,
-				     (gpointer) cf,
-				     NULL);
-}
-
 int
-tail_cap_file(char *fname, capture_file *cf) {
+start_tail_cap_file(char *fname, capture_file *cf) {
   int     err;
   int     i;
 
   err = open_cap_file(fname, cf);
-  if ((err == 0) && (cf->cd_t != WTAP_FILE_UNKNOWN)) {
-
+  if (err == 0) {
     set_menu_sensitivity("/File/Open...", FALSE);
     set_menu_sensitivity("/Display/Options...", TRUE);
+    set_menu_sensitivity("/Display/Match Selected", TRUE);
+    set_menu_sensitivity("/Display/Colorize Display...", TRUE);
+    set_menu_sensitivity("/Display/Find Frame...", TRUE);
+    set_menu_sensitivity("/Display/Go To Frame...", TRUE);
+    set_menu_sensitivity("/Tools/Follow TCP Stream", TRUE);
+    set_menu_sensitivity("/Tools/Graph", TRUE);
+    set_menu_sensitivity("/Tools/Summary", TRUE);
     set_menu_sensitivity("/Capture/Start...", FALSE);
 
     for (i = 0; i < cf->cinfo.num_cols; i++) {
@@ -391,20 +371,68 @@ tail_cap_file(char *fname, capture_file *cf) {
       }
     }
 
+    /* Yes, "open_cap_file()" set this - but it set it to a file handle
+       from Wiretap, which will be closed when we close the file; we
+       want it to remain open even after that, so that we can read
+       packet data from it. */
     cf->fh = file_open(fname, "r");
 
-    cap_input_id = gtk_input_add_full (sync_pipe[0],
-				       GDK_INPUT_READ,
-				       cap_file_input_cb,
-				       NULL,
-				       (gpointer) cf,
-				       NULL);
     gtk_statusbar_push(GTK_STATUSBAR(info_bar), file_ctx, 
 		       " <live capture in progress>");
   }
-  else {
-    close(sync_pipe[0]);
-  }
+  return err;
+}
+
+int
+continue_tail_cap_file(capture_file *cf, int to_read)
+{
+  int err;
+
+  gtk_clist_freeze(GTK_CLIST(packet_list));
+
+  wtap_loop(cf->wth, to_read, wtap_dispatch_cb, (u_char *) cf, &err);
+
+  gtk_clist_thaw(GTK_CLIST(packet_list));
+  if (auto_scroll_live)
+    gtk_clist_moveto(GTK_CLIST(packet_list), 
+		       cf->plist_end->row, -1, 1.0, 1.0);
+  return err;
+}
+
+int
+finish_tail_cap_file(capture_file *cf)
+{
+  int err;
+
+  gtk_clist_freeze(GTK_CLIST(packet_list));
+
+  wtap_loop(cf->wth, 0, wtap_dispatch_cb, (u_char *) cf, &err);
+
+  thaw_clist(cf);
+  if (auto_scroll_live)
+    gtk_clist_moveto(GTK_CLIST(packet_list), 
+		       cf->plist_end->row, -1, 1.0, 1.0);
+
+  /* There's nothing more to read from the capture file - close it. */
+  wtap_close(cf->wth);
+  cf->wth = NULL;
+
+  set_statusbar_filename(cf);
+
+  /* Restore the "File/Open" menu item. */
+  set_menu_sensitivity("/File/Open...", TRUE);
+
+  /* Enable menu items that make sense if you have a capture file
+     you've finished reading. */
+  set_menu_sensitivity("/File/Save", TRUE);
+  set_menu_sensitivity("/File/Close", TRUE);
+  set_menu_sensitivity("/File/Reload", TRUE);
+  set_menu_sensitivity("/File/Print...", TRUE);
+
+  /* Enable menu items that make sense if you're not currently running
+     a capture. */
+  set_menu_sensitivity("/Capture/Start...", TRUE);
+
   return err;
 }
 #endif /* HAVE_LIBPCAP */
@@ -412,14 +440,14 @@ tail_cap_file(char *fname, capture_file *cf) {
 /* To do: Add check_col checks to the col_add* routines */
 
 static void
-col_add_abs_time(frame_data *fd, gint el)
+col_set_abs_time(frame_data *fd, int col)
 {
   struct tm *tmp;
   time_t then;
 
   then = fd->abs_secs;
   tmp = localtime(&then);
-  col_add_fstr(fd, el, "%02d:%02d:%02d.%04ld",
+  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%02d:%02d:%02d.%04ld",
     tmp->tm_hour,
     tmp->tm_min,
     tmp->tm_sec,
@@ -427,54 +455,251 @@ col_add_abs_time(frame_data *fd, gint el)
 }
 
 static void
-col_add_rel_time(frame_data *fd, gint el)
+col_set_rel_time(frame_data *fd, int col)
 {
-  col_add_fstr(fd, el, "%d.%06d", fd->rel_secs, fd->rel_usecs);
+  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%d.%06d", fd->rel_secs,
+    fd->rel_usecs);
 }
 
 static void
-col_add_delta_time(frame_data *fd, gint el)
+col_set_delta_time(frame_data *fd, int col)
 {
-  col_add_fstr(fd, el, "%d.%06d", fd->del_secs, fd->del_usecs);
+  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%d.%06d", fd->del_secs,
+    fd->del_usecs);
 }
 
 /* Add "command-line-specified" time. */
 static void
-col_add_cls_time(frame_data *fd)
+col_set_cls_time(frame_data *fd, int col)
 {
   switch (timestamp_type) {
     case ABSOLUTE:
-      col_add_abs_time(fd, COL_CLS_TIME);
+      col_set_abs_time(fd, col);
       break;
 
     case RELATIVE:
-      col_add_rel_time(fd, COL_CLS_TIME);
+      col_set_rel_time(fd, col);
       break;
 
     case DELTA:
-      col_add_delta_time(fd, COL_CLS_TIME);
+      col_set_delta_time(fd, col);
       break;
   }
 }
 
+static void
+col_set_addr(frame_data *fd, int col, address *addr, gboolean is_res)
+{
+  u_int ipv4_addr;
+  struct e_in6_addr ipv6_addr;
+  struct atalk_ddp_addr ddp_addr;
+  struct sna_fid_type_4_addr sna_fid_type_4_addr;
+
+  switch (addr->type) {
+
+  case AT_ETHER:
+    if (is_res)
+      strncpy(fd->cinfo->col_data[col], get_ether_name(addr->data), COL_MAX_LEN);
+    else
+      strncpy(fd->cinfo->col_data[col], ether_to_str(addr->data), COL_MAX_LEN);
+    break;
+
+  case AT_IPv4:
+    memcpy(&ipv4_addr, addr->data, sizeof ipv4_addr);
+    if (is_res)
+      strncpy(fd->cinfo->col_data[col], get_hostname(ipv4_addr), COL_MAX_LEN);
+    else
+      strncpy(fd->cinfo->col_data[col], ip_to_str(addr->data), COL_MAX_LEN);
+    break;
+
+  case AT_IPv6:
+    memcpy(&ipv6_addr.s6_addr, addr->data, sizeof ipv6_addr.s6_addr);
+    if (is_res)
+      strncpy(fd->cinfo->col_data[col], get_hostname6(&ipv6_addr), COL_MAX_LEN);
+    else
+      strncpy(fd->cinfo->col_data[col], ip6_to_str(&ipv6_addr), COL_MAX_LEN);
+    break;
+
+  case AT_IPX:
+    strncpy(fd->cinfo->col_data[col],
+      ipx_addr_to_str(pntohl(&addr->data[0]), &addr->data[4]), COL_MAX_LEN);
+    break;
+
+  case AT_SNA:
+    switch (addr->len) {
+
+    case 1:
+      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%04X", addr->data[0]);
+      break;
+
+    case 2:
+      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%04X",
+        pntohs(&addr->data[0]));
+      break;
+
+    case SNA_FID_TYPE_4_ADDR_LEN:
+      memcpy(&sna_fid_type_4_addr, addr->data, SNA_FID_TYPE_4_ADDR_LEN);
+      strncpy(fd->cinfo->col_data[col],
+        sna_fid_type_4_addr_to_str(&sna_fid_type_4_addr), COL_MAX_LEN);
+      break;
+    }
+    break;
+
+  case AT_ATALK:
+    memcpy(&ddp_addr, addr->data, sizeof ddp_addr);
+    strncpy(fd->cinfo->col_data[col], atalk_addr_to_str(&ddp_addr),
+      COL_MAX_LEN);
+    break;
+
+  case AT_VINES:
+    strncpy(fd->cinfo->col_data[col], vines_addr_to_str(&addr->data[0]),
+      COL_MAX_LEN);
+    break;
+
+  default:
+    break;
+  }
+  fd->cinfo->col_data[col][COL_MAX_LEN - 1] = '\0';
+}
+
+static void
+col_set_port(frame_data *fd, int col, port_type ptype, guint32 port,
+		gboolean is_res)
+{
+  switch (ptype) {
+
+  case PT_TCP:
+    if (is_res)
+      strncpy(fd->cinfo->col_data[col], get_tcp_port(port), COL_MAX_LEN);
+    else
+      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%u", port);
+    break;
+
+  case PT_UDP:
+    if (is_res)
+      strncpy(fd->cinfo->col_data[col], get_udp_port(port), COL_MAX_LEN);
+    else
+      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%u", port);
+    break;
+
+  default:
+    break;
+  }
+  fd->cinfo->col_data[col][COL_MAX_LEN - 1] = '\0';
+}
+
 static void
 fill_in_columns(frame_data *fd)
 {
-  if (check_col(fd, COL_NUMBER))
-    col_add_fstr(fd, COL_NUMBER, "%u", fd->num);
+  int i;
 
-  /* Set any time stamp columns. */
-  if (check_col(fd, COL_CLS_TIME))
-    col_add_cls_time(fd);
-  if (check_col(fd, COL_ABS_TIME))
-    col_add_abs_time(fd, COL_ABS_TIME);
-  if (check_col(fd, COL_REL_TIME))
-    col_add_rel_time(fd, COL_REL_TIME);
-  if (check_col(fd, COL_DELTA_TIME))
-    col_add_delta_time(fd, COL_DELTA_TIME);
+  for (i = 0; i < fd->cinfo->num_cols; i++) {
+    switch (fd->cinfo->col_fmt[i]) {
 
-  if (check_col(fd, COL_PACKET_LENGTH))
-    col_add_fstr(fd, COL_PACKET_LENGTH, "%d", fd->pkt_len);
+    case COL_NUMBER:
+      snprintf(fd->cinfo->col_data[i], COL_MAX_LEN, "%u", fd->num);
+      break;
+
+    case COL_CLS_TIME:
+      col_set_cls_time(fd, i);
+      break;
+
+    case COL_ABS_TIME:
+      col_set_abs_time(fd, i);
+      break;
+
+    case COL_REL_TIME:
+      col_set_rel_time(fd, i);
+      break;
+
+    case COL_DELTA_TIME:
+      col_set_delta_time(fd, i);
+      break;
+
+    case COL_DEF_SRC:
+    case COL_RES_SRC:	/* COL_DEF_SRC is currently just like COL_RES_SRC */
+      col_set_addr(fd, i, &pi.src, TRUE);
+      break;
+
+    case COL_UNRES_SRC:
+      col_set_addr(fd, i, &pi.src, FALSE);
+      break;
+
+    case COL_DEF_DL_SRC:
+    case COL_RES_DL_SRC:
+      col_set_addr(fd, i, &pi.dl_src, TRUE);
+      break;
+
+    case COL_UNRES_DL_SRC:
+      col_set_addr(fd, i, &pi.dl_src, FALSE);
+      break;
+
+    case COL_DEF_NET_SRC:
+    case COL_RES_NET_SRC:
+      col_set_addr(fd, i, &pi.net_src, TRUE);
+      break;
+
+    case COL_UNRES_NET_SRC:
+      col_set_addr(fd, i, &pi.net_src, FALSE);
+      break;
+
+    case COL_DEF_DST:
+    case COL_RES_DST:	/* COL_DEF_DST is currently just like COL_RES_DST */
+      col_set_addr(fd, i, &pi.dst, TRUE);
+      break;
+
+    case COL_UNRES_DST:
+      col_set_addr(fd, i, &pi.dst, FALSE);
+      break;
+
+    case COL_DEF_DL_DST:
+    case COL_RES_DL_DST:
+      col_set_addr(fd, i, &pi.dl_dst, TRUE);
+      break;
+
+    case COL_UNRES_DL_DST:
+      col_set_addr(fd, i, &pi.dl_dst, FALSE);
+      break;
+
+    case COL_DEF_NET_DST:
+    case COL_RES_NET_DST:
+      col_set_addr(fd, i, &pi.net_dst, TRUE);
+      break;
+
+    case COL_UNRES_NET_DST:
+      col_set_addr(fd, i, &pi.net_dst, FALSE);
+      break;
+
+    case COL_DEF_SRC_PORT:
+    case COL_RES_SRC_PORT:	/* COL_DEF_SRC_PORT is currently just like COL_RES_SRC_PORT */
+      col_set_port(fd, i, pi.ptype, pi.srcport, TRUE);
+      break;
+
+    case COL_UNRES_SRC_PORT:
+      col_set_port(fd, i, pi.ptype, pi.srcport, FALSE);
+      break;
+
+    case COL_DEF_DST_PORT:
+    case COL_RES_DST_PORT:	/* COL_DEF_DST_PORT is currently just like COL_RES_DST_PORT */
+      col_set_port(fd, i, pi.ptype, pi.destport, TRUE);
+      break;
+
+    case COL_UNRES_DST_PORT:
+      col_set_port(fd, i, pi.ptype, pi.destport, FALSE);
+      break;
+
+    case COL_PROTOCOL:	/* currently done by dissectors */
+    case COL_INFO:	/* currently done by dissectors */
+      break;
+
+    case COL_PACKET_LENGTH:
+      snprintf(fd->cinfo->col_data[i], COL_MAX_LEN, "%d", fd->pkt_len);
+      break;
+
+    case NUM_COL_FMTS:	/* keep compiler happy - shouldn't get here */
+      break;
+    }
+  }
 }
 
 static void
@@ -510,11 +735,10 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf, const u_char *buf
   }
 
   /* Apply the filters */
-  if (DFILTER_CONTAINS_FILTER(cf->dfcode) ||
-      CFILTERS_CONTAINS_FILTER(cf)) {
+  if (cf->dfcode != NULL || CFILTERS_CONTAINS_FILTER(cf)) {
 	protocol_tree = proto_tree_create_root();
 	dissect_packet(buf, fdata, protocol_tree);
-	if( DFILTER_CONTAINS_FILTER(cf->dfcode) )
+	if (cf->dfcode != NULL)
 		fdata->passed_dfilter = dfilter_apply(cf->dfcode, protocol_tree, cf->pd);
 	else
 		fdata->passed_dfilter = TRUE;
@@ -522,6 +746,10 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf, const u_char *buf
 	color = -1;
         for(crow = 0; cf->colors->num_of_filters && 
 	      crow < cf->colors->num_of_filters; crow++) {
+
+            if(color_filter(cf,crow)->c_colorfilter == NULL) {
+		continue;
+	    }
             if(dfilter_apply(color_filter(cf,crow)->c_colorfilter, protocol_tree,
 		 cf->pd)){
                 color = crow;
@@ -577,11 +805,17 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf, const u_char *buf
         gtk_clist_set_foreground(GTK_CLIST(packet_list), row, &BLACK);
     }
 
-    /* If this was the selected packet, remember the row it's in, so
-       we can re-select it.  ("selected_packet" is 0-origin, as it's
-       a GList index; "num", however, is 1-origin.) */
-    if (cf->selected_packet == fdata->num - 1)
-      cf->selected_row = row;
+    /* If we haven't yet seen the first frame, this is it. */
+    if (cf->first_displayed == NULL)
+      cf->first_displayed = fdata;
+
+    /* This is the last frame we've seen so far. */
+    cf->last_displayed = fdata;
+
+    /* If this was the current frame, remember the row it's in, so
+       we can arrange that it's on the screen when we're done. */
+    if (cf->current_frame == fdata)
+      cf->current_row = row;
   } else
     fdata->row = -1;	/* not in the display */
   fdata->cinfo = NULL;
@@ -622,26 +856,27 @@ wtap_dispatch_cb(u_char *user, const struct wtap_pkthdr *phdr, int offset,
   fdata = (frame_data *) g_malloc(sizeof(frame_data));
 
   fdata->next = NULL;
+  fdata->prev = NULL;
   fdata->pkt_len  = phdr->len;
   fdata->cap_len  = phdr->caplen;
   fdata->file_off = offset;
   fdata->lnk_t = phdr->pkt_encap;
   fdata->abs_secs  = phdr->ts.tv_sec;
   fdata->abs_usecs = phdr->ts.tv_usec;
+  fdata->encoding = CHAR_ASCII;
   fdata->pseudo_header = phdr->pseudo_header;
   fdata->cinfo = NULL;
 
   passed = TRUE;
   if (cf->rfcode) {
-	  if (DFILTER_CONTAINS_FILTER(cf->rfcode)) {
-	    protocol_tree = proto_tree_create_root();
-	    dissect_packet(buf, fdata, protocol_tree);
-	    passed = dfilter_apply(cf->rfcode, protocol_tree, cf->pd);
-	    proto_tree_free(protocol_tree);
-	  }
+    protocol_tree = proto_tree_create_root();
+    dissect_packet(buf, fdata, protocol_tree);
+    passed = dfilter_apply(cf->rfcode, protocol_tree, cf->pd);
+    proto_tree_free(protocol_tree);
   }   
   if (passed) {
     plist_end = cf->plist_end;
+    fdata->prev = plist_end;
     if (plist_end != NULL)
       plist_end->next = fdata;
     else
@@ -654,25 +889,64 @@ wtap_dispatch_cb(u_char *user, const struct wtap_pkthdr *phdr, int offset,
     g_free(fdata);
 }
 
+int
+filter_packets(capture_file *cf, gchar *dftext)
+{
+  dfilter *dfcode;
+
+  if (dftext == NULL) {
+    /* The new filter is an empty filter (i.e., display all packets). */
+    dfcode = NULL;
+  } else {
+    /*
+     * We have a filter; try to compile it.
+     */
+    if (dfilter_compile(dftext, &dfcode) != 0) {
+      /* The attempt failed; report an error. */
+      simple_dialog(ESD_TYPE_WARN, NULL, dfilter_error_msg);
+      return 0;
+    }
+
+    /* Was it empty? */
+    if (dfcode == NULL) {
+      /* Yes - free the filter text, and set it to null. */
+      g_free(dftext);
+      dftext = NULL;
+    }
+  }
+
+  /* We have a valid filter.  Replace the current filter. */
+  if (cf->dfilter != NULL)
+    g_free(cf->dfilter);
+  cf->dfilter = dftext;
+  if (cf->dfcode != NULL)
+    dfilter_destroy(cf->dfcode);
+  cf->dfcode = dfcode;
+
+  /* Now go through the list of packets we've read from the capture file,
+     applying the current display filter, and, if the packet passes the
+     display filter, add it to the summary display, appropriately
+     colored.  (That's how we colorize the display - it's like filtering
+     the display, only we don't install a new filter.) */
+  colorize_packets(cf);
+  return 1;
+}
+
 void
-filter_packets(capture_file *cf)
+colorize_packets(capture_file *cf)
 {
   frame_data *fd;
   guint32 progbar_quantum;
   guint32 progbar_nextstep;
 
-  if (cf->dfilter == NULL) {
-	dfilter_clear_filter(cf->dfcode);
-  }
-  else {
-    /*
-     * Compile the filter.
-     */
-    if (dfilter_compile(cf->dfcode, cf->dfilter) != 0) {
-      simple_dialog(ESD_TYPE_WARN, NULL, dfilter_error_msg);
-      return;
-    }
-  }
+  /* We need to re-initialize all the state information that protocols
+     keep, because we're making a fresh pass through all the packets. */
+
+  /* Initialize the table of conversations. */
+  conversation_init();
+
+  /* Initialize protocol-specific variables */
+  init_all_protocols();
 
   gtk_progress_set_activity_mode(GTK_PROGRESS(prog_bar), FALSE);
 
@@ -683,9 +957,13 @@ filter_packets(capture_file *cf)
   /* Clear it out. */
   gtk_clist_clear(GTK_CLIST(packet_list));
 
+  /* We don't yet know which will be the first and last frames displayed. */
+  cf->first_displayed = NULL;
+  cf->last_displayed = NULL;
+
   /* If a packet was selected, we don't know yet what row, if any, it'll
      get. */
-  cf->selected_row = -1;
+  cf->current_row = -1;
 
   /* Iterate through the list of packets, calling a routine
      to run the filter on the packet, see if it matches, and
@@ -728,19 +1006,27 @@ filter_packets(capture_file *cf)
 
     cf->count++;
 
-    wtap_seek_read (cf-> cd_t, cf->fh, fd->file_off, cf->pd, fd->cap_len);
+    wtap_seek_read (cf->cd_t, cf->fh, fd->file_off, cf->pd, fd->cap_len);
 
     add_packet_to_packet_list(fd, cf, cf->pd);
   }
  
   gtk_progress_bar_update(GTK_PROGRESS_BAR(prog_bar), 0);
 
-  if (cf->selected_row != -1) {
-    /* We had a selected packet and it passed the filter. */
-    gtk_clist_select_row(GTK_CLIST(packet_list), cf->selected_row, -1);
+  if (cf->current_row != -1) {
+    /* The current frame passed the filter; make sure it's visible. */
+    if (!gtk_clist_row_is_visible(GTK_CLIST(packet_list), cf->current_row))
+      gtk_clist_moveto(GTK_CLIST(packet_list), cf->current_row, -1, 0.0, 0.0);
+    if (cf->current_frame_is_selected) {
+      /* It was selected, so re-select it. */
+      gtk_clist_select_row(GTK_CLIST(packet_list), cf->current_row, -1);
+    }
+    finfo_selected = NULL;
   } else {
-    /* If we had one, it didn't pass the filter. */
+    /* The current frame didn't pass the filter; make the first frame
+       the current frame, and leave it unselected. */
     unselect_packet(cf);
+    cf->current_frame = cf->first_displayed;
   }
 
   /* Unfreeze the packet list. */
@@ -867,11 +1153,16 @@ print_packets(capture_file *cf, print_args_t *print_args)
         dissect_packet(cf->pd, fd, protocol_tree);
 
         /* Print the information in that tree. */
-        proto_tree_print(FALSE, print_args->expand_all, (GNode *)protocol_tree,
+        proto_tree_print(FALSE, print_args, (GNode *)protocol_tree,
 			cf->pd, fd, cf->print_fh);
 
         proto_tree_free(protocol_tree);
 
+	if (print_args->print_hex) {
+	  /* Print the full packet data as hex. */
+	  print_hex_data(cf->print_fh, cf->pd, fd->cap_len, fd->encoding);
+	}
+
         /* Print a blank line if we print anything after this. */
         print_separator = TRUE;
       }
@@ -923,14 +1214,12 @@ change_time_formats(capture_file *cf)
       if (check_col(fd, COL_CLS_TIME)) {
         /* There are columns that show the time in the "command-line-specified"
            format; update them. */
-        for (i = 0; i < cf->cinfo.num_cols; i++) {
-          cf->cinfo.col_data[i][0] = '\0';
-        }
-        col_add_cls_time(fd);
         for (i = 0; i < cf->cinfo.num_cols; i++) {
           if (cf->cinfo.fmt_matx[i][COL_CLS_TIME]) {
             /* This is one of the columns that shows the time in
                "command-line-specified" format; update it. */
+            cf->cinfo.col_data[i][0] = '\0';
+            col_set_cls_time(fd, i);
             gtk_clist_set_text(GTK_CLIST(packet_list), fd->row, i,
 			  cf->cinfo.col_data[i]);
 	  }
@@ -987,6 +1276,126 @@ clear_tree_and_hex_views(void)
   gtk_tree_clear_items(GTK_TREE(tree_view), 0, -1);
 }
 
+gboolean
+find_packet(capture_file *cf, dfilter *sfcode)
+{
+  frame_data *start_fd;
+  frame_data *fd;
+  frame_data *new_fd = NULL;
+  guint32 progbar_quantum;
+  guint32 progbar_nextstep;
+  int count;
+  proto_tree *protocol_tree;
+
+  start_fd = cf->current_frame;
+  if (start_fd != NULL)  {
+    gtk_progress_set_activity_mode(GTK_PROGRESS(prog_bar), FALSE);
+
+    /* Iterate through the list of packets, starting at the packet we've
+       picked, calling a routine to run the filter on the packet, see if
+       it matches, and stop if so.  */
+    count = 0;
+    fd = start_fd;
+
+    proto_tree_is_visible = FALSE;
+
+    /* Update the progress bar when it gets to this value. */
+    progbar_nextstep = 0;
+    /* When we reach the value that triggers a progress bar update,
+       bump that value by this amount.
+
+       We base the progress bar on the extent to which we've gone through
+       the displayed packets, as those are the only ones for which we
+       have to do a significant amount of work. */
+    progbar_quantum = cf->count/N_PROGBAR_UPDATES;
+    gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(prog_bar), GTK_PROGRESS_LEFT_TO_RIGHT);
+
+    fd = start_fd;
+    for (;;) {
+      /* Update the progress bar, but do it only N_PROGBAR_UPDATES times;
+         when we update it, we have to run the GTK+ main loop to get it
+         to repaint what's pending, and doing so may involve an "ioctl()"
+         to see if there's any pending input from an X server, and doing
+         that for every packet can be costly, especially on a big file. */
+      if (count >= progbar_nextstep) {
+        /* let's not divide by zero. I should never be started
+         * with count == 0, so let's assert that
+         */
+        g_assert(cf->count > 0);
+
+        gtk_progress_bar_update(GTK_PROGRESS_BAR(prog_bar),
+		(gfloat) count / cf->count);
+
+        progbar_nextstep += progbar_quantum;
+        while (gtk_events_pending())
+          gtk_main_iteration();
+      }
+
+      /* Go past the current frame. */
+      if (cf->sbackward) {
+        /* Go on to the previous frame. */
+        fd = fd->prev;
+        if (fd == NULL)
+          fd = cf->plist_end;	/* wrap around */
+      } else {
+        /* Go on to the next frame. */
+        fd = fd->next;
+        if (fd == NULL)
+          fd = cf->plist;	/* wrap around */
+      }
+
+      if (fd == start_fd) {
+        /* We're back to the frame we were on originally.  The search
+           failed. */
+        break;
+      }
+
+      /* Is this packet in the display? */
+      if (fd->passed_dfilter) {
+        count++;
+
+        /* Yes.  Does it match the search filter? */
+        protocol_tree = proto_tree_create_root();
+        wtap_seek_read(cf->cd_t, cf->fh, fd->file_off, cf->pd, fd->cap_len);
+        dissect_packet(cf->pd, fd, protocol_tree);
+        if (dfilter_apply(sfcode, protocol_tree, cf->pd)) {
+          new_fd = fd;
+          break;	/* found it! */
+        }
+      }
+    }
+
+    gtk_progress_bar_update(GTK_PROGRESS_BAR(prog_bar), 0);
+  }
+
+  if (new_fd != NULL) {
+    /* We found a frame.  Make it visible, and select it. */
+    if (!gtk_clist_row_is_visible(GTK_CLIST(packet_list), new_fd->row))
+      gtk_clist_moveto(GTK_CLIST(packet_list), new_fd->row, -1, 0.0, 0.0);
+    gtk_clist_select_row(GTK_CLIST(packet_list), new_fd->row, -1);
+    return TRUE;	/* success */
+  } else
+    return FALSE;	/* failure */
+}
+
+gboolean
+goto_frame(capture_file *cf, guint fnumber)
+{
+  frame_data *fd;
+
+  for (fd = cf->plist; fd != NULL && fd->num < fnumber; fd = fd->next)
+    ;
+
+  if (fd != NULL && fd->passed_dfilter) {
+    /* We found that frame, and it's currently being displayed.
+       Make it visible, and select it. */
+    if (!gtk_clist_row_is_visible(GTK_CLIST(packet_list), fd->row))
+      gtk_clist_moveto(GTK_CLIST(packet_list), fd->row, -1, 0.0, 0.0);
+    gtk_clist_select_row(GTK_CLIST(packet_list), fd->row, -1);
+    return TRUE;	/* success */
+  } else
+    return FALSE;	/* failure */
+}
 
 /* Select the packet on a given row. */
 void
@@ -1007,25 +1416,28 @@ select_packet(capture_file *cf, int row)
     if (fd->row == row)
       break;
   }
-  cf->fd = fd;
 
-  /* Remember the ordinal number of that frame. */
-  cf->selected_packet = i;
+  g_assert(fd != NULL);
+
+  /* Record that this frame is the current frame, and that it's selected. */
+  cf->current_frame = fd;
+  cf->current_frame_is_selected = TRUE;
 
   /* Get the data in that frame. */
-  wtap_seek_read (cf-> cd_t, cf->fh, fd->file_off, cf->pd, fd->cap_len);
+  wtap_seek_read (cf->cd_t, cf->fh, fd->file_off, cf->pd, fd->cap_len);
 
   /* Create the logical protocol tree. */
   if (cf->protocol_tree)
       proto_tree_free(cf->protocol_tree);
   cf->protocol_tree = proto_tree_create_root();
   proto_tree_is_visible = TRUE;
-  dissect_packet(cf->pd, cf->fd, cf->protocol_tree);
+  dissect_packet(cf->pd, cf->current_frame, cf->protocol_tree);
 
   /* Display the GUI protocol tree and hex dump. */
   clear_tree_and_hex_views();
   proto_tree_draw(cf->protocol_tree, tree_view);
-  packet_hex_print(GTK_TEXT(byte_view), cf->pd, cf->fd->cap_len, -1, -1);
+  packet_hex_print(GTK_TEXT(byte_view), cf->pd, cf->current_frame->cap_len,
+			-1, -1, cf->current_frame->encoding);
   gtk_text_thaw(GTK_TEXT(byte_view));
 
   /* A packet is selected, so "File/Print Packet" has something to print. */
@@ -1038,8 +1450,7 @@ select_packet(capture_file *cf, int row)
 void
 unselect_packet(capture_file *cf)
 {
-  cf->selected_packet = -1;	/* nothing there to be selected */
-  cf->selected_row = -1;
+  cf->current_frame_is_selected = FALSE;
 
   /* Destroy the protocol tree for that packet. */
   if (cf->protocol_tree != NULL) {
@@ -1047,6 +1458,8 @@ unselect_packet(capture_file *cf)
     cf->protocol_tree = NULL;
   }
 
+  finfo_selected = NULL;
+
   /* Clear out the display of that packet. */
   clear_tree_and_hex_views();
 
