@@ -1,7 +1,7 @@
 /* file.h
  * Definitions for file structures and routines
  *
- * $Id: file.h,v 1.46 1999/09/23 04:38:52 ashokn Exp $
+ * $Id: file.h,v 1.55 1999/11/29 01:54:01 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -60,12 +60,6 @@
 #define file_read(buf, bsize, count, file) gzread((file),(buf),((count)*(bsize)))
 #define file_write(buf, bsize, count, file) gzwrite((file),(buf),((count)*(bsize)))
 #define file_close gzclose
-static inline int file_error(void *fh) {
-    int errnum;
-    gzerror(fh, &errnum);
-    if (errnum<0) return errnum;
-    return 0;
-}
 
 #else /* No zLib */
 #define FILE_T FILE *
@@ -75,7 +69,6 @@ static inline int file_error(void *fh) {
 #define file_read fread
 #define file_write fwrite
 #define file_close fclose
-#define file_error ferror
 #endif /* HAVE_LIBZ */
 
 typedef struct bpf_program bpf_prog;
@@ -97,9 +90,6 @@ typedef struct _capture_file {
   gboolean     update_progbar; /* TRUE if we should update the progress bar */
   long         progbar_quantum; /* Number of bytes read per progress bar update */
   long         progbar_nextstep; /* Next point at which to update progress bar */
-  float        bouncebar_pos; /* Position of bounce bar */
-  float        bouncebar_step; /* Step */
-  int          bouncebar_reversed; /* Are we going right-to-left? */
   gchar       *iface;     /* Interface */
   gchar       *save_file; /* File that user saved capture to */
   int          save_file_fd; /* File descriptor for saved file */
@@ -107,22 +97,23 @@ typedef struct _capture_file {
   wtap        *wth;       /* Wiretap session */
   dfilter     *rfcode;    /* Compiled read filter program */ 
   gchar       *dfilter;   /* Display filter string */
-#if 0
-  GNode       *dfcode;    /* Compiled display filter program */ 
-#endif
   colfilter   *colors;	  /* Colors for colorizing packet window */
   dfilter     *dfcode;    /* Compiled display filter program */ 
 #ifdef HAVE_LIBPCAP
   gchar       *cfilter;   /* Capture filter string */
   bpf_prog     fcode;     /* Compiled capture filter program */
 #endif
+  gchar       *sfilter;   /* Search filter string */
+  gboolean     sbackward;  /* TRUE if search is backward, FALSE if forward */
   guint8       pd[WTAP_MAX_PACKET_SIZE];  /* Packet data */
   frame_data  *plist;     /* Packet list */
   frame_data  *plist_end; /* Last packet in list */
+  frame_data  *first_displayed; /* First frame displayed */
+  frame_data  *last_displayed;  /* Last frame displayed */
   column_info  cinfo;    /* Column formatting information */
-  int          selected_packet;   /* Index in packet list of currently selected packet, if any */
-  int          selected_row;   /* Row in packet display of currently selected packet, if any */
-  frame_data  *fd;        /* Frame data for currently selected packet */
+  frame_data  *current_frame;  /* Frame data for current frame */
+  int          current_row;    /* Row in packet display of current frame */
+  gboolean     current_frame_is_selected; /* TRUE if that frame is selected */
   proto_tree  *protocol_tree; /* Protocol tree for currently selected packet */
   FILE        *print_fh;  /* File we're printing to */
 } capture_file;
@@ -130,22 +121,17 @@ typedef struct _capture_file {
 int  open_cap_file(char *, capture_file *);
 void close_cap_file(capture_file *, void *, guint);
 int  read_cap_file(capture_file *);
-int  tail_cap_file(char *, capture_file *);
+int  start_tail_cap_file(char *, capture_file *);
+int  continue_tail_cap_file(capture_file *, int);
+int  finish_tail_cap_file(capture_file *);
 /* size_t read_frame_header(capture_file *); */
 
-typedef struct {
-  gboolean	to_file;	/* TRUE if we're printing to a file */
-  char		*dest;		/* if printing to file, pathname;
-				   if not, command string */
-  gboolean	print_summary;	/* TRUE if we should just print summary;
-				   FALSE if we should print protocol tree. */
-  gboolean	expand_all;	/* TRUE if we should expand all levels;
-				   FALSE if we should expand as displayed. */
-} print_args_t;
-
+int filter_packets(capture_file *cf, gchar *dfilter);
+void colorize_packets(capture_file *);
 int print_packets(capture_file *cf, print_args_t *print_args);
-void filter_packets(capture_file *);
 void change_time_formats(capture_file *);
+gboolean find_packet(capture_file *cf, dfilter *sfcode);
+gboolean goto_frame(capture_file *cf, guint fnumber);
 void select_packet(capture_file *, int);
 void unselect_packet(capture_file *);
 
