@@ -1,7 +1,7 @@
 /* file_dlg.c
  * Dialog boxes for handling files
  *
- * $Id: file_dlg.c,v 1.4 1999/09/23 07:20:20 guy Exp $
+ * $Id: file_dlg.c,v 1.11 1999/10/20 22:36:05 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -40,6 +40,9 @@
 #ifndef __GLOBALS_H__
 #include "globals.h"
 #endif
+#ifndef __GTKGLOBALS_H__
+#include "gtkglobals.h"
+#endif
 
 #ifndef __KEYS_H__
 #include "keys.h"
@@ -57,6 +60,10 @@
 #include "menu.h"
 #endif
 
+#ifndef __MAIN_H__
+#include "main.h"
+#endif
+
 static void file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs);
 static void file_save_ok_cb(GtkWidget *w, GtkFileSelection *fs);
 static void file_save_as_ok_cb(GtkWidget *w, GtkFileSelection *fs);
@@ -104,15 +111,6 @@ file_open_cmd_cb(GtkWidget *w, gpointer data) {
     (file_sel)->cancel_button), "clicked", (GtkSignalFunc)
     gtk_widget_destroy, GTK_OBJECT (file_sel));
 
-#ifdef HAVE_LIBPCAP
-  if ((sync_mode || fork_mode) && (cf.save_file != NULL))
-#else
-  if (cf.save_file != NULL)
-#endif
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel), cf.save_file);
-  else
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel), "");
-
   gtk_widget_show(file_sel);
 }
 
@@ -126,13 +124,9 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
   cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs)));
   filter_te = gtk_object_get_data(GTK_OBJECT(w), E_RFILTER_TE_KEY);
   rfilter = gtk_entry_get_text(GTK_ENTRY(filter_te));
-  if (rfilter[0] != '\0') {
-	rfcode = dfilter_new();
-	if (dfilter_compile(rfcode, rfilter) != 0) {
-		simple_dialog(ESD_TYPE_WARN, NULL, dfilter_error_msg);
-		dfilter_destroy(rfcode);
-		return;
-	}
+  if (dfilter_compile(rfilter, &rfcode) != 0) {
+    simple_dialog(ESD_TYPE_WARN, NULL, dfilter_error_msg);
+    return;
   }
 
   /* Try to open the capture file. */
