@@ -1,7 +1,7 @@
 /* keys.h
  * Key definitions for various objects
  *
- * $Id: keys.h,v 1.5 1999/09/12 23:54:09 guy Exp $
+ * $Id: keys.h,v 1.7 1999/11/25 18:02:24 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -28,7 +28,9 @@
 
 /* Keys for gtk_object_set_data */
 
-#define E_DFILTER_TE_KEY          "display_filter_te"
+#define E_DFILTER_TE_KEY          "display_filter_entry"
+#define E_DFILTER_CM_KEY          "display_filter_combo"
+#define E_DFILTER_FL_KEY          "display_filter_list"
 #define E_RFILTER_TE_KEY          "read_filter_te"
 
 #define PRINT_CMD_LB_KEY          "printer_command_label"
@@ -36,8 +38,5 @@
 #define PRINT_FILE_BT_KEY         "printer_file_button"
 #define PRINT_FILE_TE_KEY         "printer_file_entry"
 #define PRINT_DEST_RB_KEY         "printer_destination_radio_button"
-#define PRINT_SUMMARY_RB_KEY      "printer_summary_radio_button"
-#define PRINT_EXPAND_ALL_RB_KEY   "printer_expand_all_radio_button"
-#define PRINT_AS_DISPLAYED_RB_KEY "printer_as_displayed_radio_button"
 
 #endif
