@@ -1,7 +1,7 @@
 /* packet-atm.c
  * Routines for ATM packet disassembly
  *
- * $Id: packet-atm.c,v 1.1 1999/08/20 06:55:05 guy Exp $
+ * $Id: packet-atm.c,v 1.9 1999/11/27 06:17:23 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -36,8 +36,26 @@
 #include "packet.h"
 #include "resolv.h"
 
+#if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
+#include "packet-snmp.h"
+#endif
+
 static int proto_atm = -1;
+static int hf_atm_vpi = -1;
+static int hf_atm_vci = -1;
 static int proto_atm_lane = -1;
+#if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
+static int proto_ilmi = -1;
+#endif
+
+static gint ett_atm = -1;
+static gint ett_atm_lane = -1;
+static gint ett_atm_lane_lc_lan_dest = -1;
+static gint ett_atm_lane_lc_lan_dest_rd = -1;
+static gint ett_atm_lane_lc_flags = -1;
+#if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
+static gint ett_ilmi = -1;
+#endif
 
 /*
  * See
@@ -137,7 +155,7 @@ dissect_le_client(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
   if (tree) {
     ti = proto_tree_add_item_format(tree, proto_atm_lane, offset, 2, NULL,
 					    "ATM LANE");
-    lane_tree = proto_item_add_subtree(ti, ETT_ATM_LANE);
+    lane_tree = proto_item_add_subtree(ti, ett_atm_lane);
 
     proto_tree_add_text(lane_tree, offset, 2, "LE Client: 0x%04X",
 			pntohs(&pd[offset]));
@@ -156,7 +174,7 @@ dissect_lan_destination(const u_char *pd, int offset, const char *type, proto_tr
 
   td = proto_tree_add_text(tree, offset, 8, "%s LAN destination",
     			type);
-  dest_tree = proto_item_add_subtree(td, ETT_ATM_LANE_LC_LAN_DEST);
+  dest_tree = proto_item_add_subtree(td, ett_atm_lane_lc_lan_dest);
   tag = pntohs(&pd[offset]);
   proto_tree_add_text(dest_tree, offset, 2, "Tag: %s",
 	val_to_str(tag, le_control_landest_tag_vals,
@@ -175,7 +193,7 @@ dissect_lan_destination(const u_char *pd, int offset, const char *type, proto_tr
     route_descriptor = pntohs(&pd[offset]);
     trd = proto_tree_add_text(dest_tree, offset, 2, "Route descriptor: 0x%02X",
     			route_descriptor);
-    rd_tree = proto_item_add_subtree(td, ETT_ATM_LANE_LC_LAN_DEST_RD);
+    rd_tree = proto_item_add_subtree(td, ett_atm_lane_lc_lan_dest_rd);
     proto_tree_add_text(rd_tree, offset, 2,
 	    decode_numeric_bitfield(route_descriptor, 0xFFF0, 2*8,
 			"LAN ID = %u"));
@@ -202,7 +220,7 @@ dissect_le_control(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
   if (tree) {
     ti = proto_tree_add_item_format(tree, proto_atm_lane, offset, 108, NULL,
 					    "ATM LANE");
-    lane_tree = proto_item_add_subtree(ti, ETT_ATM_LANE);
+    lane_tree = proto_item_add_subtree(ti, ett_atm_lane);
 
     proto_tree_add_text(lane_tree, offset, 2, "Marker: 0x%04X",
 			pntohs(&pd[offset]));
@@ -246,7 +264,7 @@ dissect_le_control(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
     flags = pntohs(&pd[offset]);
     tf = proto_tree_add_text(lane_tree, offset, 2, "Flags: 0x%04X",
     			pntohs(&pd[offset]));
-    flags_tree = proto_item_add_subtree(tf, ETT_ATM_LANE_LC_FLAGS);
+    flags_tree = proto_item_add_subtree(tf, ett_atm_lane_lc_flags);
     proto_tree_add_text(flags_tree, offset, 2, "%s",
 	decode_boolean_bitfield(flags, 0x0001, 8*2,
 				"Remote address", "Local address"));
@@ -398,6 +416,74 @@ static const value_string ipsilon_type_vals[] = {
 	{ 0,                NULL }
 };
 
+/*
+ * We don't know what kind of traffic this is; try to guess.
+ * We at least know it's AAL5....
+ */
+static void
+atm_guess_content(const u_char *pd, frame_data *fd)
+{
+	if (fd->pseudo_header.ngsniffer_atm.Vpi == 0) {
+		/*
+		 * Traffic on some PVCs with a VPI of 0 and certain
+		 * VCIs is of particular types.
+		 */
+		switch (fd->pseudo_header.ngsniffer_atm.Vci) {
+
+		case 5:
+			/*
+			 * Signalling AAL.
+			 */
+			fd->pseudo_header.ngsniffer_atm.AppTrafType =
+			    ATT_AAL_SIGNALLING;
+			return;
+
+		case 16:
+			/*
+			 * ILMI.
+			 */
+			fd->pseudo_header.ngsniffer_atm.AppTrafType |=
+			    ATT_HL_ILMI;
+			return;
+		}
+	}
+
+	/*
+	 * OK, we can't tell what it is based on the VPI/VCI; try
+	 * guessing based on the contents.
+	 */
+	if (pd[0] == 0xaa && pd[1] == 0xaa && pd[2] == 0x03) {
+		/*
+		 * Looks like a SNAP header; assume it's LLC multiplexed
+		 * RFC 1483 traffic.
+		 */
+		fd->pseudo_header.ngsniffer_atm.AppTrafType |= ATT_HL_LLCMX;
+	} else {
+		/*
+		 * Assume it's LANE.
+		 */
+		fd->pseudo_header.ngsniffer_atm.AppTrafType |= ATT_HL_LANE;
+		if (pd[0] == 0xff && pd[1] == 0x00) {
+			/*
+			 * Looks like LE Control traffic.
+			 */
+			fd->pseudo_header.ngsniffer_atm.AppHLType =
+			    AHLT_LANE_LE_CTRL;
+		} else {
+			/*
+			 * XXX - Ethernet, or Token Ring?
+			 * Assume Ethernet for now; if we see earlier
+			 * LANE traffic, we may be able to figure out
+			 * the traffic type from that, but there may
+			 * still be situations where the user has to
+			 * tell us.
+			 */
+			fd->pseudo_header.ngsniffer_atm.AppHLType =
+			    AHLT_LANE_802_3;
+		}
+	}
+}
+
 void
 dissect_atm(const u_char *pd, frame_data *fd, proto_tree *tree) 
 {
@@ -409,6 +495,38 @@ dissect_atm(const u_char *pd, frame_data *fd, proto_tree *tree)
 
   aal_type = fd->pseudo_header.ngsniffer_atm.AppTrafType & ATT_AALTYPE;
   hl_type = fd->pseudo_header.ngsniffer_atm.AppTrafType & ATT_HLTYPE;
+  if (aal_type == ATT_AAL5) {
+    if (hl_type == ATT_HL_UNKNOWN ||
+	fd->pseudo_header.ngsniffer_atm.AppHLType == AHLT_UNKNOWN) {
+      /*
+       * The joys of a connection-oriented link layer; the type of
+       * traffic may be implied by the connection on which it's
+       * traveling, rather than being specified in the packet itself.
+       *
+       * For this packet, the program that captured the packet didn't
+       * save the type of traffic, presumably because it didn't know
+       * the traffic type (either it didn't see the connection setup
+       * and wasn't running on one of the endpoints, and wasn't later
+       * told, e.g. by the human running it, what type of traffic was
+       * on that circuit, or was running on one of the endpoints but
+       * was using, to capture the packets, a mechanism that either
+       * doesn't have access to data saying what's going over the
+       * connection or doesn't bother providing that information).
+       *
+       * For now, we try to guess the traffic type based on the VPI/VCI
+       * or the packet header; later, we should provide a mechanism
+       * by which the user can specify what sort of traffic is on a
+       * particular circuit.
+       */
+      atm_guess_content(pd, fd);
+
+      /*
+       * OK, now get the AAL type and high-layer type again.
+       */
+      aal_type = fd->pseudo_header.ngsniffer_atm.AppTrafType & ATT_AALTYPE;
+      hl_type = fd->pseudo_header.ngsniffer_atm.AppTrafType & ATT_HLTYPE;
+    }
+  }
 
   if (check_col(fd, COL_PROTOCOL))
     col_add_str(fd, COL_PROTOCOL, "ATM");
@@ -446,17 +564,19 @@ dissect_atm(const u_char *pd, frame_data *fd, proto_tree *tree)
   if (tree) {
     ti = proto_tree_add_item_format(tree, proto_atm, 0, 0, NULL,
 					    "ATM");
-    atm_tree = proto_item_add_subtree(ti, ETT_ATM);
+    atm_tree = proto_item_add_subtree(ti, ett_atm);
 
     proto_tree_add_text(atm_tree, 0, 0, "AAL: %s",
 	val_to_str(aal_type, aal_vals, "Unknown AAL (%x)"));
     if (aal_type == ATT_AAL5) {
       proto_tree_add_text(atm_tree, 0, 0, "Traffic type: %s",
 	val_to_str(hl_type, aal5_hltype_vals, "Unknown AAL5 traffic type (%x)"));
-    }
-    if (aal_type == ATT_AAL5) {
       switch (hl_type) {
 
+      case ATT_HL_LLCMX:
+        proto_tree_add_text(atm_tree, 0, 0, "LLC multiplexed traffic");
+        break;
+
       case ATT_HL_VCMX:
         proto_tree_add_text(atm_tree, 0, 0, "VC multiplexed traffic type: %s",
 		val_to_str(fd->pseudo_header.ngsniffer_atm.AppHLType,
@@ -476,9 +596,9 @@ dissect_atm(const u_char *pd, frame_data *fd, proto_tree *tree)
         break;
       }
     }
-    proto_tree_add_text(atm_tree, 0, 0, "VPI: %u",
+    proto_tree_add_item(atm_tree, hf_atm_vpi, 0, 0,
 		fd->pseudo_header.ngsniffer_atm.Vpi);
-    proto_tree_add_text(atm_tree, 0, 0, "VCI: %u",
+    proto_tree_add_item(atm_tree, hf_atm_vci, 0, 0,
 		fd->pseudo_header.ngsniffer_atm.Vci);
     switch (fd->pseudo_header.ngsniffer_atm.channel) {
 
@@ -498,31 +618,57 @@ dissect_atm(const u_char *pd, frame_data *fd, proto_tree *tree)
  		fd->pseudo_header.ngsniffer_atm.channel);
       break;
     }
-    proto_tree_add_text(atm_tree, 0, 0, "Cells: %u",
+    if (fd->pseudo_header.ngsniffer_atm.cells != 0) {
+      /*
+       * If the cell count is 0, assume it means we don't know how
+       * many cells it was.
+       *
+       * XXX - also, if this is AAL5 traffic, assume it means we don't
+       * know what was in the AAL5 trailer.  We may, however, find
+       * some capture program that can give us the AAL5 trailer
+       * information but not the cell count, in which case we need
+       * some other way of indicating whether we have the AAL5 trailer
+       * information.
+       */
+      proto_tree_add_text(atm_tree, 0, 0, "Cells: %u",
 		fd->pseudo_header.ngsniffer_atm.cells);
-    if (aal_type == ATT_AAL5) {
-      proto_tree_add_text(atm_tree, 0, 0, "AAL5 U2U: %u",
+      if (aal_type == ATT_AAL5) {
+        proto_tree_add_text(atm_tree, 0, 0, "AAL5 U2U: %u",
 		fd->pseudo_header.ngsniffer_atm.aal5t_u2u);
-      proto_tree_add_text(atm_tree, 0, 0, "AAL5 len: %u",
+        proto_tree_add_text(atm_tree, 0, 0, "AAL5 len: %u",
 		fd->pseudo_header.ngsniffer_atm.aal5t_len);
-      proto_tree_add_text(atm_tree, 0, 0, "AAL5 checksum: 0x%08X",
+        proto_tree_add_text(atm_tree, 0, 0, "AAL5 checksum: 0x%08X",
 		fd->pseudo_header.ngsniffer_atm.aal5t_chksum);
+      }
     }
   }
 
-  if (aal_type == ATT_AAL5) {
+  switch (aal_type) {
+
+  case ATT_AAL_SIGNALLING:
+    dissect_sscop(pd, offset, fd, tree);
+    break;
+
+  case ATT_AAL5:
     switch (hl_type) {
 
-#if 0
     case ATT_HL_LLCMX:
       /* Dissect as WTAP_ENCAP_ATM_RFC1483 */
+      /* The ATM iptrace capture that we have hows LLC at this point,
+       * so that's what I'm calling */
+      dissect_llc(pd, offset, fd, tree);
       break;
-#endif
 
     case ATT_HL_LANE:
       dissect_lane(pd, offset, fd, tree);
       break;
 
+#if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
+    case ATT_HL_ILMI:
+      dissect_snmp_pdu(pd, offset, fd, tree, "ILMI", proto_ilmi, ett_ilmi);
+      break;
+#endif
+
     default:
       if (tree) {
         /* Dump it as raw data. */
@@ -530,17 +676,44 @@ dissect_atm(const u_char *pd, frame_data *fd, proto_tree *tree)
         break;
       }
     }
-  } else {
+    break;
+
+  default:
     if (tree) {
       /* Dump it as raw data.  (Is this a single cell?) */
       dissect_data(pd, offset, fd, tree);
     }
+    break;
   }
 }
 
 void
 proto_register_atm(void)
 {
+	static hf_register_info hf[] = {
+		{ &hf_atm_vpi,
+		{ "VPI",		"atm.vpi", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
+
+		{ &hf_atm_vci,
+		{ "VCI",		"atm.vci", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
+	};
+	static gint *ett[] = {
+		&ett_atm,
+#if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
+		&ett_ilmi,
+#endif
+		&ett_atm_lane,
+		&ett_atm_lane_lc_lan_dest,
+		&ett_atm_lane_lc_lan_dest_rd,
+		&ett_atm_lane_lc_flags,
+	};
 	proto_atm = proto_register_protocol("ATM", "atm");
+	proto_register_field_array(proto_atm, hf, array_length(hf));
+#if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
+	proto_ilmi = proto_register_protocol("ILMI", "ilmi");
+#endif
 	proto_atm_lane = proto_register_protocol("ATM LANE", "lane");
+	proto_register_subtree_array(ett, array_length(ett));
 }
