@@ -1,7 +1,7 @@
 /* packet-eth.c
  * Routines for ethernet packet disassembly
  *
- * $Id: packet-eth.c,v 1.19 1999/09/15 06:26:42 gram Exp $
+ * $Id: packet-eth.c,v 1.24 1999/11/20 03:27:02 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -45,6 +45,9 @@ static int hf_eth_src = -1;
 static int hf_eth_len = -1;
 static int hf_eth_type = -1;
 
+static gint ett_ieee8023 = -1;
+static gint ett_ether2 = -1;
+
 #define ETH_HEADER_SIZE	14
 
 /* These are the Netware-ish names for the different Ethernet frame types.
@@ -118,18 +121,15 @@ dissect_eth(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     return;
   }
 
-  if (check_col(fd, COL_RES_DL_DST))
-    col_add_str(fd, COL_RES_DL_DST, get_ether_name((u_char *)&pd[offset+0]));
-  if (check_col(fd, COL_RES_DL_SRC))
-    col_add_str(fd, COL_RES_DL_SRC, get_ether_name((u_char *)&pd[offset+6]));
-  if (check_col(fd, COL_UNRES_DL_DST))
-    col_add_str(fd, COL_UNRES_DL_DST, ether_to_str((u_char *)&pd[offset+0]));
-  if (check_col(fd, COL_UNRES_DL_SRC))
-    col_add_str(fd, COL_UNRES_DL_SRC, ether_to_str((u_char *)&pd[offset+6]));
+  SET_ADDRESS(&pi.dl_src, AT_ETHER, 6, &pd[offset+6]);
+  SET_ADDRESS(&pi.src, AT_ETHER, 6, &pd[offset+6]);
+  SET_ADDRESS(&pi.dl_dst, AT_ETHER, 6, &pd[offset+0]);
+  SET_ADDRESS(&pi.dst, AT_ETHER, 6, &pd[offset+0]);
+
   if (check_col(fd, COL_PROTOCOL))
     col_add_str(fd, COL_PROTOCOL, "Ethernet");
 
-  etype = (pd[offset+12] << 8) | pd[offset+13];
+  etype = pntohs(&pd[offset+12]);
 
 	/* either ethernet802.3 or ethernet802.2 */
   if (etype <= IEEE_802_3_MAX_LEN) {
@@ -154,14 +154,14 @@ dissect_eth(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     }
     if (tree) {
 
-	ti = proto_tree_add_item_format(tree, proto_eth, 0, ETH_HEADER_SIZE,
+	ti = proto_tree_add_item_format(tree, proto_eth, offset, ETH_HEADER_SIZE,
 		NULL, "IEEE 802.3 %s", (ethhdr_type == ETHERNET_802_3 ? "Raw " : ""));
 
-	fh_tree = proto_item_add_subtree(ti, ETT_IEEE8023);
+	fh_tree = proto_item_add_subtree(ti, ett_ieee8023);
 
-	proto_tree_add_item(fh_tree, hf_eth_dst, 0, 6, &pd[offset+0]);
-	proto_tree_add_item(fh_tree, hf_eth_src, 6, 6, &pd[offset+6]);
-	proto_tree_add_item(fh_tree, hf_eth_len, 12, 2, length);
+	proto_tree_add_item(fh_tree, hf_eth_dst, offset+0, 6, &pd[offset+0]);
+	proto_tree_add_item(fh_tree, hf_eth_src, offset+6, 6, &pd[offset+6]);
+	proto_tree_add_item(fh_tree, hf_eth_len, offset+12, 2, length);
 
 	/* Convert the LLC length from the 802.3 header to a total
 	   length, by adding in the Ethernet header size, and set
@@ -180,18 +180,13 @@ dissect_eth(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       col_add_str(fd, COL_INFO, "Ethernet II");
     if (tree) {
 
-	ti = proto_tree_add_item_format(tree, proto_eth, 0, ETH_HEADER_SIZE,
+	ti = proto_tree_add_item_format(tree, proto_eth, offset, ETH_HEADER_SIZE,
 		NULL, "Ethernet II");
 
-	fh_tree = proto_item_add_subtree(ti, ETT_ETHER2);
+	fh_tree = proto_item_add_subtree(ti, ett_ether2);
 
-	proto_tree_add_item_format(fh_tree, hf_eth_dst, 0, 6, &pd[offset+0],
-		"Destination: %s (%s)", ether_to_str((guint8 *) &pd[offset+0]),
-		get_ether_name((u_char *) &pd[offset+0]));
-
-	proto_tree_add_item_format(fh_tree, hf_eth_src, 6, 6, &pd[offset+6],
-		"Source: %s (%s)", ether_to_str((guint8 *) &pd[offset+6]),
-		get_ether_name((u_char *) &pd[offset+6]));
+	proto_tree_add_item(fh_tree, hf_eth_dst, offset+0, 6, &pd[offset+0]);
+	proto_tree_add_item(fh_tree, hf_eth_src, offset+6, 6, &pd[offset+6]);
     }
   }
   offset += ETH_HEADER_SIZE;
@@ -215,19 +210,28 @@ proto_register_eth(void)
 	static hf_register_info hf[] = {
 
 		{ &hf_eth_dst,
-		{ "Destination",	"eth.dst", FT_ETHER, NULL }},
+		{ "Destination",	"eth.dst", FT_ETHER, BASE_NONE, NULL, 0x0,
+			"Destination Hardware Address" }},
 
 		{ &hf_eth_src,
-		{ "Source",		"eth.src", FT_ETHER, NULL }},
+		{ "Source",		"eth.src", FT_ETHER, BASE_NONE, NULL, 0x0,
+			"Source Hardware Address" }},
 
 		{ &hf_eth_len,
-		{ "Length",		"eth.len", FT_UINT16, NULL }},
+		{ "Length",		"eth.len", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		/* registered here but handled in ethertype.c */
 		{ &hf_eth_type,
-		{ "Type",		"eth.type", FT_VALS_UINT16, VALS(etype_vals) }}
+		{ "Type",		"eth.type", FT_UINT16, BASE_HEX, VALS(etype_vals), 0x0,
+			"" }}
+	};
+	static gint *ett[] = {
+		&ett_ieee8023,
+		&ett_ether2,
 	};
 
 	proto_eth = proto_register_protocol ("Ethernet", "eth" );
 	proto_register_field_array(proto_eth, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
