@@ -1,7 +1,7 @@
 /* packet-ip.c
  * Routines for IP and miscellaneous IP protocol packet disassembly
  *
- * $Id: packet-ip.c,v 1.48 1999/09/23 19:05:28 guy Exp $
+ * $Id: packet-ip.c,v 1.64 1999/11/21 14:43:52 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -60,17 +60,31 @@ static int hf_ip_version = -1;
 static int hf_ip_hdr_len = -1;
 static int hf_ip_tos = -1;
 static int hf_ip_tos_precedence = -1;
+static int hf_ip_tos_delay = -1;
+static int hf_ip_tos_throughput = -1;
+static int hf_ip_tos_reliability = -1;
+static int hf_ip_tos_cost = -1;
 static int hf_ip_len = -1;
 static int hf_ip_id = -1;
 static int hf_ip_dst = -1;
 static int hf_ip_src = -1;
 static int hf_ip_addr = -1;
 static int hf_ip_flags = -1;
+static int hf_ip_flags_df = -1;
+static int hf_ip_flags_mf = -1;
 static int hf_ip_frag_offset = -1;
 static int hf_ip_ttl = -1;
 static int hf_ip_proto = -1;
 static int hf_ip_checksum = -1;
 
+static gint ett_ip = -1;
+static gint ett_ip_tos = -1;
+static gint ett_ip_off = -1;
+static gint ett_ip_options = -1;
+static gint ett_ip_option_sec = -1;
+static gint ett_ip_option_route = -1;
+static gint ett_ip_option_timestamp = -1;
+
 static int proto_igmp = -1;
 static int hf_igmp_version = -1;
 static int hf_igmp_type = -1;
@@ -78,8 +92,14 @@ static int hf_igmp_unused = -1;
 static int hf_igmp_checksum = -1;
 static int hf_igmp_group = -1;
 
-static int proto_icmp = -1;
+static gint ett_igmp = -1;
 
+static int proto_icmp = -1;
+static int hf_icmp_type = -1;
+static int hf_icmp_code = -1;
+static int hf_icmp_checksum = -1;
+
+static gint ett_icmp = -1;
 
 /* ICMP structs and definitions */
 typedef struct _e_icmp {
@@ -156,20 +176,66 @@ typedef struct _e_igmp {
 #define IGMP_MTRC_RESP 0x1e
 #define IGMP_MTRC      0x1f
 
+/* EIGRP Structs and Definitions. */    
+
+/* EIGRP Opcodes */
+
+#define EIGRP_UPDATE    0x01
+#define EIGRP_REQUEST   0x02
+#define EIGRP_QUERY     0x03
+#define EIGRP_REPLY     0x04
+#define EIGRP_HELLO     0x05
+
+typedef struct _e_eigrp 
+   {
+   guint8 eigrp_version;
+   guint8 eigrp_opcode;
+   guint16 eigrp_checksum;
+   guint16 eigrp_subnets;
+   guint16 eigrp_networks;
+   guint32 eigrp_sequence;
+   guint32 eigrp_asnumber;
+   guint8 eigrp_type1;
+   guint8 eigrp_subtype1;
+   guint16 eigrp_length1;
+   guint16 eigrp_holdtime;
+   guint8 eigrp_type2;
+   guint8 eigrp_subtype2;
+   guint16 eigrp_length2;
+   guint8 eigrp_level;
+   guint16 eigrp_dummy;
+   } e_eigrp;
+
 /* IP structs and definitions */
 
-typedef struct _e_ip {
-  guint8  ip_v_hl; /* combines ip_v and ip_hl */
-  guint8  ip_tos;
-  guint16 ip_len;
-  guint16 ip_id;
-  guint16 ip_off;
-  guint8  ip_ttl;
-  guint8  ip_p;
-  guint16 ip_sum;
-  guint32 ip_src;
-  guint32 ip_dst;
-} e_ip;
+typedef struct _e_ip 
+   {
+   guint8  ip_v_hl; /* combines ip_v and ip_hl */
+   guint8  ip_tos;
+   guint16 ip_len;
+   guint16 ip_id;
+   guint16 ip_off;
+   guint8  ip_ttl;
+   guint8  ip_p;
+   guint16 ip_sum;
+   guint32 ip_src;
+   guint32 ip_dst;
+   } e_ip;
+
+/* Offsets of fields within an IP header. */
+#define	IPH_V_HL	0
+#define	IPH_TOS		1
+#define	IPH_LEN		2
+#define	IPH_ID		4
+#define	IPH_TTL		6
+#define	IPH_OFF		8
+#define	IPH_P		9
+#define	IPH_SUM		10
+#define	IPH_SRC		12
+#define	IPH_DST		16
+
+/* Minimum IP header length. */
+#define	IPH_MIN_LEN	20
 
 /* IP flags. */
 #define IP_CE		0x8000		/* Flag: "Congestion"		*/
@@ -187,15 +253,16 @@ typedef struct _e_ip {
 #define IPTOS_SECURITY    0x1E
 
 #define IPTOS_PREC_MASK		0xE0
-#define IPTOS_PREC(tos)		((tos)&IPTOS_PREC_MASK)
-#define IPTOS_PREC_NETCONTROL           0xe0
-#define IPTOS_PREC_INTERNETCONTROL      0xc0
-#define IPTOS_PREC_CRITIC_ECP           0xa0
-#define IPTOS_PREC_FLASHOVERRIDE        0x80
-#define IPTOS_PREC_FLASH                0x60
-#define IPTOS_PREC_IMMEDIATE            0x40
-#define IPTOS_PREC_PRIORITY             0x20
-#define IPTOS_PREC_ROUTINE              0x00
+#define IPTOS_PREC_SHIFT	5
+#define IPTOS_PREC(tos)		(((tos)&IPTOS_PREC_MASK)>>IPTOS_PREC_SHIFT)
+#define IPTOS_PREC_NETCONTROL           7
+#define IPTOS_PREC_INTERNETCONTROL      6
+#define IPTOS_PREC_CRITIC_ECP           5
+#define IPTOS_PREC_FLASHOVERRIDE        4
+#define IPTOS_PREC_FLASH                3
+#define IPTOS_PREC_IMMEDIATE            2
+#define IPTOS_PREC_PRIORITY             1
+#define IPTOS_PREC_ROUTINE              0
 
 /* IP options */
 #define IPOPT_COPY		0x80
@@ -293,7 +360,7 @@ dissect_ipopt_security(const ip_tcp_opt *optp, const u_char *opd, int offset,
     {0,                  NULL          } };
 
   tf = proto_tree_add_text(opt_tree, offset,      optlen, "%s:", optp->name);
-  field_tree = proto_item_add_subtree(tf, optp->subtree_index);
+  field_tree = proto_item_add_subtree(tf, *optp->subtree_index);
   offset += 2;
 
   val = pntohs(opd);
@@ -329,7 +396,7 @@ dissect_ipopt_route(const ip_tcp_opt *optp, const u_char *opd, int offset,
 
   tf = proto_tree_add_text(opt_tree, offset,      optlen, "%s (%d bytes)",
 				optp->name, optlen);
-  field_tree = proto_item_add_subtree(tf, optp->subtree_index);
+  field_tree = proto_item_add_subtree(tf, *optp->subtree_index);
 
   optoffset += 2;	/* skip past type and length */
   optlen -= 2;		/* subtract size of type and length */
@@ -391,7 +458,7 @@ dissect_ipopt_timestamp(const ip_tcp_opt *optp, const u_char *opd,
   guint ts;
 
   tf = proto_tree_add_text(opt_tree, offset,      optlen, "%s:", optp->name);
-  field_tree = proto_item_add_subtree(tf, optp->subtree_index);
+  field_tree = proto_item_add_subtree(tf, *optp->subtree_index);
 
   optoffset += 2;	/* skip past type and length */
   optlen -= 2;		/* subtract size of type and length */
@@ -462,7 +529,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_END,
     "EOL",
-    -1,
+    NULL,
     NO_LENGTH,
     0,
     NULL,
@@ -470,7 +537,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_NOOP,
     "NOP",
-    -1,
+    NULL,
     NO_LENGTH,
     0,
     NULL,
@@ -478,7 +545,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_SEC,
     "Security",
-    ETT_IP_OPTION_SEC,
+    &ett_ip_option_sec,
     FIXED_LENGTH,
     IPOLEN_SEC,
     dissect_ipopt_security
@@ -486,7 +553,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_SSRR,
     "Strict source route",
-    ETT_IP_OPTION_ROUTE,
+    &ett_ip_option_route,
     VARIABLE_LENGTH,
     IPOLEN_SSRR_MIN,
     dissect_ipopt_route
@@ -494,7 +561,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_LSRR,
     "Loose source route",
-    ETT_IP_OPTION_ROUTE,
+    &ett_ip_option_route,
     VARIABLE_LENGTH,
     IPOLEN_LSRR_MIN,
     dissect_ipopt_route
@@ -502,7 +569,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_RR,
     "Record route",
-    ETT_IP_OPTION_ROUTE,
+    &ett_ip_option_route,
     VARIABLE_LENGTH,
     IPOLEN_RR_MIN,
     dissect_ipopt_route
@@ -510,7 +577,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_SID,
     "Stream identifier",
-    -1,
+    NULL,
     FIXED_LENGTH,
     IPOLEN_SID,
     dissect_ipopt_sid
@@ -518,7 +585,7 @@ static const ip_tcp_opt ipopts[] = {
   {
     IPOPT_TIMESTAMP,
     "Time stamp",
-    ETT_IP_OPTION_TIMESTAMP,
+    &ett_ip_option_timestamp,
     VARIABLE_LENGTH,
     IPOLEN_TIMESTAMP_MIN,
     dissect_ipopt_timestamp
@@ -634,9 +701,16 @@ dissect_ip_tcp_options(const u_char *opd, int offset, guint length,
 
 static const value_string proto_vals[] = { {IP_PROTO_ICMP, "ICMP"},
                                            {IP_PROTO_IGMP, "IGMP"},
+                                           {IP_PROTO_EIGRP, "IGRP/EIGRP"},
                                            {IP_PROTO_TCP,  "TCP" },
                                            {IP_PROTO_UDP,  "UDP" },
                                            {IP_PROTO_OSPF, "OSPF"},
+                                           {IP_PROTO_RSVP, "RSVP"},
+                                           {IP_PROTO_AH,   "AH"  },
+                                           {IP_PROTO_GRE,  "GRE" },
+                                           {IP_PROTO_ESP,  "ESP" },
+                                           {IP_PROTO_IPV6, "IPv6"},
+                                           {IP_PROTO_PIM,  "PIM" },
                                            {0,             NULL  } };
 
 static const value_string precedence_vals[] = {
@@ -660,6 +734,21 @@ static const value_string iptos_vals[] = {
 	{ 0,			NULL }
 };
 
+static const true_false_string tos_set_low = {
+  "Low",
+  "Normal"
+};
+
+static const true_false_string tos_set_high = {
+  "High",
+  "Normal"
+};
+
+static const true_false_string flags_set_truth = {
+  "Set",
+  "Not set"
+};
+
 void
 dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   e_ip       iph;
@@ -671,7 +760,12 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   int        advance;
   guint8     nxt;
 
-  /* To do: check for runts, errs, etc. */
+  /* To do: check for errs, etc. */
+  if (!BYTES_ARE_IN_FRAME(offset, IPH_MIN_LEN)) {
+    dissect_data(pd, offset, fd, tree);
+    return;
+  }
+
   /* Avoids alignment problems on many architectures. */
   memcpy(&iph, &pd[offset], sizeof(e_ip));
   iph.ip_len = ntohs(iph.ip_len);
@@ -690,6 +784,7 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   if (pi.captured_len > len)
     pi.captured_len = len;
 
+  /* XXX - check to make sure this is at least IPH_MIN_LEN. */
   hlen = lo_nibble(iph.ip_v_hl) * 4;	/* IP header length, in bytes */
   
   switch (iph.ip_p) {
@@ -702,24 +797,17 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     case IP_PROTO_ESP:
     case IP_PROTO_AH:
     case IP_PROTO_IPV6:
+    case IP_PROTO_PIM:
       /* Names are set in the associated dissect_* routines */
       break;
     default:
       if (check_col(fd, COL_PROTOCOL))
         col_add_str(fd, COL_PROTOCOL, "IP");
       if (check_col(fd, COL_INFO))
-        col_add_fstr(fd, COL_INFO, "Unknown IP protocol (0x%02x)", iph.ip_p);
+        col_add_fstr(fd, COL_INFO, "%s (0x%02x)",
+	    ipprotostr(iph.ip_p), iph.ip_p);
   }
 
-  if (check_col(fd, COL_RES_NET_SRC))
-    col_add_str(fd, COL_RES_NET_SRC, get_hostname(iph.ip_src));
-  if (check_col(fd, COL_UNRES_NET_SRC))
-    col_add_str(fd, COL_UNRES_NET_SRC, ip_to_str((guint8 *) &iph.ip_src));
-  if (check_col(fd, COL_RES_NET_DST))
-    col_add_str(fd, COL_RES_NET_DST, get_hostname(iph.ip_dst));
-  if (check_col(fd, COL_UNRES_NET_DST))
-    col_add_str(fd, COL_UNRES_NET_DST, ip_to_str((guint8 *) &iph.ip_dst));
-    
   if (tree) {
 
     switch (IPTOS_TOS(iph.ip_tos)) {
@@ -747,54 +835,36 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     }
 
     ti = proto_tree_add_item(tree, proto_ip, offset, hlen, NULL);
-    ip_tree = proto_item_add_subtree(ti, ETT_IP);
+    ip_tree = proto_item_add_subtree(ti, ett_ip);
 
     proto_tree_add_item(ip_tree, hf_ip_version, offset, 1, hi_nibble(iph.ip_v_hl));
     proto_tree_add_item_format(ip_tree, hf_ip_hdr_len, offset, 1, hlen,
-	"Header length: %d bytes", hlen); 
+	"Header length: %u bytes", hlen);
     tf = proto_tree_add_item_format(ip_tree, hf_ip_tos, offset + 1, 1, iph.ip_tos,
 	"Type of service: 0x%02x (%s)", iph.ip_tos,
 	val_to_str( IPTOS_TOS(iph.ip_tos), iptos_vals, "Unknown") );
 
-    field_tree = proto_item_add_subtree(tf, ETT_IP_TOS);
-    proto_tree_add_item_format(field_tree, hf_ip_tos_precedence, offset + 1, 1,
-	iph.ip_tos & IPTOS_PREC_MASK, decode_enumerated_bitfield(iph.ip_tos, IPTOS_PREC_MASK,
-			   sizeof (iph.ip_tos)*8, precedence_vals, "%s precedence"));
-
-    proto_tree_add_text(field_tree, offset + 1, 1, "%s",
-       decode_boolean_bitfield(iph.ip_tos, IPTOS_LOWDELAY,
-                sizeof (iph.ip_tos)*8, "low delay", "normal delay"));
-    proto_tree_add_text(field_tree, offset + 1, 1, "%s",
-       decode_boolean_bitfield(iph.ip_tos, IPTOS_THROUGHPUT,
-            sizeof (iph.ip_tos)*8, "high throughput", "normal throughput"));
-    proto_tree_add_text(field_tree, offset + 1, 1, "%s",
-       decode_boolean_bitfield(iph.ip_tos, IPTOS_RELIABILITY,
-            sizeof (iph.ip_tos)*8, "high reliability", "normal reliability"));
-    proto_tree_add_text(field_tree, offset + 1, 1, "%s",
-       decode_boolean_bitfield(iph.ip_tos, IPTOS_LOWCOST,
-            sizeof (iph.ip_tos)*8, "low cost", "normal cost"));
+    field_tree = proto_item_add_subtree(tf, ett_ip_tos);
+    proto_tree_add_item(field_tree, hf_ip_tos_precedence, offset + 1, 1, iph.ip_tos);
+    proto_tree_add_item(field_tree, hf_ip_tos_delay, offset + 1, 1, iph.ip_tos);
+    proto_tree_add_item(field_tree, hf_ip_tos_throughput, offset + 1, 1, iph.ip_tos);
+    proto_tree_add_item(field_tree, hf_ip_tos_reliability, offset + 1, 1, iph.ip_tos);
+    proto_tree_add_item(field_tree, hf_ip_tos_cost, offset + 1, 1, iph.ip_tos);
     proto_tree_add_item(ip_tree, hf_ip_len, offset +  2, 2, iph.ip_len);
-    proto_tree_add_item_format(ip_tree, hf_ip_id, offset +  4, 2, iph.ip_id, "Identification: 0x%04x",
-      iph.ip_id);
+    proto_tree_add_item(ip_tree, hf_ip_id, offset +  4, 2, iph.ip_id);
 
     flags = (iph.ip_off & (IP_DF|IP_MF)) >> 12;
-    tf = proto_tree_add_item_format(ip_tree, hf_ip_flags, offset +  6, 1, flags,
-		   "Flags: 0x%x", flags);
-    field_tree = proto_item_add_subtree(tf, ETT_IP_OFF);
-    proto_tree_add_text(field_tree, offset + 6, 1, "%s",
-      decode_boolean_bitfield(iph.ip_off >> 12, IP_DF >> 12, 4, "don't fragment",
-                                           "may fragment"));
-    proto_tree_add_text(field_tree, offset + 6, 1, "%s",
-      decode_boolean_bitfield(iph.ip_off >> 12, IP_MF >> 12, 4, "more fragments",
-                                           "last fragment"));
+    tf = proto_tree_add_item(ip_tree, hf_ip_flags, offset +  6, 1, flags);
+    field_tree = proto_item_add_subtree(tf, ett_ip_off);
+    proto_tree_add_item(field_tree, hf_ip_flags_df, offset + 6, 1, flags),
+    proto_tree_add_item(field_tree, hf_ip_flags_mf, offset + 6, 1, flags),
 
     proto_tree_add_item(ip_tree, hf_ip_frag_offset, offset +  6, 2,
       iph.ip_off & IP_OFFSET);
     proto_tree_add_item(ip_tree, hf_ip_ttl, offset +  8, 1, iph.ip_ttl);
-    proto_tree_add_item(ip_tree, hf_ip_proto, offset +  9, 1, iph.ip_p);
-    proto_tree_add_item_format(ip_tree, hf_ip_checksum, offset + 10, 2, iph.ip_sum,
-	    "Header checksum: 0x%04x", iph.ip_sum);
-
+    proto_tree_add_item_format(ip_tree, hf_ip_proto, offset +  9, 1, iph.ip_p,
+	"Protocol: %s (0x%02x)", ipprotostr(iph.ip_p), iph.ip_p);
+    proto_tree_add_item(ip_tree, hf_ip_checksum, offset + 10, 2, iph.ip_sum);
     proto_tree_add_item(ip_tree, hf_ip_src, offset + 12, 4, iph.ip_src);
     proto_tree_add_item(ip_tree, hf_ip_dst, offset + 16, 4, iph.ip_dst);
     proto_tree_add_item_hidden(ip_tree, hf_ip_addr, offset + 12, 4, iph.ip_src);
@@ -807,7 +877,7 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       optlen = hlen - sizeof (e_ip);	/* length of options, in bytes */
       tf = proto_tree_add_text(ip_tree, offset +  20, optlen,
         "Options: (%d bytes)", optlen);
-      field_tree = proto_item_add_subtree(tf, ETT_IP_OPTIONS);
+      field_tree = proto_item_add_subtree(tf, ett_ip_options);
       dissect_ip_tcp_options(&pd[offset + 20], offset + 20, optlen,
          ipopts, N_IP_OPTS, IPOPT_END, field_tree);
     }
@@ -816,8 +886,10 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   pi.ipproto = iph.ip_p;
   pi.iplen = iph.ip_len;
   pi.iphdrlen = lo_nibble(iph.ip_v_hl);
-  pi.ip_src = iph.ip_src;
-  pi.ip_dst = iph.ip_dst;
+  SET_ADDRESS(&pi.net_src, AT_IPv4, 4, &pd[offset + IPH_SRC]);
+  SET_ADDRESS(&pi.src, AT_IPv4, 4, &pd[offset + IPH_SRC]);
+  SET_ADDRESS(&pi.net_dst, AT_IPv4, 4, &pd[offset + IPH_DST]);
+  SET_ADDRESS(&pi.dst, AT_IPv4, 4, &pd[offset + IPH_DST]);
 
   /* Skip over header + options */
   offset += hlen;
@@ -827,8 +899,8 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     if (check_col(fd, COL_PROTOCOL))
       col_add_str(fd, COL_PROTOCOL, "IP");
     if (check_col(fd, COL_INFO))
-      col_add_fstr(fd, COL_INFO, "Fragmented IP protocol (proto=%02x, off=%d)",
-	iph.ip_p, iph.ip_off & IP_OFFSET);
+      col_add_fstr(fd, COL_INFO, "Fragmented IP protocol (proto=%s 0x%02x, off=%d)",
+	ipprotostr(iph.ip_p), iph.ip_p, iph.ip_off & IP_OFFSET);
     dissect_data(pd, offset, fd, tree);
     return;
   }
@@ -841,18 +913,21 @@ again:
     case IP_PROTO_IGMP:
       dissect_igmp(pd, offset, fd, tree);
      break;
+    case IP_PROTO_EIGRP:
+      dissect_eigrp(pd, offset, fd, tree);
+      break;  
     case IP_PROTO_TCP:
       dissect_tcp(pd, offset, fd, tree);
-     break;
-   case IP_PROTO_UDP:
+      break;
+    case IP_PROTO_UDP:
       dissect_udp(pd, offset, fd, tree);
       break;
     case IP_PROTO_OSPF:
       dissect_ospf(pd, offset, fd, tree);
-     break;
+      break;
     case IP_PROTO_RSVP:
       dissect_rsvp(pd, offset, fd, tree);
-     break;
+      break;
     case IP_PROTO_AH:
       advance = dissect_ah(pd, offset, fd, tree);
       nxt = pd[offset];
@@ -867,6 +942,15 @@ again:
     case IP_PROTO_IPV6:
       dissect_ipv6(pd, offset, fd, tree);
       break;
+    case IP_PROTO_PIM:
+      dissect_pim(pd, offset, fd, tree);
+      break;
+    case IP_PROTO_IPCOMP:
+      dissect_ipcomp(pd, offset, fd, tree);
+      break;
+    case IP_PROTO_VRRP:
+      dissect_vrrp(pd, offset, fd, tree);
+      break;
     default:
       dissect_data(pd, offset, fd, tree);
       break;
@@ -1002,13 +1086,17 @@ dissect_icmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
   if (tree) {
     ti = proto_tree_add_item(tree, proto_icmp, offset, 4, NULL);
-    icmp_tree = proto_item_add_subtree(ti, ETT_ICMP);
-    proto_tree_add_text(icmp_tree, offset,      1, "Type: %d (%s)",
-      ih.icmp_type, type_str);
-    proto_tree_add_text(icmp_tree, offset +  1, 1, "Code: %d %s",
-      ih.icmp_code, code_str);
-    proto_tree_add_text(icmp_tree, offset +  2, 2, "Checksum: 0x%04x",
-      cksum);
+    icmp_tree = proto_item_add_subtree(ti, ett_icmp);
+    proto_tree_add_item_format(icmp_tree, hf_icmp_type, offset,      1, 
+			       ih.icmp_type,
+			       "Type: %d (%s)",
+			       ih.icmp_type, type_str);
+    proto_tree_add_item_format(icmp_tree, hf_icmp_code,	offset +  1, 1, 
+			       ih.icmp_code,
+			       "Code: %d %s",
+			       ih.icmp_code, code_str);
+    proto_tree_add_item(icmp_tree, hf_icmp_checksum, offset +  2, 2, 
+			cksum);
 
     /* Decode the second 4 bytes of the packet. */
     switch (ih.icmp_type) {
@@ -1139,7 +1227,7 @@ dissect_igmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       strcpy(type_str, "PIM");
       break;
     case IGMP_V2_M_RPT:
-      strcpy(type_str, "Host reponse (v2)");
+      strcpy(type_str, "Host response (v2)");
       break;
     case IGMP_MTRC_RESP:
       strcpy(type_str, "Traceroute response");
@@ -1157,17 +1245,21 @@ dissect_igmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     col_add_str(fd, COL_INFO, type_str);
   if (tree) {
     ti = proto_tree_add_item(tree, proto_igmp, offset, 8, NULL);
-    igmp_tree = proto_item_add_subtree(ti, ETT_IGMP);
-    proto_tree_add_text(igmp_tree, offset,     1, "Version: %d",
-      hi_nibble(ih.igmp_v_t));
-    proto_tree_add_text(igmp_tree, offset    , 1, "Type: %d (%s)",
-      lo_nibble(ih.igmp_v_t), type_str);
-    proto_tree_add_text(igmp_tree, offset + 1, 1, "Unused: 0x%02x",
-      ih.igmp_unused);
-    proto_tree_add_text(igmp_tree, offset + 2, 2, "Checksum: 0x%04x",
-      cksum);
-    proto_tree_add_text(igmp_tree, offset + 4, 4, "Group address: %s",
-      ip_to_str((guint8 *) &ih.igmp_gaddr));
+    igmp_tree = proto_item_add_subtree(ti, ett_igmp);
+    proto_tree_add_item(igmp_tree, hf_igmp_version, offset,     1, 
+			hi_nibble(ih.igmp_v_t));
+    proto_tree_add_item_format(igmp_tree, hf_igmp_type,  offset    , 1, 
+			       lo_nibble(ih.igmp_v_t),
+			       "Type: %d (%s)",
+			       lo_nibble(ih.igmp_v_t), type_str);
+    proto_tree_add_item_format(igmp_tree, hf_igmp_unused, offset + 1, 1,
+			       ih.igmp_unused,
+			       "Unused: 0x%02x",
+			       ih.igmp_unused);
+    proto_tree_add_item(igmp_tree, hf_igmp_checksum, offset + 2, 2, 
+			cksum);
+    proto_tree_add_item(igmp_tree, hf_igmp_group, offset + 4, 4, 
+			ih.igmp_gaddr);
   }
 }
 
@@ -1177,23 +1269,32 @@ proto_register_igmp(void)
 	static hf_register_info hf[] = {
 
 		{ &hf_igmp_version,
-		{ "Version",		"igmp.version", FT_UINT8, NULL }},
+		{ "Version",		"igmp.version", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_igmp_type,
-		{ "Type",		"igmp.type", FT_UINT8, NULL }},
+		{ "Type",		"igmp.type", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_igmp_unused,
-		{ "Unused",		"igmp.unused", FT_UINT8, NULL }},
+		{ "Unused",		"igmp.unused", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_igmp_checksum,
-		{ "Checksum",		"igmp.checksum", FT_UINT16, NULL }},
+		{ "Checksum",		"igmp.checksum", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
 
 		{ &hf_igmp_group,
-		{ "Group address",	"igmp.group", FT_IPv4, NULL }}
+		{ "Group address",	"igmp.group", FT_IPv4, BASE_NONE, NULL, 0x0,
+			"" }},
+	};
+	static gint *ett[] = {
+		&ett_igmp,
 	};
 
 	proto_igmp = proto_register_protocol ("Internet Group Management Protocol", "igmp");
 	proto_register_field_array(proto_igmp, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
 
 void
@@ -1202,54 +1303,185 @@ proto_register_ip(void)
 	static hf_register_info hf[] = {
 
 		{ &hf_ip_version,
-		{ "Version",		"ip.version", FT_UINT8, NULL }},
+		{ "Version",		"ip.version", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_hdr_len,
-		{ "Header Length",	"ip.hdr_len", FT_UINT8, NULL }},
+		{ "Header Length",	"ip.hdr_len", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_tos,
-		{ "Type of Service",	"ip.tos", FT_UINT8, NULL }},
+		{ "Type of Service",	"ip.tos", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_tos_precedence,
-		{ "Precedence",		"ip.tos.precedence", FT_VALS_UINT8, VALS(precedence_vals) }},
+		{ "Precedence",		"ip.tos.precedence", FT_UINT8, BASE_DEC, VALS(precedence_vals),
+			IPTOS_PREC_MASK,
+			"" }},
+
+		{ &hf_ip_tos_delay,
+		{ "Delay",		"ip.tos.delay", FT_BOOLEAN, 8, TFS(&tos_set_low),
+			IPTOS_LOWDELAY,
+			"" }},
+
+		{ &hf_ip_tos_throughput,
+		{ "Throughput",		"ip.tos.throughput", FT_BOOLEAN, 8, TFS(&tos_set_high),
+			IPTOS_THROUGHPUT,
+			"" }},
+
+		{ &hf_ip_tos_reliability,
+		{ "Reliability",	"ip.tos.reliability", FT_BOOLEAN, 8, TFS(&tos_set_high),
+			IPTOS_RELIABILITY,
+			"" }},
+
+		{ &hf_ip_tos_cost,
+		{ "Cost",		"ip.tos.cost", FT_BOOLEAN, 8, TFS(&tos_set_low),
+			IPTOS_LOWCOST,
+			"" }},
 
 		{ &hf_ip_len,
-		{ "Total Length",	"ip.len", FT_UINT16 }},
+		{ "Total Length",	"ip.len", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_id,
-		{ "Identification",	"ip.id", FT_UINT32 }},
+		{ "Identification",	"ip.id", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_dst,
-		{ "Destination",	"ip.dst", FT_IPv4, NULL }},
+		{ "Destination",	"ip.dst", FT_IPv4, BASE_NONE, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_src,
-		{ "Source",		"ip.src", FT_IPv4, NULL }},
+		{ "Source",		"ip.src", FT_IPv4, BASE_NONE, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_addr,
-		{ "Source or Destination Address", "ip.addr", FT_IPv4, NULL }},
+		{ "Source or Destination Address", "ip.addr", FT_IPv4, BASE_NONE, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_flags,
-		{ "Flags",		"ip.flags", FT_UINT8, NULL }},
+		{ "Flags",		"ip.flags", FT_UINT8, BASE_HEX, NULL, 0x0,
+			"" }},
+
+		{ &hf_ip_flags_df,
+		{ "Don't fragment",	"ip.flags.df", FT_BOOLEAN, 4, TFS(&flags_set_truth), IP_DF>>12,
+			"" }},
+
+		{ &hf_ip_flags_mf,
+		{ "More fragments",	"ip.flags.mf", FT_BOOLEAN, 4, TFS(&flags_set_truth), IP_MF>>12,
+			"" }},
 
 		{ &hf_ip_frag_offset,
-		{ "Fragment offset",	"ip.frag_offset", FT_UINT16, NULL }},
+		{ "Fragment offset",	"ip.frag_offset", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_ttl,
-		{ "Time to live",	"ip.ttl", FT_UINT8, NULL }},
+		{ "Time to live",	"ip.ttl", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_proto,
-		{ "Protocol",		"ip.proto", FT_VALS_UINT8, VALS(proto_vals) }},
+		{ "Protocol",		"ip.proto", FT_UINT8, BASE_HEX, NULL, 0x0,
+			"" }},
 
 		{ &hf_ip_checksum,
-		{ "Header checksum",	"ip.checksum", FT_UINT16, NULL }}
+		{ "Header checksum",	"ip.checksum", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
+	};
+	static gint *ett[] = {
+		&ett_ip,
+		&ett_ip_tos,
+		&ett_ip_off,
+		&ett_ip_options,
+		&ett_ip_option_sec,
+		&ett_ip_option_route,
+		&ett_ip_option_timestamp,
 	};
 
 	proto_ip = proto_register_protocol ("Internet Protocol", "ip");
 	proto_register_field_array(proto_ip, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
 
 void
 proto_register_icmp(void)
 {
-	proto_icmp = proto_register_protocol ("Internet Control Message Protocol", "icmp");
+  static hf_register_info hf[] = {
+    
+    { &hf_icmp_type,
+      { "Type",		"icmp.type",		FT_UINT8, BASE_DEC,	NULL, 0x0,
+      	"" }},
+
+    { &hf_icmp_code,
+      { "Code",		"icmp.code",		FT_UINT8, BASE_HEX,	NULL, 0x0,
+      	"" }},    
+
+    { &hf_icmp_checksum,
+      { "Checksum",	"icmp.checksum",	FT_UINT16, BASE_HEX,	NULL, 0x0,
+      	"" }},
+  };
+  static gint *ett[] = {
+    &ett_icmp,
+  };
+  
+  proto_icmp = proto_register_protocol ("Internet Control Message Protocol", 
+					"icmp");
+  proto_register_field_array(proto_icmp, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
 }
+
+static int proto_eigrp = -1;
+
+static gint ett_eigrp = -1;
+
+void
+dissect_eigrp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
+  e_eigrp     ih;
+  proto_tree *eigrp_tree;
+  proto_item *ti;
+  guint16    cksum;
+  gchar      type_str[64] = "";
+
+  /* Avoids alignment problems on many architectures. */
+static const value_string eigrp_opcode_vals[] = {
+	{ EIGRP_HELLO,		"Hello/Ack" },
+	{ EIGRP_UPDATE,		"Update" },
+   	{ EIGRP_REPLY, 		"Reply" },
+   	{ EIGRP_QUERY, 		"Query" },
+	{ EIGRP_REQUEST,	"Request" },
+	{ 0,				NULL }    
+};
+  
+   memcpy(&ih, &pd[offset], sizeof(e_eigrp));
+  /* To do: check for runts, errs, etc. */
+  cksum = ntohs(ih.eigrp_checksum);
+  
+  if (check_col(fd, COL_PROTOCOL))
+    col_add_str(fd, COL_PROTOCOL, "EIGRP");
+  if (check_col(fd, COL_INFO))
+    col_add_str(fd, COL_INFO, type_str);
+  if (tree) {
+
+     ti = proto_tree_add_item(tree, proto_eigrp, offset, END_OF_FRAME, NULL);
+     eigrp_tree = proto_item_add_subtree(ti, ett_eigrp);
+  
+     proto_tree_add_text(eigrp_tree, offset, 1, "Version: %d", ih.eigrp_version); 
+     proto_tree_add_text(eigrp_tree, offset + 1, 1, "Opcode: %d (%s)", ih.eigrp_opcode,
+         val_to_str( ih.eigrp_opcode, eigrp_opcode_vals, "Unknown") );
+     proto_tree_add_text(eigrp_tree, offset + 2, 2, "Checksum: 0x%x", cksum); 
+     proto_tree_add_text(eigrp_tree, offset + 4, 2, "Subnets in local net: %d", ih.eigrp_subnets); 
+     proto_tree_add_text(eigrp_tree, offset + 6, 2, "Networks in Autonomous System: %d", ih.eigrp_networks); 
+     proto_tree_add_text(eigrp_tree, offset + 8, 4, "Sequence Number: 0x%x", ih.eigrp_sequence); 
+     proto_tree_add_text(eigrp_tree, offset + 12, 4, "Autonomous System number: %ld", ih.eigrp_asnumber); 
+   }
+}
+
+
+void
+proto_register_eigrp(void)
+   {
+      static gint *ett[] = {
+        &ett_eigrp,
+      };
+   proto_eigrp = proto_register_protocol("Enhanced Interior Gateway Routing Protocol", "eigrp");
+   proto_register_subtree_array(ett, array_length(ett));
+   }
