@@ -1,7 +1,7 @@
 /* packet-ipv6.h
  * Definitions for IPv6 packet disassembly 
  *
- * $Id: packet-ipv6.h,v 1.5 1999/09/10 07:19:40 guy Exp $
+ * $Id: packet-ipv6.h,v 1.9 1999/11/18 01:45:05 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -86,8 +86,15 @@ struct ip6_hdr {
 #define ip6_hlim	ip6_ctlun.ip6_un1.ip6_un1_hlim
 #define ip6_hops	ip6_ctlun.ip6_un1.ip6_un1_hlim
 
-#define IPV6_VERSION		0x60
-#define IPV6_VERSION_MASK	0xf0
+/* Offsets of fields within an IPv6 header. */
+#define	IP6H_CTL	0
+#define	IP6H_CTL_FLOW	0
+#define	IP6H_CTL_PLEN	4
+#define	IP6H_CTL_NXT	6
+#define	IP6H_CTL_HLIM	7
+#define	IP6H_CTL_VFC	0
+#define	IP6H_SRC	8
+#define	IP6H_DST	24
 
 #if BYTE_ORDER == BIG_ENDIAN
 #define IPV6_FLOWINFO_MASK	0x0fffffff	/* flow info (28 bits) */
@@ -410,12 +417,12 @@ struct icmp6_namelookup {
 struct icmp6_router_renum {	/* router renumbering header */
 	struct icmp6_hdr	rr_hdr;
 	guint8		rr_segnum;
-	guint8		rr_test : 1;
-	guint8		rr_reqresult : 1;
-	guint8		rr_forceapply : 1;
-	guint8		rr_specsite : 1;
-	guint8		rr_prevdone : 1;
-	guint8		rr_flags_reserved : 3;
+	guint32		rr_test : 1;
+	guint32		rr_reqresult : 1;
+	guint32		rr_forceapply : 1;
+	guint32		rr_specsite : 1;
+	guint32		rr_prevdone : 1;
+	guint32		rr_flags_reserved : 3;
 	guint16		rr_maxdelay;
 	guint32		rr_reserved;
 };
@@ -423,12 +430,12 @@ struct icmp6_router_renum {	/* router renumbering header */
 struct icmp6_router_renum {	/* router renumbering header */
 	struct icmp6_hdr	rr_hdr;
 	guint8		rr_segnum;
-	guint8		rr_flags_reserved : 3;
-	guint8		rr_prevdone : 1;
-	guint8		rr_specsite : 1;
-	guint8		rr_forceapply : 1;
-	guint8		rr_reqresult : 1;
-	guint8		rr_test : 1;
+	guint32		rr_flags_reserved : 3;
+	guint32		rr_prevdone : 1;
+	guint32		rr_specsite : 1;
+	guint32		rr_forceapply : 1;
+	guint32		rr_reqresult : 1;
+	guint32		rr_test : 1;
 	guint16		rr_maxdelay;
 	guint32		rr_reserved;
 };
@@ -459,12 +466,12 @@ struct rr_pco_match {		/* match prefix part */
 struct rr_pco_use {		/* use prefix part */
 	guint8	rpu_uselen;
 	guint8	rpu_keeplen;
-	guint8	rpu_mask_onlink : 1;
-	guint8	rpu_mask_autonomous : 1;
-	guint8	rpu_mask_reserved : 6;
-	guint8	rpu_onlink : 1;
-	guint8	rpu_autonomous : 1;
-	guint8	rpu_raflags_reserved : 6;
+	guint32	rpu_mask_onlink : 1;
+	guint32	rpu_mask_autonomous : 1;
+	guint32	rpu_mask_reserved : 6;
+	guint32	rpu_onlink : 1;
+	guint32	rpu_autonomous : 1;
+	guint32	rpu_raflags_reserved : 6;
 	guint32	rpu_vltime;
 	guint32	rpu_pltime;
 	guint32	rpu_decr_vltime : 1;
@@ -477,12 +484,12 @@ struct rr_pco_use {		/* use prefix part */
 struct rr_pco_use {		/* use prefix part */
 	guint8	rpu_uselen;
 	guint8	rpu_keeplen;
-	guint8	rpu_mask_reserved : 6;
-	guint8	rpu_mask_autonomous : 1;
-	guint8	rpu_mask_onlink : 1;
-	guint8	rpu_raflags_reserved : 6;
-	guint8	rpu_autonomous : 1;
-	guint8	rpu_onlink : 1;
+	guint32	rpu_mask_reserved : 6;
+	guint32	rpu_mask_autonomous : 1;
+	guint32	rpu_mask_onlink : 1;
+	guint32	rpu_raflags_reserved : 6;
+	guint32	rpu_autonomous : 1;
+	guint32	rpu_onlink : 1;
 	guint32	rpu_vltime;
 	guint32	rpu_pltime;
 	guint32	rpu_flags_reserved : 6;
@@ -496,9 +503,9 @@ struct rr_pco_use {		/* use prefix part */
 #if BYTE_ORDER == BIG_ENDIAN /* net byte order */
 struct rr_result {		/* router renumbering result message */
 	guint8	rrr_reserved;
-	guint8	rrr_flags_reserved : 6;
-	guint8	rrr_outofbound : 1;
-	guint8	rrr_forbidden : 1;
+	guint32	rrr_flags_reserved : 6;
+	guint32	rrr_outofbound : 1;
+	guint32	rrr_forbidden : 1;
 	guint8	rrr_ordinal;
 	guint8	rrr_matchedlen;
 	guint32	rrr_ifid;
@@ -507,9 +514,9 @@ struct rr_result {		/* router renumbering result message */
 #elif BYTE_ORDER == LITTLE_ENDIAN
 struct rr_result {		/* router renumbering result message */
 	guint8	rrr_reserved;
-	guint8	rrr_forbidden : 1;
-	guint8	rrr_outofbound : 1;
-	guint8	rrr_flags_reserved : 6;
+	guint32	rrr_forbidden : 1;
+	guint32	rrr_outofbound : 1;
+	guint32	rrr_flags_reserved : 6;
 	guint8	rrr_ordinal;
 	guint8	rrr_matchedlen;
 	guint32	rrr_ifid;
@@ -517,7 +524,4 @@ struct rr_result {		/* router renumbering result message */
 };
 #endif /* BYTE_ORDER */
 
-struct e_in6_addr;
-gchar*     ip6_to_str(struct e_in6_addr *);
-
 #endif /* __PACKET_IPV6_H_DEFINED__ */
