@@ -2,7 +2,7 @@
  * Routines for NetWare's IPX
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
- * $Id: packet-ipx.c,v 1.27 1999/09/15 22:33:17 gram Exp $
+ * $Id: packet-ipx.c,v 1.36 1999/11/22 06:03:45 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -37,6 +37,7 @@
 #include "packet.h"
 #include "packet-ipx.h"
 #include "packet-ncp.h"
+#include "resolv.h"
 
 /* The information in this module (IPX, SPX, NCP) comes from:
 	NetWare LAN Analysis, Second Edition
@@ -61,9 +62,31 @@ static int hf_ipx_snet = -1;
 static int hf_ipx_snode = -1;
 static int hf_ipx_ssocket = -1;
 
+static gint ett_ipx = -1;
+
 static int proto_spx = -1;
+static int hf_spx_connection_control = -1;
+static int hf_spx_datastream_type = -1;
+static int hf_spx_src_id = -1;
+static int hf_spx_dst_id = -1;
+static int hf_spx_seq_nr = -1;
+static int hf_spx_ack_nr = -1;
+static int hf_spx_all_nr = -1;
+
+static gint ett_spx = -1;
+
 static int proto_ipxrip = -1;
+static int hf_ipxrip_request = -1;
+static int hf_ipxrip_response = -1;
+
+static gint ett_ipxrip = -1;
+
 static int proto_sap = -1;
+static int hf_sap_request = -1;
+static int hf_sap_response = -1;
+
+static gint ett_ipxsap = -1;
+static gint ett_ipxsap_server = -1;
 
 static void
 dissect_spx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
@@ -72,7 +95,7 @@ static void
 dissect_ipxrip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
 
 static void
-dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
+dissect_ipxsap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
 
 typedef	void	(dissect_func_t)(const u_char *, int, frame_data *, proto_tree *);
 
@@ -104,13 +127,20 @@ struct server_info {
 #define IPX_SOCKET_SERIALIZATION	0x0457
 #define IPX_SOCKET_NWLINK_SMB_NAMEQUERY	0x0551
 #define IPX_SOCKET_NWLINK_SMB_DGRAM	0x0553
+#define IPX_SOCKET_NWLINK_SMB_BROWSE	0x0555 /* ? not sure on this
+	but I guessed based on the content of the packet I saw */
 #define IPX_SOCKET_ATTACHMATE_GW	0x055d
 #define IPX_SOCKET_IPX_MESSAGE		0x4001
+#define IPX_SOCKET_ADSM                 0x8522 /* www.tivoli.com */
+#define IPX_SOCKET_SNMP_AGENT           0x900F /* RFC 1906 */
+#define IPX_SOCKET_SNMP_SINK            0x9010 /* RFC 1906 */
+#define IPX_SOCKET_TCP_TUNNEL           0x9091 /* RFC 1791 */
+#define IPX_SOCKET_UDP_TUNNEL           0x9092 /* RFC 1791 */
 
 static struct port_info	ports[] = {
 	{ IPX_SOCKET_NCP,			dissect_ncp,
 				"NCP" },
-	{ IPX_SOCKET_SAP,			dissect_sap,
+	{ IPX_SOCKET_SAP,			dissect_ipxsap,
 				"SAP" },
 	{ IPX_SOCKET_IPXRIP,			dissect_ipxrip,
 				"RIP" },
@@ -124,10 +154,18 @@ static struct port_info	ports[] = {
 				"NWLink SMB Name Query" },
 	{ IPX_SOCKET_NWLINK_SMB_DGRAM,		dissect_nwlink_dg,
 				"NWLink SMB Datagram" },
+	{ IPX_SOCKET_NWLINK_SMB_BROWSE,	NULL,
+				"NWLink SMB Browse" },
 	{ IPX_SOCKET_ATTACHMATE_GW,		NULL,
 				"Attachmate Gateway" },
 	{ IPX_SOCKET_IPX_MESSAGE,		NULL,
 				"IPX Message" },
+	{ IPX_SOCKET_SNMP_AGENT, NULL, "SNMP Agent" },
+	{ IPX_SOCKET_SNMP_SINK,	NULL, "SNMP Sink" },
+	{ IPX_SOCKET_UDP_TUNNEL, NULL, "UDP Tunnel" },
+	{ IPX_SOCKET_TCP_TUNNEL, NULL, "TCP Tunnel" },
+	{ IPX_SOCKET_TCP_TUNNEL, NULL, "TCP Tunnel" },
+	{ IPX_SOCKET_ADSM, NULL, "ADSM" },
 	{ 0x0000,				NULL,
 				NULL }
 };
@@ -192,26 +230,21 @@ static const value_string ipx_packet_type_vals[] = {
 gchar*
 ipxnet_to_string(const guint8 *ad)
 {
-	static gchar	str[3][12];
-	static gchar	*cur;
-
-	if (cur == &str[0][0]) {
-		cur = &str[1][0];
-	} else if (cur == &str[1][0]) {
-		cur = &str[2][0];
-	} else {
-		cur = &str[0][0];
-	}
-
-	sprintf(cur, "%02X %02X %02X %02X", ad[0], ad[1], ad[2], ad[3]);
-	return cur;
+	guint32	addr = pntohl(ad);
+	return ipxnet_to_str_punct(addr, ' ');
 }
 
+/* We use a different representation of hardware addresses
+ * than ether_to_str(); we don't put punctuation between the hex
+ * digits.
+ */
+
 gchar*
 ipx_addr_to_str(guint32 net, const guint8 *ad)
 {
-	static gchar	str[3][22];
+	static gchar	str[3][8+1+MAXNAMELEN+1]; /* 8 digits, 1 period, NAME, 1 null */
 	static gchar	*cur;
+	char		*name;
 
 	if (cur == &str[0][0]) {
 		cur = &str[1][0];
@@ -221,11 +254,52 @@ ipx_addr_to_str(guint32 net, const guint8 *ad)
 		cur = &str[0][0];
 	}
 
-	sprintf(cur, "%X.%02x%02x%02x%02x%02x%02x", net, 
-		ad[0], ad[1], ad[2], ad[3], ad[4], ad[5]);
+	name = get_ether_name_if_known(ad);
+
+	if (name) {
+		sprintf(cur, "%s.%s", get_ipxnet_name(net), name);
+	}
+	else {
+		sprintf(cur, "%s.%s", get_ipxnet_name(net), ether_to_str_punct(ad, '\0'));
+	}
 	return cur;
 }
 
+gchar *
+ipxnet_to_str_punct(const guint32 ad, char punct) {
+  static gchar  str[3][12];
+  static gchar *cur;
+  gchar        *p;
+  int          i;
+  guint32      octet;
+  static const gchar hex_digits[16] = "0123456789ABCDEF";
+  static const guint32  octet_mask[4] =
+	  { 0xff000000 , 0x00ff0000, 0x0000ff00, 0x000000ff };
+
+  if (cur == &str[0][0]) {
+    cur = &str[1][0];
+  } else if (cur == &str[1][0]) {  
+    cur = &str[2][0];
+  } else {  
+    cur = &str[0][0];
+  }
+  p = &cur[12];
+  *--p = '\0';
+  i = 3;
+  for (;;) {
+    octet = (ad & octet_mask[i]) >> ((3 - i) * 8);
+    *--p = hex_digits[octet&0xF];
+    octet >>= 4;
+    *--p = hex_digits[octet&0xF];
+    if (i == 0)
+      break;
+    if (punct)
+      *--p = punct;
+    i--;
+  }
+  return p;
+}
+
 void
 dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
@@ -236,7 +310,6 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	int		len;
 	guint8		*ipx_snode, *ipx_dnode, *ipx_snet, *ipx_dnet;
 
-	gchar		*str_dnet, *str_snet;
 	guint16		ipx_dsocket, ipx_ssocket;
 	dissect_func_t	*dissect;
 	guint32		ipx_dnet_val, ipx_snet_val;
@@ -244,8 +317,6 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	/* Calculate here for use in pinfo and in tree */
 	ipx_dnet = (guint8*)&pd[offset+6];
 	ipx_snet = (guint8*)&pd[offset+18];
-	str_dnet = ipxnet_to_string(ipx_dnet);
-	str_snet = ipxnet_to_string(ipx_snet);
 	ipx_dnet_val = pntohl(ipx_dnet);
 	ipx_snet_val = pntohl(ipx_snet);
 	ipx_dsocket = pntohs(&pd[offset+16]);
@@ -256,7 +327,7 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	ipx_length = pntohs(&pd[offset+2]);
 
 	/* Length of IPX datagram plus headers above it. */
-		len = ipx_length + offset;
+	len = ipx_length + offset;
 
 	/* Set the payload and captured-payload lengths to the minima of
 	   (the IPX length plus the length of the headers above it) and
@@ -266,12 +337,10 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	if (pi.captured_len > len)
 		pi.captured_len = len;
 
-	if (check_col(fd, COL_RES_DL_DST))
-		col_add_str(fd, COL_RES_DL_DST,
-				ipx_addr_to_str(pntohl(ipx_dnet), ipx_dnode));
-	if (check_col(fd, COL_RES_DL_SRC))
-		col_add_str(fd, COL_RES_DL_SRC,
-				ipx_addr_to_str(pntohl(ipx_snet), ipx_snode));
+	SET_ADDRESS(&pi.net_src, AT_IPX, 10, &pd[offset+18]);
+	SET_ADDRESS(&pi.src, AT_IPX, 10, &pd[offset+18]);
+	SET_ADDRESS(&pi.net_dst, AT_IPX, 10, &pd[offset+6]);
+	SET_ADDRESS(&pi.dst, AT_IPX, 10, &pd[offset+6]);
 
 	if (check_col(fd, COL_PROTOCOL))
 		col_add_str(fd, COL_PROTOCOL, "IPX");
@@ -284,9 +353,8 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 		ipx_hops = pd[offset+4];
 
 		ti = proto_tree_add_item(tree, proto_ipx, offset, 30, NULL);
-		ipx_tree = proto_item_add_subtree(ti, ETT_IPX);
-		proto_tree_add_item_format(ipx_tree, hf_ipx_checksum, offset, 2, ipx_checksum,
-			"Checksum: 0x%04x", ipx_checksum);
+		ipx_tree = proto_item_add_subtree(ti, ett_ipx);
+		proto_tree_add_item(ipx_tree, hf_ipx_checksum, offset, 2, ipx_checksum);
 		proto_tree_add_item_format(ipx_tree, hf_ipx_len, offset+2, 2, ipx_length,
 			"Length: %d bytes", ipx_length);
 		proto_tree_add_item_format(ipx_tree, hf_ipx_hops, offset+4, 1, ipx_hops,
@@ -366,7 +434,8 @@ spx_conn_ctrl(u_char ctrl)
 		{ 0x10, "End-of-Message" },
 		{ 0x20, "Attention" },
 		{ 0x40, "Acknowledgment Required"},
-		{ 0x80, "System Packet"}
+		{ 0x80, "System Packet"},
+		{ 0x00, NULL }
 	};
 
 	while (conns[i].text != NULL) {
@@ -403,30 +472,41 @@ dissect_spx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_spx, offset, 12, NULL);
-		spx_tree = proto_item_add_subtree(ti, ETT_SPX);
+		spx_tree = proto_item_add_subtree(ti, ett_spx);
 
-		proto_tree_add_text(spx_tree, offset,      1,
-			"Connection Control: %s (0x%02X)",
-			spx_conn_ctrl(pd[offset]), pd[offset]);
+		proto_tree_add_item_format(spx_tree, hf_spx_connection_control,
+					   offset,      1,
+					   pd[offset],
+					   "Connection Control: %s (0x%02X)",
+					   spx_conn_ctrl(pd[offset]), 
+					   pd[offset]);
 
-		proto_tree_add_text(spx_tree, offset+1,     1,
-			"Datastream Type: %s (0x%02X)",
-			spx_datastream(pd[offset+1]), pd[offset+1]);
+		proto_tree_add_item_format(spx_tree, hf_spx_datastream_type,
+					   offset+1,     1,
+					   pd[offset+1],
+					   "Datastream Type: %s (0x%02X)",
+					   spx_datastream(pd[offset+1]), 
+					   pd[offset+1]);
 
-		proto_tree_add_text(spx_tree, offset+2,     2,
-			"Source Connection ID: %d", pntohs( &pd[offset+2] ) );
+		proto_tree_add_item(spx_tree, hf_spx_src_id, 
+				    offset+2,     2,
+				    pntohs( &pd[offset+2] ));
 
-		proto_tree_add_text(spx_tree, offset+4,     2,
-			"Destination Connection ID: %d", pntohs( &pd[offset+4] ) );
+		proto_tree_add_item(spx_tree, hf_spx_dst_id,
+				    offset+4,     2,
+				    pntohs( &pd[offset+4] ));
 
-		proto_tree_add_text(spx_tree, offset+6,     2,
-			"Sequence Number: %d", pntohs( &pd[offset+6] ) );
+		proto_tree_add_item(spx_tree, hf_spx_seq_nr, 
+				    offset+6,     2,
+				    pntohs( &pd[offset+6] ) );
 
-		proto_tree_add_text(spx_tree, offset+8,     2,
-			"Acknowledgment Number: %d", pntohs( &pd[offset+8] ) );
+		proto_tree_add_item(spx_tree, hf_spx_ack_nr,
+				    offset+8,     2,
+				    pntohs( &pd[offset+8] ) );
 
-		proto_tree_add_text(spx_tree, offset+10,     2,
-			"Allocation Number: %d", pntohs( &pd[offset+10] ) );
+		proto_tree_add_item(spx_tree, hf_spx_all_nr,
+				    offset+10,     2,
+				    pntohs( &pd[offset+10] ) );
 
 		offset += 12;
 		dissect_data(pd, offset, fd, tree);
@@ -461,11 +541,22 @@ dissect_ipxrip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_ipxrip, offset, END_OF_FRAME, NULL);
-		rip_tree = proto_item_add_subtree(ti, ETT_IPXRIP);
+		rip_tree = proto_item_add_subtree(ti, ett_ipxrip);
 
 		if (operation < 2) {
 			proto_tree_add_text(rip_tree, offset, 2,
 			"RIP packet type: %s", rip_type[operation]);
+
+			if (operation == 0) {
+			  proto_tree_add_item_hidden(rip_tree, 
+						     hf_ipxrip_request, 
+						     offset, 2, 1);
+			} else {
+			  proto_tree_add_item_hidden(rip_tree, 
+						     hf_ipxrip_response, 
+						     offset, 2, 1);
+			}
+
 		}
 		else {
 			proto_tree_add_text(rip_tree, offset, 2, "Unknown RIP packet type");
@@ -536,10 +627,28 @@ server_type(guint16 type)
 		{ 0x0111,	"Test server" },
 		{ 0x0133,	"NetWare Name Service" },
 		{ 0x0166,	"NetWare management" },
+		{ 0x023f,	"SMS Testing and Development" },
 		{ 0x026a,	"NetWare management" },
 		{ 0x026b,	"Time synchronization" },
+		{ 0x027b,	"NetWare Management Agent" },
 		{ 0x0278,	"NetWare Directory server" },
+		{ 0x030c,	"HP LaserJet / Quick Silver" },
+		{ 0x0355,	"Arcada Software" },
+		{ 0x0361,	"NETINELO" },
+		{ 0x037e,	"Powerchute UPS Monitoring" },
+		{ 0x03e1,	"UnixWare Application Server" },
+		{ 0x044c,	"Archive" },
 		{ 0x055d,	"Attachmate SNA gateway" },
+		{ 0x0610,	"Adaptec SCSI Management" },
+		{ 0x0640,	"NT Server-RPC/GW for NW/Win95 User Level Sec" },
+		{ 0x064e,	"NT Server-IIS" },
+		{ 0x0810,	"ELAN License Server Demo" },
+		{ 0x8002,	"Intel NetPort Print Server" },
+
+/* For any unidentified ones, I found a really big list of them at: */
+/*    http://www.inpnet.org/cnpweb/saplist.txt */
+/*    http://www.isi.edu/in-notes/iana/assignments/novell-sap-numbers */
+
 		{ 0x0000,	NULL }
 	};
 
@@ -553,7 +662,7 @@ server_type(guint16 type)
 }
 
 static void
-dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
+dissect_ipxsap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	proto_tree	*sap_tree, *s_tree;
 	proto_item	*ti;
@@ -580,10 +689,19 @@ dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_sap, offset, END_OF_FRAME, NULL);
-		sap_tree = proto_item_add_subtree(ti, ETT_IPXSAP);
+		sap_tree = proto_item_add_subtree(ti, ett_ipxsap);
 
 		if (query.query_type >= 1 && query.query_type <= 4) {
 			proto_tree_add_text(sap_tree, offset, 2, sap_type[query.query_type - 1]);
+			if ((query.query_type - 1) % 2) {
+			  proto_tree_add_item_hidden(sap_tree, 
+						     hf_sap_response, 
+						     offset, 2, 1);
+			} else {
+			  proto_tree_add_item_hidden(sap_tree, 
+						     hf_sap_request, 
+						     offset, 2, 1);
+			}
 		}
 		else {
 			proto_tree_add_text(sap_tree, offset, 2,
@@ -603,7 +721,7 @@ dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 				ti = proto_tree_add_text(sap_tree, cursor+2, 48,
 					"Server Name: %s", server.server_name);
-				s_tree = proto_item_add_subtree(ti, ETT_IPXSAP_SERVER);
+				s_tree = proto_item_add_subtree(ti, ett_ipxsap_server);
 
 				proto_tree_add_text(s_tree, cursor, 2, "Server Type: %s (0x%04X)",
 						server_type(server.server_type), server.server_type);
@@ -631,42 +749,126 @@ proto_register_ipx(void)
 {
 	static hf_register_info hf_ipx[] = {
 		{ &hf_ipx_checksum,
-		{ "Checksum",		"ipx.checksum", FT_UINT16, NULL }},
+		{ "Checksum",		"ipx.checksum", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_len,
-		{ "Length",		"ipx.len", FT_UINT16, NULL }},
+		{ "Length",		"ipx.len", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_hops,
-		{ "Transport Control (Hops)", "ipx.hops", FT_UINT8, NULL }},
+		{ "Transport Control (Hops)", "ipx.hops", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_packet_type,
-		{ "Packet Type",	"ipx.packet_type", FT_VALS_UINT8, VALS(ipx_packet_type_vals) }},
+		{ "Packet Type",	"ipx.packet_type", FT_UINT8, BASE_HEX, VALS(ipx_packet_type_vals),
+			0x0,
+			"" }},
 
 		{ &hf_ipx_dnet,
-		{ "Destination Network","ipx.dstnet", FT_IPXNET, NULL }},
+		{ "Destination Network","ipx.dst.net", FT_IPXNET, BASE_NONE, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_dnode,
-		{ "Destination Node",	"ipx.dstnode", FT_ETHER, NULL }},
+		{ "Destination Node",	"ipx.dst.node", FT_ETHER, BASE_NONE, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_dsocket,
-		{ "Destination Socket",	"ipx.dstsocket", FT_UINT16, NULL }},
+		{ "Destination Socket",	"ipx.dst.socket", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_snet,
-		{ "Source Network","ipx.srcnet", FT_IPXNET, NULL }},
+		{ "Source Network","ipx.src.net", FT_IPXNET, BASE_NONE, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_snode,
-		{ "Source Node",	"ipx.srcnode", FT_ETHER, NULL }},
+		{ "Source Node",	"ipx.src.node", FT_ETHER, BASE_NONE, NULL, 0x0,
+			"" }},
 
 		{ &hf_ipx_ssocket,
-		{ "Source Socket",	"ipx.srcsocket", FT_UINT16, NULL }},
+		{ "Source Socket",	"ipx.src.socket", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
 	};
-		
+
+	static hf_register_info hf_spx[] = {
+		{ &hf_spx_connection_control,
+		{ "Connection Control",		"spx.ctl", 
+		  FT_UINT8,	BASE_HEX,	NULL,	0x0,
+		  "" }},
+
+		{ &hf_spx_datastream_type,
+		{ "Datastream type",	       	"spx.type", 
+		  FT_UINT8,	BASE_HEX,	NULL,	0x0,
+		  "" }},
+
+		{ &hf_spx_src_id,
+		{ "Source Connection ID",	"spx.src", 
+		  FT_UINT16,	BASE_DEC,	NULL,	0x0,
+		  "" }},
+
+		{ &hf_spx_dst_id,
+		{ "Destination Connection ID",	"spx.dst", 
+		  FT_UINT16,	BASE_DEC,	NULL,	0x0,
+		  "" }},
+
+		{ &hf_spx_seq_nr,
+		{ "Sequence Number",		"spx.seq", 
+		  FT_UINT16,	BASE_DEC,	NULL,	0x0,
+		  "" }},
+
+		{ &hf_spx_ack_nr,
+		{ "Acknowledgment Number",	"spx.ack", 
+		  FT_UINT16,	BASE_DEC,	NULL,	0x0,
+		  "" }},
+
+		{ &hf_spx_all_nr,
+		{ "Allocation Number",		"spx.alloc", 
+		  FT_UINT16,	BASE_DEC,	NULL,	0x0,
+		  "" }}
+	};
+
+	static hf_register_info hf_ipxrip[] = {
+		{ &hf_ipxrip_request,
+		{ "Request",			"ipxrip.request", 
+		  FT_BOOLEAN,	BASE_NONE,	NULL,	0x0,
+		  "TRUE if IPX RIP request" }},
+
+		{ &hf_ipxrip_response,
+		{ "Response",			"ipxrip.response", 
+		  FT_BOOLEAN,	BASE_NONE,	NULL,	0x0,
+		  "TRUE if IPX RIP response" }}
+	};
+
+	static hf_register_info hf_sap[] = {
+		{ &hf_sap_request,
+		{ "Request",			"sap.request", 
+		  FT_BOOLEAN,	BASE_NONE,	NULL,	0x0,
+		  "TRUE if SAP request" }},
+
+		{ &hf_sap_response,
+		{ "Response",			"sap.response", 
+		  FT_BOOLEAN,	BASE_NONE,	NULL,	0x0,
+		  "TRUE if SAP response" }}
+	};
+	static gint *ett[] = {
+		&ett_ipx,
+		&ett_spx,
+		&ett_ipxrip,
+		&ett_ipxsap,
+		&ett_ipxsap_server,
+	};
+
 	proto_ipx = proto_register_protocol ("Internetwork Packet eXchange", "ipx");
 	proto_register_field_array(proto_ipx, hf_ipx, array_length(hf_ipx));
 
 	proto_spx = proto_register_protocol ("Sequenced Packet eXchange", "spx");
-	
+	proto_register_field_array(proto_spx, hf_spx, array_length(hf_spx));
+
 	proto_ipxrip = proto_register_protocol ("IPX Routing Information Protocol", "ipxrip");
+	proto_register_field_array(proto_ipxrip, hf_ipxrip, array_length(hf_ipxrip));
 
 	proto_sap = proto_register_protocol ("Service Advertisement Protocol", "sap");
+	proto_register_field_array(proto_sap, hf_sap, array_length(hf_sap));
+
+	proto_register_subtree_array(ett, array_length(ett));
 }
