@@ -2,7 +2,7 @@
  * Routines for NetWare Core Protocol
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
- * $Id: packet-ncp.c,v 1.19 1999/09/02 23:17:57 guy Exp $
+ * $Id: packet-ncp.c,v 1.23 1999/11/18 01:45:02 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -44,6 +44,14 @@
 #include "packet-ncp.h"
 
 static int proto_ncp = -1;
+static int hf_ncp_type = -1;
+static int hf_ncp_seq = -1;
+static int hf_ncp_connection = -1;
+static int hf_ncp_task = -1;
+
+static gint ett_ncp = -1;
+static gint ett_ncp_request_fields = -1;
+static gint ett_ncp_reply_fields = -1;
 
 struct svc_record;
 
@@ -150,7 +158,7 @@ enum nfamily {
 		NCP_QUEUE_SERVICES,		/* print queues */
 		NCP_FILE_SERVICES,		/* file serving */
 		NCP_BINDERY_SERVICES,	/* bindery database */
-		NCP_CONNECTION_SERVICES, /* communication */
+		NCP_CONNECTION_SERVICES /* communication */
 };
 
 /* I had to put this function prototype after the enum nfamily declaration */
@@ -380,7 +388,7 @@ guint ncp_hash  (gconstpointer v)
 
 /* Initializes the hash table and the mem_chunk area each time a new
  * file is loaded or re-loaded in ethereal */
-void
+static void
 ncp_init_protocol(void)
 {
 	#if defined(DEBUG_NCP_HASH)
@@ -451,20 +459,24 @@ dissect_ncp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_ncp, offset, END_OF_FRAME, NULL);
-		ncp_tree = proto_item_add_subtree(ti, ETT_NCP);
+		ncp_tree = proto_item_add_subtree(ti, ett_ncp);
 
-		proto_tree_add_text(ncp_tree, offset,      2,
-			"Type: %s", val_to_str( header.type,
-			request_reply_values, "Unknown (%04X)"));
+		proto_tree_add_item_format(ncp_tree, hf_ncp_type, 
+					   offset,      2,
+					   header.type,
+					   "Type: %s", 
+					   val_to_str( header.type,
+						       request_reply_values,
+						       "Unknown (%04X)"));
 
-		proto_tree_add_text(ncp_tree, offset+2,    1,
-			"Sequence Number: %d", header.sequence);
+		proto_tree_add_item(ncp_tree, hf_ncp_seq, 
+				    offset+2,    1, header.sequence);
 
-		proto_tree_add_text(ncp_tree, offset+3,    3,
-			"Connection Number: %d", nw_connection);
+		proto_tree_add_item(ncp_tree, hf_ncp_connection,
+				    offset+3,    3, nw_connection);
 
-		proto_tree_add_text(ncp_tree, offset+4,    1,
-			"Task Number: %d", header.task);
+		proto_tree_add_item(ncp_tree, hf_ncp_task, 
+				    offset+4,    1, header.task);
 	}
 
 	/* Note how I use ncp_tree *and* tree in my args for ncp request/reply */
@@ -528,7 +540,7 @@ dissect_ncp_request(const u_char *pd, int offset, frame_data *fd,
 			if (ncp_request->req) {
 				ti = proto_tree_add_text(ncp_tree, offset, END_OF_FRAME,
 				"NCP Request Packet");
-				field_tree = proto_item_add_subtree(ti, ETT_NCP_REQUEST_FIELDS);
+				field_tree = proto_item_add_subtree(ti, ett_ncp_request_fields);
 
 				parse_ncp_svc_fields(pd, field_tree, offset, ncp_request->req);
 			}
@@ -613,7 +625,7 @@ dissect_ncp_reply(const u_char *pd, int offset, frame_data *fd,
 			if (ncp_request->rep) {
 				ti = proto_tree_add_text(ncp_tree, offset+8, END_OF_FRAME,
 				"NCP Reply Packet");
-				field_tree = proto_item_add_subtree(ti, ETT_NCP_REPLY_FIELDS);
+				field_tree = proto_item_add_subtree(ti, ett_ncp_reply_fields);
 
 				parse_ncp_svc_fields(pd, field_tree, offset+8, ncp_request->rep);
 			}
@@ -851,11 +863,33 @@ ncp_completion_code(guint8 ccode, enum nfamily family)
 void
 proto_register_ncp(void)
 {
-/*        static hf_register_info hf[] = {
-                { &variable,
-                { "Name",           "ncp.abbreviation", TYPE, VALS_POINTER }},
-        };*/
 
-        proto_ncp = proto_register_protocol("NetWare Core Protocol", "ncp");
- /*       proto_register_field_array(proto_ncp, hf, array_length(hf));*/
+  static hf_register_info hf[] = {
+    { &hf_ncp_type,
+      { "Type",			"ncp.type",
+	FT_UINT16, BASE_HEX, NULL, 0x0,
+	"NCP message type" }},
+    { &hf_ncp_seq,
+      { "Sequence Number",     	"ncp.seq",
+	FT_UINT8, BASE_DEC, NULL, 0x0,
+	"" }},
+    { &hf_ncp_connection,
+      { "Connection Number",    "ncp.connection",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+	"" }},
+    { &hf_ncp_task,
+      { "Task Number",     	"ncp.task",
+	FT_UINT8, BASE_DEC, NULL, 0x0,
+	"" }}
+  };
+  static gint *ett[] = {
+    &ett_ncp,
+    &ett_ncp_request_fields,
+    &ett_ncp_reply_fields,
+  };
+
+  proto_ncp = proto_register_protocol("NetWare Core Protocol", "ncp");
+  proto_register_field_array(proto_ncp, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
+  register_init_routine(&ncp_init_protocol);
 }
