@@ -3,7 +3,7 @@
  *
  * (c) Copyright Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: packet-rsvp.c,v 1.8 1999/08/29 04:15:30 gram Exp $
+ * $Id: packet-rsvp.c,v 1.10 1999/11/16 11:42:52 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -75,6 +75,25 @@
 
 static int proto_rsvp = -1;
 
+static gint ett_rsvp = -1;
+static gint ett_rsvp_hdr = -1;
+static gint ett_rsvp_session = -1;
+static gint ett_rsvp_hop = -1;
+static gint ett_rsvp_time_values = -1;
+static gint ett_rsvp_error = -1;
+static gint ett_rsvp_scope = -1;
+static gint ett_rsvp_style = -1;
+static gint ett_rsvp_confirm = -1;
+static gint ett_rsvp_sender_template = -1;
+static gint ett_rsvp_filter_spec = -1;
+static gint ett_rsvp_sender_tspec = -1;
+static gint ett_rsvp_flowspec = -1;
+static gint ett_rsvp_adspec = -1;
+static gint ett_rsvp_adspec_subtree = -1;
+static gint ett_rsvp_integrity = -1;
+static gint ett_rsvp_policy = -1;
+static gint ett_rsvp_unknown_class = -1;
+
 /* Stuff for IEEE float handling */
 
 #define IEEE_NUMBER_WIDTH	32	/* bits in number */
@@ -153,73 +172,125 @@ static hf_register_info rsvpf_info[] = {
 
     /* Message type number */
     {&rsvp_filter[RSVPF_MSG], 
-     { "Message Type", "rsvp.msg", FT_VALS_UINT8, VALS(message_type_vals) }},
+     { "Message Type", "rsvp.msg", FT_UINT8, BASE_NONE, message_type_vals, 0x0,
+     	"" }},
 
     /* Message type shorthands */
     {&rsvp_filter[RSVPF_PATH], 
-     { "Path Message", "rsvp.path", FT_UINT8, NULL }},
+     { "Path Message", "rsvp.path", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_RESV], 
-     { "Resv Message", "rsvp.resv", FT_UINT8, NULL }},
+     { "Resv Message", "rsvp.resv", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_PATHERR], 
-     { "Path Error Message", "rsvp.perr", FT_UINT8, NULL }},
+     { "Path Error Message", "rsvp.perr", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_RESVERR], 
-     { "Resv Error Message", "rsvp.rerr", FT_UINT8, NULL }},
+     { "Resv Error Message", "rsvp.rerr", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_PATHTEAR], 
-     { "Path Tear Message", "rsvp.ptear", FT_UINT8, NULL }},
+     { "Path Tear Message", "rsvp.ptear", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_RESVTEAR], 
-     { "Resv Tear Message", "rsvp.rtear", FT_UINT8, NULL }},
+     { "Resv Tear Message", "rsvp.rtear", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_RCONFIRM], 
-     { "Resv Confirm Message", "rsvp.resvconf", FT_UINT8, NULL }},
+     { "Resv Confirm Message", "rsvp.resvconf", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
 
     /* Object present */
     {&rsvp_filter[RSVPF_OBJECT], 
-     { "", "rsvp.object", FT_VALS_UINT8, VALS(rsvp_class_vals) }},
+     { "", "rsvp.object", FT_UINT8, BASE_NONE, rsvp_class_vals, 0x0,
+     	"" }},
 
     /* Object present shorthands */
     {&rsvp_filter[RSVPF_SESSION], 
-     { "SESSION", "rsvp.session", FT_UINT8, NULL }},
+     { "SESSION", "rsvp.session", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_HOP], 
-     { "HOP", "rsvp.hop", FT_UINT8, NULL }},
+     { "HOP", "rsvp.hop", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_INTEGRITY], 
-     { "INTEGRITY", "rsvp.integrity", FT_UINT8, NULL }},
+     { "INTEGRITY", "rsvp.integrity", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_TIME_VALUES], 
-     { "TIME VALUES", "rsvp.time", FT_UINT8, NULL }},
+     { "TIME VALUES", "rsvp.time", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_ERROR], 
-     { "ERROR", "rsvp.error", FT_UINT8, NULL }},
+     { "ERROR", "rsvp.error", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_SCOPE], 
-     { "SCOPE", "rsvp.scope", FT_UINT8, NULL }},
+     { "SCOPE", "rsvp.scope", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_STYLE], 
-     { "STYLE", "rsvp.style", FT_UINT8, NULL }},
+     { "STYLE", "rsvp.style", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_FLOWSPEC], 
-     { "FLOWSPEC", "rsvp.flowspec", FT_UINT8, NULL }},
+     { "FLOWSPEC", "rsvp.flowspec", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_FILTER_SPEC], 
-     { "FILTERSPEC", "rsvp.filter", FT_UINT8, NULL }},
+     { "FILTERSPEC", "rsvp.filter", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_SENDER], 
-     { "SENDER TEMPLATE", "rsvp.sender", FT_UINT8, NULL }},
+     { "SENDER TEMPLATE", "rsvp.sender", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_TSPEC], 
-     { "SENDER TSPEC", "rsvp.tspec", FT_UINT8, NULL }},
+     { "SENDER TSPEC", "rsvp.tspec", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_ADSPEC], 
-     { "ADSPEC", "rsvp.adspec", FT_UINT8, NULL }},
+     { "ADSPEC", "rsvp.adspec", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_POLICY], 
-     { "POLICY", "rsvp.policy", FT_UINT8, NULL }},
+     { "POLICY", "rsvp.policy", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_CONFIRM], 
-     { "CONFIRM", "rsvp.confirm", FT_UINT8, NULL }},
+     { "CONFIRM", "rsvp.confirm", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_UNKNOWN_OBJ], 
-     { "Unknown object", "rsvp.obj_unknown", FT_UINT8, NULL }},
+     { "Unknown object", "rsvp.obj_unknown", FT_UINT8, BASE_NONE, NULL, 0x0,
+     	"" }},
 
     /* Session fields */
     {&rsvp_filter[RSVPF_SESSION_IP], 
-     { "Destination address", "rsvp.session.ip", FT_IPv4, NULL }},
+     { "Destination address", "rsvp.session.ip", FT_IPv4, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_SESSION_PORT], 
-     { "Port number", "rsvp.session.port", FT_UINT16, NULL }},
+     { "Port number", "rsvp.session.port", FT_UINT16, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_SESSION_PROTO], 
-     { "Protocol", "rsvp.session.proto", FT_VALS_UINT8, VALS(proto_vals) }},
+     { "Protocol", "rsvp.session.proto", FT_UINT8, BASE_NONE, VALS(proto_vals), 0x0,
+     	"" }},
 
     /* Sender template fields */
     {&rsvp_filter[RSVPF_SENDER_IP], 
-     { "Sender Template IPv4 address", "rsvp.template.ip", FT_IPv4, NULL }},
+     { "Sender Template IPv4 address", "rsvp.template.ip", FT_IPv4, BASE_NONE, NULL, 0x0,
+     	"" }},
+
     {&rsvp_filter[RSVPF_SENDER_PORT], 
-     { "Sender Template port number", "rsvp.template.port", FT_UINT16, NULL }}
+     { "Sender Template port number", "rsvp.template.port", FT_UINT16, BASE_NONE, NULL, 0x0,
+     	"" }}
 };
 
 static inline int rsvp_class_to_filter_num(int classnum)
@@ -321,11 +392,11 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     if (tree) {
 	msg_length = pntohs(pd+offset+6);
 	ti = proto_tree_add_item(tree, proto_rsvp, offset, msg_length, NULL);
-	rsvp_tree = proto_item_add_subtree(ti, ETT_RSVP);
+	rsvp_tree = proto_item_add_subtree(ti, ett_rsvp);
 
 	ti = proto_tree_add_text(rsvp_tree, offset, 
 			      sizeof(rsvp_header), "RSVP Header"); 
-	rsvp_header_tree = proto_item_add_subtree(ti, ETT_RSVP_HDR);
+	rsvp_header_tree = proto_item_add_subtree(ti, ett_rsvp_hdr);
 
         proto_tree_add_text(rsvp_header_tree, offset, 1, "RSVP Version: %d", 
 			 (hdr->ver_flags & 0xf0)>>4);  
@@ -374,7 +445,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	    switch(obj->class) {
 
 	    case RSVP_CLASS_SESSION : 		
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_SESSION);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_session);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -436,7 +507,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		break;
 		
 	    case RSVP_CLASS_HOP :		
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_HOP);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_hop);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -481,7 +552,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		break;
 		
 	    case RSVP_CLASS_TIME_VALUES : 
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_TIME_VALUES);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_time_values);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -511,7 +582,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		break;
 
 	    case RSVP_CLASS_ERROR :
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_ERROR);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_error);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -573,7 +644,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 
 	    case RSVP_CLASS_SCOPE : 
 		mylen = obj_length;
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_SCOPE);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_scope);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -622,7 +693,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		break;
 		
 	    case RSVP_CLASS_STYLE : 
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_STYLE);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_style);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -653,7 +724,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		break;
 	    
 	    case RSVP_CLASS_CONFIRM :		
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_CONFIRM);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_confirm);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -692,7 +763,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		break;
 
 	    case RSVP_CLASS_SENDER_TEMPLATE :
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_SENDER_TEMPLATE);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_sender_template);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -700,7 +771,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 				    obj->class, object_type);
 		goto common_template;
 	    case RSVP_CLASS_FILTER_SPEC :
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_FILTER_SPEC);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_filter_spec);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -751,7 +822,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		char *str;
 
 		mylen = obj_length;
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_SENDER_TSPEC);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_sender_tspec);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -863,7 +934,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		char *str;
 
 		mylen = obj_length;
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_FLOWSPEC);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_flowspec);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -991,10 +1062,9 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		param_hdr *phdr; 
 
 		char *str;
-		int tree_num;
 
 		mylen = obj_length;
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_ADSPEC);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_adspec);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -1008,7 +1078,6 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 				    "Data length: %d words, not including header", 
 				    pntohs(pd+offset2+2));
 		offset2+=4;
-		tree_num=ETT_RSVP_ADSPEC_SUBTREE1;
 		mylen -= 4;
 		while (mylen > 4) {
 		    shdr = (service_hdr *)(pd + offset2);
@@ -1017,7 +1086,8 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		    ti = proto_tree_add_text(rsvp_object_tree, offset2, 
 					     (pntohs(&shdr->length)+1)<<2,
 					     str?str:"Unknown");
-		    adspec_tree = proto_item_add_subtree(ti, tree_num++);
+		    adspec_tree = proto_item_add_subtree(ti,
+		    			ett_rsvp_adspec_subtree);
 		    proto_tree_add_text(adspec_tree, offset2, 1,
 					"Service header %d - %s",
 					shdr->service_num, str);
@@ -1080,7 +1150,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	    }
 
 	    case RSVP_CLASS_INTEGRITY :
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_INTEGRITY);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_integrity);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -1089,7 +1159,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		goto default_class;
 
 	    case RSVP_CLASS_POLICY :
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_POLICY);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_policy);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -1098,7 +1168,7 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		goto default_class;
 
 	    default :
-		rsvp_object_tree = proto_item_add_subtree(ti, ETT_RSVP_UNKNOWN_CLASS);
+		rsvp_object_tree = proto_item_add_subtree(ti, ett_rsvp_unknown_class);
 		proto_tree_add_text(rsvp_object_tree, offset, 2, "Length: %d", 
 				    obj_length);
 		proto_tree_add_text(rsvp_object_tree, offset+2, 1, 
@@ -1124,6 +1194,28 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 void
 proto_register_rsvp(void)
 {
+	static gint *ett[] = {
+		&ett_rsvp,
+		&ett_rsvp_hdr,
+		&ett_rsvp_session,
+		&ett_rsvp_hop,
+		&ett_rsvp_time_values,
+		&ett_rsvp_error,
+		&ett_rsvp_scope,
+		&ett_rsvp_style,
+		&ett_rsvp_confirm,
+		&ett_rsvp_sender_template,
+		&ett_rsvp_filter_spec,
+		&ett_rsvp_sender_tspec,
+		&ett_rsvp_flowspec,
+		&ett_rsvp_adspec,
+		&ett_rsvp_adspec_subtree,
+		&ett_rsvp_integrity,
+		&ett_rsvp_policy,
+		&ett_rsvp_unknown_class,
+	};
+
         proto_rsvp = proto_register_protocol("Resource ReserVation Protocol (RSVP)", "rsvp");
         proto_register_field_array(proto_rsvp, rsvpf_info, array_length(rsvpf_info));
+	proto_register_subtree_array(ett, array_length(ett));
 }
