@@ -2,7 +2,7 @@
  * Routines for SNMP (simple network management protocol)
  * D.Jorand (c) 1998
  *
- * $Id: packet-snmp.c,v 1.9 1999/09/12 14:34:18 deniel Exp $
+ * $Id: packet-snmp.c,v 1.14 1999/11/18 07:29:53 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -52,10 +52,14 @@
 #include <glib.h>
 #include "packet.h"
 
+#include "packet-snmp.h"
+
 #define in_addr_t u_int
 
 static int proto_snmp = -1;
 
+static gint ett_snmp = -1;
+
 #ifdef WITH_SNMP_UCD
 /* should be defined only if supported in ucd-snmp */
 #define OPAQUE_SPECIAL_TYPES 1
@@ -283,6 +287,13 @@ dissect_snmp_error(const u_char *pd, int offset, frame_data *fd,
 
 void
 dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) 
+{
+	dissect_snmp_pdu(pd, offset, fd, tree, "SNMP", proto_snmp, ett_snmp);
+}
+
+void
+dissect_snmp_pdu(const u_char *pd, int offset, frame_data *fd,
+    proto_tree *tree, char *proto_name, int proto, gint ett)
 {
 	int length=fd->pkt_len-offset;
 	u_char *data, *tmp_data;
@@ -332,7 +343,7 @@ dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	proto_item *item=NULL;
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "SNMP");
+		col_add_str(fd, COL_PROTOCOL, proto_name);
 
 	/* NOTE: we have to parse the message piece by piece, since the
 	 * capture length may be less than the message length: a 'global'
@@ -341,7 +352,7 @@ dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	
 #ifdef WITH_SNMP_UCD	
 	/* parse the SNMP header */
-	if(NULL == asn_parse_header( &pd[offset], &length, &type)) {
+	if(NULL == asn_parse_header((u_char*)&pd[offset], &length, &type)) {
 		dissect_snmp_error(pd, offset, fd, tree,
 			"Couldn't parse SNMP header");
 		return;
@@ -355,7 +366,7 @@ dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	/* authenticates message */
 	length=fd->pkt_len-offset;
 	header_length=length;
-	data = snmp_comstr_parse(&pd[offset], &length, community, &community_length, (int*)&version);
+	data = snmp_comstr_parse((u_char*)&pd[offset], &length, community, &community_length, (long*)&version);
 	if(NULL == data) {
 		dissect_snmp_error(pd, offset, fd, tree,
 		    "Couldn't parse authentication");
@@ -368,7 +379,7 @@ dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	header_length=length;	
 
 	/* parse the SNMP header */
-	data = asn_parse_header( &pd[offset], &length, &type);
+	data = asn_parse_header((u_char*)&pd[offset], &length, &type);
 	if(NULL == data) {
 		dissect_snmp_error(pd, offset, fd, tree,
 			"Couldn't parse SNMP header");
@@ -454,8 +465,8 @@ dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		if (tree) {
 			/* all_length=header_length+pdu_type_length+request_id_length+error_status_length+error_index_length; */
 			all_length=fd->pkt_len-offset;
-			item = proto_tree_add_item(tree, proto_snmp, offset, all_length, NULL);
-			snmp_tree = proto_item_add_subtree(item, ETT_SNMP);
+			item = proto_tree_add_item(tree, proto, offset, all_length, NULL);
+			snmp_tree = proto_item_add_subtree(item, ett);
 			proto_tree_add_text(snmp_tree, offset, header_length, "Community: \"%s\", Version: %s", community, val_to_str(version, versions, "Unknown version %#x"));
 			offset+=header_length;
 			proto_tree_add_text(snmp_tree, offset, pdu_type_length, "%s", pdu_type_string);
@@ -482,8 +493,8 @@ dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			col_add_str(fd, COL_INFO, pdu_type_string);
 		if(tree) {
 			all_length=fd->pkt_len-offset;
-			item = proto_tree_add_item(tree, proto_snmp, offset, all_length, NULL);
-			snmp_tree = proto_item_add_subtree(item, ETT_SNMP);
+			item = proto_tree_add_item(tree, proto, offset, all_length, NULL);
+			snmp_tree = proto_item_add_subtree(item, ett);
 			proto_tree_add_text(snmp_tree, offset, header_length, "Community: \"%s\", Version: %s", community, val_to_str(version, versions, "Unknown version %#x"));
 			offset+=header_length;
 			proto_tree_add_text(snmp_tree, offset, pdu_type_length, "Pdu type: %s", pdu_type_string);
@@ -858,17 +869,30 @@ dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	}
 }
 
+#endif /* WITH_SNMP: CMU or UCD */
+
+/*
+ * Allow the stuff that builds "register.c" to scan "packet-snmp.c" even
+ * if we're not enabling SNMP decoding (for "canned" source distributions
+ * such as BSD ports, it may be a pain to arrange that it be scanned only
+ * if SNMP is being used), by having "proto_register_snmp()" always be
+ * defined, but not do anything if SNMP decoding isn't enabled.
+ */
 void
 proto_register_snmp(void)
 {
+#if defined(WITH_SNMP_CMU) || defined(WITH_SNMP_UCD)
 /*        static hf_register_info hf[] = {
                 { &variable,
                 { "Name",           "snmp.abbreviation", TYPE, VALS_POINTER }},
         };*/
+	static gint *ett[] = {
+		&ett_snmp,
+	};
 
 	init_mib();
         proto_snmp = proto_register_protocol("Simple Network Management Protocol", "snmp");
  /*       proto_register_field_array(proto_snmp, hf, array_length(hf));*/
-}
-
+	proto_register_subtree_array(ett, array_length(ett));
 #endif /* WITH_SNMP: CMU or UCD */
+}
