@@ -1,7 +1,7 @@
 /* packet-tcp.c
  * Routines for TCP packet disassembly
  *
- * $Id: packet-tcp.c,v 1.33 1999/09/17 05:56:56 guy Exp $
+ * $Id: packet-tcp.c,v 1.47 1999/11/28 03:35:10 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -60,12 +60,28 @@ extern FILE* data_out_file;
 static gchar info_str[COL_MAX_LEN];
 static int   info_len;
 
-int proto_tcp = -1;
-int hf_tcp_srcport = -1;
-int hf_tcp_dstport = -1;
-int hf_tcp_port = -1;
-int hf_tcp_seq = -1;
-int hf_tcp_ack = -1;
+static int proto_tcp = -1;
+static int hf_tcp_srcport = -1;
+static int hf_tcp_dstport = -1;
+static int hf_tcp_port = -1;
+static int hf_tcp_seq = -1;
+static int hf_tcp_ack = -1;
+static int hf_tcp_hdr_len = -1;
+static int hf_tcp_flags = -1;
+static int hf_tcp_flags_urg = -1;
+static int hf_tcp_flags_ack = -1;
+static int hf_tcp_flags_push = -1;
+static int hf_tcp_flags_reset = -1;
+static int hf_tcp_flags_syn = -1;
+static int hf_tcp_flags_fin = -1;
+static int hf_tcp_window_size = -1;
+static int hf_tcp_checksum = -1;
+static int hf_tcp_urgent_pointer = -1;
+
+static gint ett_tcp = -1;
+static gint ett_tcp_flags = -1;
+static gint ett_tcp_options = -1;
+static gint ett_tcp_option_sack = -1;
 
 /* TCP Ports */
 
@@ -76,11 +92,16 @@ int hf_tcp_ack = -1;
 #define TCP_PORT_HTTP     80
 #define TCP_PORT_POP      110
 #define TCP_PORT_NNTP     119
+#define TCP_PORT_NTP      123
 #define TCP_PORT_NBSS     139
+#define TCP_PORT_IMAP     143
+#define TCP_PORT_BGP      179
 #define TCP_PORT_PRINTER  515
 #define TCP_ALT_PORT_HTTP 8080
 #define TCP_PORT_PPTP     1723
 #define TCP_PORT_RTSP     554
+#define TCP_PORT_YHOO     5050
+#define TCP_PORT_MAPI     1065
 
 /* TCP structs and definitions */
 
@@ -178,7 +199,7 @@ dissect_tcpopt_sack(const ip_tcp_opt *optp, const u_char *opd,
   while (optlen > 0) {
     if (field_tree == NULL) {
       /* Haven't yet made a subtree out of this option.  Do so. */
-      field_tree = proto_item_add_subtree(tf, optp->subtree_index);
+      field_tree = proto_item_add_subtree(tf, *optp->subtree_index);
     }
     if (optlen < 4) {
       proto_tree_add_text(field_tree, offset,      optlen,
@@ -238,7 +259,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_EOL,
     "EOL",
-    -1,
+    NULL,
     NO_LENGTH,
     0,
     NULL,
@@ -246,7 +267,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_NOP,
     "NOP",
-    -1,
+    NULL,
     NO_LENGTH,
     0,
     NULL,
@@ -254,7 +275,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_MSS,
     "Maximum segment size",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_MSS,
     dissect_tcpopt_maxseg
@@ -262,7 +283,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_WINDOW,
     "Window scale",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_WINDOW,
     dissect_tcpopt_wscale
@@ -270,7 +291,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_SACK_PERM,
     "SACK permitted",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_SACK_PERM,
     NULL,
@@ -278,7 +299,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_SACK,
     "SACK",
-    ETT_TCP_OPTION_SACK,
+    &ett_tcp_option_sack,
     VARIABLE_LENGTH,
     TCPOLEN_SACK_MIN,
     dissect_tcpopt_sack
@@ -286,7 +307,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_ECHO,
     "Echo",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_ECHO,
     dissect_tcpopt_echo
@@ -294,7 +315,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_ECHOREPLY,
     "Echo reply",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_ECHOREPLY,
     dissect_tcpopt_echo
@@ -302,7 +323,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_TIMESTAMP,
     "Time stamp",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_TIMESTAMP,
     dissect_tcpopt_timestamp
@@ -310,7 +331,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_CC,
     "CC",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_CC,
     dissect_tcpopt_cc
@@ -318,7 +339,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_CCNEW,
     "CC.NEW",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOPT_CCNEW,
     dissect_tcpopt_cc
@@ -326,7 +347,7 @@ static const ip_tcp_opt tcpopts[] = {
   {
     TCPOPT_CCECHO,
     "CC.ECHO",
-    -1,
+    NULL,
     FIXED_LENGTH,
     TCPOLEN_CCECHO,
     dissect_tcpopt_cc
@@ -335,6 +356,12 @@ static const ip_tcp_opt tcpopts[] = {
 
 #define N_TCP_OPTS	(sizeof tcpopts / sizeof tcpopts[0])
 
+/* TCP flags flag */
+static const true_false_string flags_set_truth = {
+  "Set",
+  "Not set"
+};
+
 void
 dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   e_tcphdr   th;
@@ -378,14 +405,6 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   
   hlen = hi_nibble(th.th_off_x2) * 4;  /* TCP header length, in bytes */
 
-  if (check_col(fd, COL_RES_SRC_PORT))
-    col_add_str(fd, COL_RES_SRC_PORT, get_tcp_port(th.th_sport));
-  if (check_col(fd, COL_UNRES_SRC_PORT))
-    col_add_fstr(fd, COL_UNRES_SRC_PORT, "%u", th.th_sport);
-  if (check_col(fd, COL_RES_DST_PORT))
-    col_add_str(fd, COL_RES_DST_PORT, get_tcp_port(th.th_dport));
-  if (check_col(fd, COL_UNRES_DST_PORT))
-    col_add_fstr(fd, COL_UNRES_DST_PORT, "%u", th.th_dport);
   if (check_col(fd, COL_PROTOCOL))
     col_add_str(fd, COL_PROTOCOL, "TCP");
   if (check_col(fd, COL_INFO)) {
@@ -403,8 +422,8 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   }
   
   if (tree) {
-    ti = proto_tree_add_item(tree, proto_tcp, offset, hlen, NULL);
-    tcp_tree = proto_item_add_subtree(ti, ETT_TCP);
+    ti = proto_tree_add_item_format(tree, proto_tcp, offset, hlen, NULL, "Transmission Control Protocol, Src Port: %s (%u), Dst Port: %s (%u), Seq: %u, Ack: %u", get_tcp_port(th.th_sport), th.th_sport, get_tcp_port(th.th_dport), th.th_dport, th.th_seq, th.th_ack);
+    tcp_tree = proto_item_add_subtree(ti, ett_tcp);
     proto_tree_add_item_format(tcp_tree, hf_tcp_srcport, offset, 2, th.th_sport,
 	"Source port: %s (%u)", get_tcp_port(th.th_sport), th.th_sport);
     proto_tree_add_item_format(tcp_tree, hf_tcp_dstport, offset + 2, 2, th.th_dport,
@@ -414,32 +433,21 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     proto_tree_add_item(tcp_tree, hf_tcp_seq, offset + 4, 4, th.th_seq);
     if (th.th_flags & TH_ACK)
       proto_tree_add_item(tcp_tree, hf_tcp_ack, offset + 8, 4, th.th_ack);
-    proto_tree_add_text(tcp_tree, offset + 12, 1, "Header length: %u bytes", hlen);
-     tf = proto_tree_add_text(tcp_tree, offset + 13, 1, "Flags: 0x%x", th.th_flags);
-     field_tree = proto_item_add_subtree(tf, ETT_TCP_FLAGS);
-     proto_tree_add_text(field_tree, offset + 13, 1, "%s",
-       decode_boolean_bitfield(th.th_flags, TH_URG, sizeof (th.th_flags)*8,
-                         "Urgent pointer", "No urgent pointer"));
-     proto_tree_add_text(field_tree, offset + 13, 1, "%s",
-       decode_boolean_bitfield(th.th_flags, TH_ACK, sizeof (th.th_flags)*8,
-                         "Acknowledgment", "No acknowledgment"));
-     proto_tree_add_text(field_tree, offset + 13, 1, "%s",
-       decode_boolean_bitfield(th.th_flags, TH_PUSH, sizeof (th.th_flags)*8,
-                         "Push", "No push"));
-     proto_tree_add_text(field_tree, offset + 13, 1, "%s",
-       decode_boolean_bitfield(th.th_flags, TH_RST, sizeof (th.th_flags)*8,
-                         "Reset", "No reset"));
-     proto_tree_add_text(field_tree, offset + 13, 1, "%s",
-       decode_boolean_bitfield(th.th_flags, TH_SYN, sizeof (th.th_flags)*8,
-                         "Syn", "No Syn"));
-     proto_tree_add_text(field_tree, offset + 13, 1, "%s",
-       decode_boolean_bitfield(th.th_flags, TH_FIN, sizeof (th.th_flags)*8,
-                         "Fin", "No Fin"));
-    proto_tree_add_text(tcp_tree, offset + 14, 2, "Window size: %u", th.th_win);
-    proto_tree_add_text(tcp_tree, offset + 16, 2, "Checksum: 0x%04x", th.th_sum);
+    proto_tree_add_item_format(tcp_tree, hf_tcp_hdr_len, offset + 12, 1, hlen,
+	"Header length: %u bytes", hlen);
+    tf = proto_tree_add_item_format(tcp_tree, hf_tcp_flags, offset + 13, 1,
+	th.th_flags, "Flags: 0x%04x (%s)", th.th_flags, flags);
+    field_tree = proto_item_add_subtree(tf, ett_tcp_flags);
+    proto_tree_add_item(field_tree, hf_tcp_flags_urg, offset + 13, 1, th.th_flags);
+    proto_tree_add_item(field_tree, hf_tcp_flags_ack, offset + 13, 1, th.th_flags);
+    proto_tree_add_item(field_tree, hf_tcp_flags_push, offset + 13, 1, th.th_flags);
+    proto_tree_add_item(field_tree, hf_tcp_flags_reset, offset + 13, 1, th.th_flags);
+    proto_tree_add_item(field_tree, hf_tcp_flags_syn, offset + 13, 1, th.th_flags);
+    proto_tree_add_item(field_tree, hf_tcp_flags_fin, offset + 13, 1, th.th_flags);
+    proto_tree_add_item(tcp_tree, hf_tcp_window_size, offset + 14, 2, th.th_win);
+    proto_tree_add_item(tcp_tree, hf_tcp_checksum, offset + 16, 2, th.th_sum);
     if (th.th_flags & TH_URG)
-      proto_tree_add_text(tcp_tree, offset + 18, 2, "Urgent pointer: 0x%04x",
-        th.th_urp);
+      proto_tree_add_item(tcp_tree, hf_tcp_urgent_pointer, offset + 18, 2, th.th_urp);
   }
 
   /* Decode TCP options, if any. */
@@ -449,7 +457,7 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     optlen = hlen - sizeof (e_tcphdr); /* length of options, in bytes */
     tf = proto_tree_add_text(tcp_tree, offset +  20, optlen,
       "Options: (%d bytes)", optlen);
-    field_tree = proto_item_add_subtree(tf, ETT_TCP_OPTIONS);
+    field_tree = proto_item_add_subtree(tf, ett_tcp_options);
     dissect_ip_tcp_options(&pd[offset + 20], offset + 20, optlen,
       tcpopts, N_TCP_OPTS, TCPOPT_EOL, field_tree);
   }
@@ -460,12 +468,20 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   /* Skip over header + options */
   offset += hlen;
 
+  pi.ptype = PT_TCP;
   pi.srcport = th.th_sport;
   pi.destport = th.th_dport;
   
   /* Check the packet length to see if there's more data
      (it could be an ACK-only packet) */
   if (packet_max > offset) {
+
+    /* ONC RPC.  We can't base this on anything in the TCP header; we have
+       to look at the payload.  If "dissect_rpc()" returns TRUE, it was
+       an RPC packet, otherwise it's some other type of packet. */
+    if (dissect_rpc(pd, offset, fd, tree))
+      goto reas;
+
     /* XXX - this should be handled the way UDP handles this, with a table
        of port numbers to which stuff can be added */
 #define PORT_IS(port)	(th.th_sport == port || th.th_dport == port)
@@ -483,9 +499,15 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     } else if (PORT_IS(TCP_PORT_POP)) {
       pi.match_port = TCP_PORT_POP;
       dissect_pop(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_IMAP)) {
+      pi.match_port = TCP_PORT_IMAP;
+      dissect_imap(pd, offset, fd, tree);
     } else if (PORT_IS(TCP_PORT_NNTP)) {
       pi.match_port = TCP_PORT_NNTP;
       dissect_nntp(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_NTP)) {
+      pi.match_port = TCP_PORT_NTP;
+      dissect_ntp(pd, offset, fd, tree);
     } else if (PORT_IS(TCP_PORT_PPTP)) {
       pi.match_port = TCP_PORT_PPTP;
       dissect_pptp(pd, offset, fd, tree);
@@ -497,17 +519,30 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       dissect_nbss(pd, offset, fd, tree);
     } else if (PORT_IS(TCP_PORT_RTSP))
       dissect_rtsp(pd, offset, fd, tree);
-    else {
+    else if (PORT_IS(TCP_PORT_BGP)) {
+      pi.match_port = TCP_PORT_BGP;
+      dissect_bgp(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_MAPI)) {
+      pi.match_port = TCP_PORT_MAPI;
+      dissect_mapi(pd, offset, fd, tree);
+    } else {
         /* check existence of high level protocols */
 
         if (memcmp(&pd[offset], "GIOP",  4) == 0) {
           dissect_giop(pd, offset, fd, tree);
         }
+	else if ( PORT_IS(TCP_PORT_YHOO) && 
+		(memcmp(&pd[offset], "YPNS",  4) == 0 ||
+			memcmp(&pd[offset], "YHOO",  4) == 0 )) {
+	  dissect_yhoo(pd, offset, fd, tree);
+	}
         else {
           dissect_data(pd, offset, fd, tree);
         }
     }
   }
+
+reas:
  
   if( data_out_file ) {
     reassemble_tcp( th.th_seq,		/* sequence number */
@@ -515,10 +550,12 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
         ( pd+offset ),			/* data */
         ( pi.captured_len - offset ),	/* captured data length */
         ( th.th_flags & TH_SYN ),	/* is syn set? */
-        pi.ip_src,
-	pi.ip_dst,
+        &pi.net_src,
+	&pi.net_dst,
 	pi.srcport,
-	pi.destport); 
+	pi.destport,
+	fd->rel_secs,
+	fd->rel_usecs);
   }
 }
 
@@ -528,21 +565,77 @@ proto_register_tcp(void)
 	static hf_register_info hf[] = {
 
 		{ &hf_tcp_srcport,
-		{ "Source Port",		"tcp.srcport", FT_UINT16, NULL }},
+		{ "Source Port",		"tcp.srcport", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_tcp_dstport,
-		{ "Destination Port",		"tcp.dstport", FT_UINT16, NULL }},
+		{ "Destination Port",		"tcp.dstport", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_tcp_port,
-		{ "Source or Destination Port",	"tcp.port", FT_UINT16, NULL }},
+		{ "Source or Destination Port",	"tcp.port", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_tcp_seq,
-		{ "Sequence number",		"tcp.seq", FT_UINT32, NULL }},
+		{ "Sequence number",		"tcp.seq", FT_UINT32, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_tcp_ack,
-		{ "Acknowledgement number",	"tcp.ack", FT_UINT32, NULL }},
+		{ "Acknowledgement number",	"tcp.ack", FT_UINT32, BASE_DEC, NULL, 0x0,
+			"" }},
+
+		{ &hf_tcp_hdr_len,
+		{ "Header Length",		"tcp.hdr_len", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
+
+		{ &hf_tcp_flags,
+		{ "Flags",			"tcp.flags", FT_UINT8, BASE_HEX, NULL, 0x0,
+			"" }},
+
+		{ &hf_tcp_flags_urg,
+		{ "Urgent",			"tcp.flags.urg", FT_BOOLEAN, 8, TFS(&flags_set_truth), TH_URG,
+			"" }},
+
+		{ &hf_tcp_flags_ack,
+		{ "Acknowledgment",		"tcp.flags.ack", FT_BOOLEAN, 8, TFS(&flags_set_truth), TH_ACK,
+			"" }},
+
+		{ &hf_tcp_flags_push,
+		{ "Push",			"tcp.flags.push", FT_BOOLEAN, 8, TFS(&flags_set_truth), TH_PUSH,
+			"" }},
+
+		{ &hf_tcp_flags_reset,
+		{ "Reset",			"tcp.flags.reset", FT_BOOLEAN, 8, TFS(&flags_set_truth), TH_RST,
+			"" }},
+
+		{ &hf_tcp_flags_syn,
+		{ "Syn",			"tcp.flags.syn", FT_BOOLEAN, 8, TFS(&flags_set_truth), TH_SYN,
+			"" }},
+
+		{ &hf_tcp_flags_fin,
+		{ "Fin",			"tcp.flags.fin", FT_BOOLEAN, 8, TFS(&flags_set_truth), TH_FIN,
+			"" }},
+
+		{ &hf_tcp_window_size,
+		{ "Window size",		"tcp.window_size", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
+
+		{ &hf_tcp_checksum,
+		{ "Checksum",			"tcp.checksum", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
+
+		{ &hf_tcp_urgent_pointer,
+		{ "Urgent pointer",		"tcp.urgent_pointer", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
+	};
+	static gint *ett[] = {
+		&ett_tcp,
+		&ett_tcp_flags,
+		&ett_tcp_options,
+		&ett_tcp_option_sack,
 	};
 
 	proto_tcp = proto_register_protocol ("Transmission Control Protocol", "tcp");
 	proto_register_field_array(proto_tcp, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
