@@ -1,7 +1,7 @@
 /* packet-udp.c
  * Routines for UDP packet disassembly
  *
- * $Id: packet-udp.c,v 1.24 1999/09/14 08:06:23 guy Exp $
+ * $Id: packet-udp.c,v 1.38 1999/11/23 17:09:57 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -44,12 +44,14 @@
 #include "packet.h"
 #include "resolv.h"
 
-int proto_udp = -1;		
-int hf_udp_srcport = -1;
-int hf_udp_dstport = -1;
-int hf_udp_port = -1;
-int hf_udp_length = -1;
-int hf_udp_checksum = -1;
+static int proto_udp = -1;		
+static int hf_udp_srcport = -1;
+static int hf_udp_dstport = -1;
+static int hf_udp_port = -1;
+static int hf_udp_length = -1;
+static int hf_udp_checksum = -1;
+
+static gint ett_udp = -1;
 
 /* UDP structs and definitions */
 
@@ -66,17 +68,26 @@ typedef struct _e_udphdr {
 #define UDP_PORT_BOOTPS  67
 #define UDP_PORT_TFTP    69
 #define UDP_PORT_IPX    213
+#define UDP_PORT_NTP	123
 #define UDP_PORT_NBNS	137
 #define UDP_PORT_NBDGM	138
 #define UDP_PORT_SNMP   161
+#define UDP_PORT_PIM_RP_DISC 496
 #define UDP_PORT_ISAKMP	500
 #define UDP_PORT_RIP    520
+#define UDP_PORT_RIPNG  521
 #define UDP_PORT_VINES	573
 #define UDP_PORT_RADIUS 1645
 #define UDP_PORT_RADIUS_NEW 1812
 #define UDP_PORT_RADACCT 1646
 #define UDP_PORT_RADACCT_NEW 1813
+#define UDP_PORT_HSRP   1985
 #define UDP_PORT_ICP    3130
+#define UDP_PORT_ICQ	4000
+#define UDP_PORT_SAP	9875
+#define UDP_PORT_RX_LOW 7000
+#define UDP_PORT_RX_HIGH 7009
+#define UDP_PORT_RX_AFS_BACKUPS 7021
 
 struct hash_struct {
   guint16 proto;
@@ -163,6 +174,9 @@ void init_dissect_udp(void) {
 
   udp_hash_add(UDP_PORT_BOOTPS, dissect_bootp);
   udp_hash_add(UDP_PORT_TFTP, dissect_tftp);
+  udp_hash_add(UDP_PORT_SAP, dissect_sap);
+  udp_hash_add(UDP_PORT_HSRP, dissect_hsrp);
+  udp_hash_add(UDP_PORT_PIM_RP_DISC, dissect_auto_rp);
 
 }
 
@@ -174,7 +188,11 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   proto_tree *udp_tree;
   proto_item *ti;
 
-  /* To do: Check for {cap len,pkt len} < struct len */
+  if (!BYTES_ARE_IN_FRAME(offset, sizeof(e_udphdr))) {
+    dissect_data(pd, offset, fd, tree);
+    return;
+  }
+
   /* Avoids alignment problems on many architectures. */
   memcpy(&uh, &pd[offset], sizeof(e_udphdr));
   uh_sport = ntohs(uh.uh_sport);
@@ -187,18 +205,10 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   if (check_col(fd, COL_INFO))
     col_add_fstr(fd, COL_INFO, "Source port: %s  Destination port: %s",
 	    get_udp_port(uh_sport), get_udp_port(uh_dport));
-  if (check_col(fd, COL_RES_SRC_PORT))
-    col_add_str(fd, COL_RES_SRC_PORT, get_udp_port(uh_sport));
-  if (check_col(fd, COL_UNRES_SRC_PORT))
-    col_add_fstr(fd, COL_UNRES_SRC_PORT, "%u", uh_sport);
-  if (check_col(fd, COL_RES_DST_PORT))
-    col_add_str(fd, COL_RES_DST_PORT, get_udp_port(uh_dport));
-  if (check_col(fd, COL_UNRES_DST_PORT))
-    col_add_fstr(fd, COL_UNRES_DST_PORT, "%u", uh_dport);
     
   if (tree) {
     ti = proto_tree_add_item(tree, proto_udp, offset, 8);
-    udp_tree = proto_item_add_subtree(ti, ETT_UDP);
+    udp_tree = proto_item_add_subtree(ti, ett_udp);
 
     proto_tree_add_item_format(udp_tree, hf_udp_srcport, offset, 2, uh_sport,
 	"Source port: %s (%u)", get_udp_port(uh_sport), uh_sport);
@@ -216,41 +226,61 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   /* Skip over header */
   offset += 8;
 
+  pi.ptype = PT_UDP;
+  pi.srcport = uh_sport;
+  pi.destport = uh_dport;
+
+  /* ONC RPC.  We can't base this on anything in the UDP header; we have
+     to look at the payload.  If "dissect_rpc()" returns TRUE, it was
+     an RPC packet, otherwise it's some other type of packet. */
+  if (dissect_rpc(pd, offset, fd, tree))
+    return;
+
   /* XXX - we should do all of this through the table of ports. */
 #define PORT_IS(port)	(uh_sport == port || uh_dport == port)
- if (PORT_IS(UDP_PORT_BOOTPS))
+  if (PORT_IS(UDP_PORT_BOOTPS))
       dissect_bootp(pd, offset, fd, tree);
- else if (PORT_IS(UDP_PORT_DNS))
+  else if (PORT_IS(UDP_PORT_DNS))
       dissect_dns(pd, offset, fd, tree);
- else if (PORT_IS(UDP_PORT_ISAKMP))
+  else if (PORT_IS(UDP_PORT_ISAKMP))
       dissect_isakmp(pd, offset, fd, tree);
- else if (PORT_IS(UDP_PORT_RIP)) {
+  else if (PORT_IS(UDP_PORT_RIP)) {
       /* we should check the source port too (RIP: UDP src and dst port 520) */
       dissect_rip(pd, offset, fd, tree);
- } else if (PORT_IS(UDP_PORT_NBNS))
+  } else if (PORT_IS(UDP_PORT_RIPNG))
+      dissect_ripng(pd, offset, fd, tree);
+  else if (PORT_IS(UDP_PORT_NBNS))
       dissect_nbns(pd, offset, fd, tree);
- else if (PORT_IS(UDP_PORT_NBDGM))
+  else if (PORT_IS(UDP_PORT_NBDGM))
       dissect_nbdgm(pd, offset, fd, tree);
- else if (PORT_IS(UDP_PORT_IPX)) /* RFC 1234 */
+  else if (PORT_IS(UDP_PORT_NTP))
+      dissect_ntp(pd, offset, fd, tree);
+  else if (PORT_IS(UDP_PORT_IPX)) /* RFC 1234 */
       dissect_ipx(pd, offset, fd, tree);
+  else if ((uh_sport >= UDP_PORT_RX_LOW && uh_sport <= UDP_PORT_RX_HIGH) ||
+	(uh_dport >= UDP_PORT_RX_LOW && uh_dport <= UDP_PORT_RX_HIGH) ||
+	PORT_IS(UDP_PORT_RX_AFS_BACKUPS)) 
+      dissect_rx(pd, offset, fd, tree); /* transarc AFS's RX protocol */
 #if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
- else if (PORT_IS(UDP_PORT_SNMP))
+  else if (PORT_IS(UDP_PORT_SNMP))
       dissect_snmp(pd, offset, fd, tree);
 #endif
- else if (PORT_IS(UDP_PORT_VINES)) {
+  else if (PORT_IS(UDP_PORT_VINES)) {
       /* FIXME: AFAIK, src and dst port must be the same */
       dissect_vines_frp(pd, offset, fd, tree);
- } else if (PORT_IS(UDP_PORT_TFTP)) {
+  } else if (PORT_IS(UDP_PORT_TFTP)) {
       /* This is the first point of call, but it adds a dynamic call */
       udp_hash_add(MAX(uh_sport, uh_dport), dissect_tftp);  /* Add to table */
       dissect_tftp(pd, offset, fd, tree);
- } else if (PORT_IS(UDP_PORT_RADIUS) ||
+  } else if (PORT_IS(UDP_PORT_RADIUS) ||
 		PORT_IS(UDP_PORT_RADACCT) ||
 		PORT_IS(UDP_PORT_RADIUS_NEW) ||
 		PORT_IS(UDP_PORT_RADACCT_NEW) ) {
       dissect_radius(pd, offset, fd, tree);
- } else if ( PORT_IS(UDP_PORT_ICP)) {
+  } else if ( PORT_IS(UDP_PORT_ICP)) {
 	dissect_icp(pd,offset,fd,tree);
+ } else if ( PORT_IS(UDP_PORT_ICQ)) {
+        dissect_icq(pd,offset,fd,tree);
  } else {
       /* OK, find a routine in the table, else use the default */
 
@@ -283,22 +313,30 @@ proto_register_udp(void)
 {
 	static hf_register_info hf[] = {
 		{ &hf_udp_srcport,
-		{ "Source Port",	"udp.srcport", FT_UINT16, NULL }},
+		{ "Source Port",	"udp.srcport", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_udp_dstport,
-		{ "Destination Port",	"udp.dstport", FT_UINT16, NULL }},
+		{ "Destination Port",	"udp.dstport", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_udp_port,
-		{ "Source or Destination Port",	"udp.port", FT_UINT16, NULL }},
+		{ "Source or Destination Port",	"udp.port", FT_UINT16, BASE_DEC,  NULL, 0x0,
+			"" }},
 
 		{ &hf_udp_length,
-		{ "Length",		"udp.length", FT_UINT16, NULL }},
+		{ "Length",		"udp.length", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_udp_checksum,
-		{ "Checksum",		"udp.checksum", FT_UINT16, NULL }}
+		{ "Checksum",		"udp.checksum", FT_UINT16, BASE_HEX, NULL, 0x0,
+			"" }},
+	};
+	static gint *ett[] = {
+		&ett_udp,
 	};
 
 	proto_udp = proto_register_protocol("User Datagram Protocol", "udp");
 	proto_register_field_array(proto_udp, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
-
