@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c,v 1.44 1999/09/18 15:44:41 deniel Exp $
+ * $Id: packet.c,v 1.57 1999/11/27 04:01:42 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -55,22 +55,42 @@
 # include <netinet/in.h>
 #endif
 
+#include <arpa/inet.h>
+
+#ifdef NEED_INET_V6DEFS_H
+# include "inet_v6defs.h"
+#endif
+
 #include "packet.h"
+#include "print.h"
 #include "file.h"
 
 extern capture_file  cf;
 
-gboolean	tree_is_expanded[NUM_TREE_TYPES];
+static int proto_frame = -1;
+static int hf_frame_arrival_time = -1;
+static int hf_frame_time_delta = -1;
+static int hf_frame_number = -1;
+static int hf_frame_packet_len = -1;
+static int hf_frame_capture_len = -1;
 
-int proto_frame = -1;
-int hf_frame_arrival_time = -1;
-int hf_frame_time_delta = -1;
-int hf_frame_number = -1;
-int hf_frame_packet_len = -1;
-int hf_frame_capture_len = -1;
+static gint ett_frame = -1;
 
+/* Wrapper for the most common case of asking
+ * for a string using a colon as the hex-digit separator.
+ */
 gchar *
-ether_to_str(const guint8 *ad) {
+ether_to_str(const guint8 *ad)
+{
+	return ether_to_str_punct(ad, ':');
+}
+
+/* Places char punct in the string as the hex-digit separator.
+ * If punct is '\0', no punctuation is applied (and thus
+ * the resulting string is 5 bytes shorter)
+ */
+gchar *
+ether_to_str_punct(const guint8 *ad, char punct) {
   static gchar  str[3][18];
   static gchar *cur;
   gchar        *p;
@@ -95,7 +115,8 @@ ether_to_str(const guint8 *ad) {
     *--p = hex_digits[octet&0xF];
     if (i == 0)
       break;
-    *--p = ':';
+    if (punct)
+      *--p = punct;
     i--;
   }
   return p;
@@ -138,6 +159,18 @@ ip_to_str(const guint8 *ad) {
   return p;
 }
 
+gchar *
+ip6_to_str(struct e_in6_addr *ad) {
+#ifndef INET6_ADDRSTRLEN
+#define INET6_ADDRSTRLEN 46
+#endif
+  static gchar buf[INET6_ADDRSTRLEN];
+
+  inet_ntop(AF_INET6, (u_char*)ad, (gchar*)buf, sizeof(buf));
+  return buf;
+}
+
+
 #define	PLURALIZE(n)	(((n) > 1) ? "s" : "")
 #define	COMMA(do_it)	((do_it) ? ", " : "")
 
@@ -192,22 +225,21 @@ time_secs_to_str(guint32 time)
 #define	MAX_BYTE_STR_LEN	20
 
 /* Turn an array of bytes into a string showing the bytes in hex. */
+#define	N_BYTES_TO_STR_STRINGS	6
 gchar *
 bytes_to_str(const guint8 *bd, int bd_len) {
-  static gchar  str[3][MAX_BYTE_STR_LEN+3+1];
-  static gchar *cur;
+  static gchar  str[N_BYTES_TO_STR_STRINGS][MAX_BYTE_STR_LEN+3+1];
+  static int    cur_idx;
+  gchar        *cur;
   gchar        *p;
   int           len;
   static const char hex[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
                                 '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
 
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {  
-    cur = &str[2][0];
-  } else {  
-    cur = &str[0][0];
-  }
+  cur_idx++;
+  if (cur_idx >= N_BYTES_TO_STR_STRINGS)
+    cur_idx = 0;
+  cur = &str[cur_idx][0];
   p = cur;
   len = MAX_BYTE_STR_LEN;
   while (bd_len > 0 && len > 0) {
@@ -516,7 +548,7 @@ match_strval(guint32 val, const value_string *vs) {
 
 /* Generate, into "buf", a string showing the bits of a bitfield.
    Return a pointer to the character after that string. */
-static char *
+char *
 decode_bitfield_value(char *buf, guint32 val, guint32 mask, int width)
 {
   int i;
@@ -618,28 +650,118 @@ check_col(frame_data *fd, gint el) {
 /* Adds a vararg list to a packet info string. */
 void
 col_add_fstr(frame_data *fd, gint el, gchar *format, ...) {
-  va_list    ap;
-  int        i;
+  va_list ap;
+  int     i;
+  size_t  max_len;
   
   va_start(ap, format);
   for (i = 0; i < fd->cinfo->num_cols; i++) {
-    if (fd->cinfo->fmt_matx[i][el])
-      vsnprintf(fd->cinfo->col_data[i], COL_MAX_LEN, format, ap);
+    if (fd->cinfo->fmt_matx[i][el]) {
+      if (el == COL_INFO)
+	max_len = COL_MAX_INFO_LEN;
+      else
+	max_len = COL_MAX_LEN;
+      vsnprintf(fd->cinfo->col_data[i], max_len, format, ap);
+    }
   }
 }
 
 void
 col_add_str(frame_data *fd, gint el, const gchar* str) {
-  int i;
-  
+  int    i;
+  size_t max_len;
+
   for (i = 0; i < fd->cinfo->num_cols; i++) {
     if (fd->cinfo->fmt_matx[i][el]) {
-      strncpy(fd->cinfo->col_data[i], str, COL_MAX_LEN);
-      fd->cinfo->col_data[i][COL_MAX_LEN - 1] = 0;
+      if (el == COL_INFO)
+	max_len = COL_MAX_INFO_LEN;
+      else
+	max_len = COL_MAX_LEN;
+      strncpy(fd->cinfo->col_data[i], str, max_len);
+      fd->cinfo->col_data[i][max_len - 1] = 0;
     }
   }
 }
 
+/* Appends a vararg list to a packet info string. */
+void
+col_append_fstr(frame_data *fd, gint el, gchar *format, ...) {
+  va_list ap;
+  int     i;
+  size_t  len, max_len;
+  
+  va_start(ap, format);
+  for (i = 0; i < fd->cinfo->num_cols; i++) {
+    if (fd->cinfo->fmt_matx[i][el]) {
+      len = strlen(fd->cinfo->col_data[i]);
+      if (el == COL_INFO)
+	max_len = COL_MAX_INFO_LEN;
+      else
+	max_len = COL_MAX_LEN;
+      vsnprintf(&fd->cinfo->col_data[i][len], max_len - len, format, ap);
+    }
+  }
+}
+
+void
+col_append_str(frame_data *fd, gint el, gchar* str) {
+  int    i;
+  size_t len, max_len;
+
+  for (i = 0; i < fd->cinfo->num_cols; i++) {
+    if (fd->cinfo->fmt_matx[i][el]) {
+      len = strlen(fd->cinfo->col_data[i]);
+      if (el == COL_INFO)
+	max_len = COL_MAX_LEN;
+      else
+	max_len = COL_MAX_INFO_LEN;
+      strncat(fd->cinfo->col_data[i], str, max_len - len);
+      fd->cinfo->col_data[i][max_len - 1] = 0;
+    }
+  }
+}
+	
+void blank_packetinfo(void)
+{
+  pi.dl_src.type = AT_NONE;
+  pi.dl_dst.type = AT_NONE;
+  pi.net_src.type = AT_NONE;
+  pi.net_dst.type = AT_NONE;
+  pi.src.type = AT_NONE;
+  pi.dst.type = AT_NONE;
+  pi.ipproto  = 0;
+  pi.ptype = PT_NONE;
+  pi.srcport  = 0;
+  pi.destport = 0;
+}
+
+/* Allow protocols to register "init" routines, which are called before
+   we make a pass through a capture file and dissect all its packets
+   (e.g., when we read in a new capture file, or run a "filter packets"
+   or "colorize packets" pass over the current capture file). */
+static GSList *init_routines;
+
+void
+register_init_routine(void (*func)(void))
+{
+	init_routines = g_slist_append(init_routines, func);
+}
+
+/* Call all the registered "init" routines. */
+static void
+call_init_routine(gpointer routine, gpointer dummy)
+{
+	void (*func)(void) = routine;
+
+	(*func)();
+}
+
+void
+init_all_protocols(void)
+{
+	g_slist_foreach(init_routines, &call_init_routine, NULL);
+}
+
 /* this routine checks the frame type from the cf structure */
 void
 dissect_packet(const u_char *pd, frame_data *fd, proto_tree *tree)
@@ -653,7 +775,7 @@ dissect_packet(const u_char *pd, frame_data *fd, proto_tree *tree)
 	  ti = proto_tree_add_item_format(tree, proto_frame, 0, fd->cap_len,
 	    NULL, "Frame (%d on wire, %d captured)", fd->pkt_len, fd->cap_len);
 
-	  fh_tree = proto_item_add_subtree(ti, ETT_FRAME);
+	  fh_tree = proto_item_add_subtree(ti, ett_frame);
 
 	  tv.tv_sec = fd->abs_secs;
 	  tv.tv_usec = fd->abs_usecs;
@@ -679,6 +801,8 @@ dissect_packet(const u_char *pd, frame_data *fd, proto_tree *tree)
 		plurality(fd->cap_len, "", "s"));
 	}
 
+	blank_packetinfo();
+
 	/* Set the initial payload to the packet length, and the initial
 	   captured payload to the capture length (other protocols may
 	   reduce them if their headers say they're less). */
@@ -719,6 +843,9 @@ dissect_packet(const u_char *pd, frame_data *fd, proto_tree *tree)
 		case WTAP_ENCAP_ASCEND :
 			dissect_ascend(pd, fd, tree);
 			break;
+		case WTAP_ENCAP_LAPD :
+			dissect_lapd(pd, fd, tree);
+			break;
 	}
 }
 
@@ -727,21 +854,31 @@ proto_register_frame(void)
 {
 	static hf_register_info hf[] = {
 		{ &hf_frame_arrival_time,
-		{ "Arrival Time",		"frame.time", FT_ABSOLUTE_TIME, NULL }},
+		{ "Arrival Time",		"frame.time", FT_ABSOLUTE_TIME, BASE_NONE, NULL, 0x0,
+			""}},
 
 		{ &hf_frame_time_delta,
-		{ "Time delta from previous packet",	"frame.time_delta", FT_RELATIVE_TIME, NULL }},
+		{ "Time delta from previous packet",	"frame.time_delta", FT_RELATIVE_TIME, BASE_NONE, NULL,
+			0x0,
+			"" }},
 
 		{ &hf_frame_number,
-		{ "Frame Number",		"frame.number", FT_UINT32, NULL }},
+		{ "Frame Number",		"frame.number", FT_UINT32, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_frame_packet_len,
-		{ "Total Frame Length",		"frame.pkt_len", FT_UINT32, NULL }},
+		{ "Total Frame Length",		"frame.pkt_len", FT_UINT32, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_frame_capture_len,
-		{ "Capture Frame Length",	"frame.cap_len", FT_UINT32, NULL }}
+		{ "Capture Frame Length",	"frame.cap_len", FT_UINT32, BASE_DEC, NULL, 0x0,
+			"" }},
+	};
+	static gint *ett[] = {
+		&ett_frame,
 	};
 
 	proto_frame = proto_register_protocol("Frame", "frame");
 	proto_register_field_array(proto_frame, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
