@@ -1,7 +1,7 @@
 /* prefs.c
  * Routines for handling preferences
  *
- * $Id: prefs.c,v 1.16 1999/03/01 18:57:01 gram Exp $
+ * $Id: prefs.c,v 1.22 1999/09/09 03:31:50 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,173 +31,38 @@
 #include <sys/types.h>
 #endif
 
-#include <gtk/gtk.h>
+#ifdef HAVE_DIRECT_H
+#include <direct.h>
+#endif
 
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
 #include <sys/stat.h>
 
-#include "ethereal.h"
+#include "globals.h"
 #include "packet.h"
 #include "file.h"
 #include "prefs.h"
 #include "column.h"
 #include "print.h"
-#include "filter.h"
 #include "util.h"
 
 /* Internal functions */
 static int    set_pref(gchar*, gchar*);
-static void   write_prefs();
-static void   prefs_main_ok_cb(GtkWidget *, gpointer);
-static void   prefs_main_save_cb(GtkWidget *, gpointer);
-static void   prefs_main_cancel_cb(GtkWidget *, gpointer);
 static GList *get_string_list(gchar *);
 static void   clear_string_list(GList *);
 
-e_prefs prefs;
-static int init_prefs = 1;
-
 #define PF_NAME "preferences"
 
-#define E_PRINT_PAGE_KEY  "printer_options_page"
-#define E_FILTER_PAGE_KEY "filter_options_page"
-#define E_COLUMN_PAGE_KEY "column_options_page"
-
+static int init_prefs = 1;
 static gchar *pf_path = NULL;
 
-void
-prefs_cb(GtkWidget *w, gpointer sp) {
-  GtkWidget *prefs_w, *main_vb, *top_hb, *bbox, *prefs_nb,
-            *ok_bt, *save_bt, *cancel_bt;
-  GtkWidget *print_pg, *filter_pg, *column_pg, *filter_te, *label;
-
-/*  GtkWidget *nlabel; */
-  gint       start_page = (gint) sp;
-
-  filter_pg = NULL;
-  filter_te = NULL;
-
-  prefs_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(prefs_w), "Ethereal: Preferences");
-  
-  /* Container for each row of widgets */
-  main_vb = gtk_vbox_new(FALSE, 5);
-  gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
-  gtk_container_add(GTK_CONTAINER(prefs_w), main_vb);
-  gtk_widget_show(main_vb);
-  
-  /* Top row: Preferences notebook */
-  top_hb = gtk_hbox_new(FALSE, 1);
-  gtk_container_add(GTK_CONTAINER(main_vb), top_hb);
-  gtk_widget_show(top_hb);
-  
-  prefs_nb = gtk_notebook_new();
-  gtk_container_add(GTK_CONTAINER(main_vb), prefs_nb);
-  gtk_widget_show(prefs_nb);
-  
-  /* General prefs */
-/*   nlabel = gtk_label_new("Nothing here yet...");
-  gtk_widget_show (nlabel);
-
-  label = gtk_label_new ("General");
-  gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), nlabel, label);
- */  
-  /* Printing prefs */
-  print_pg = printer_prefs_show();
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_PRINT_PAGE_KEY, print_pg);
-  label = gtk_label_new ("Printing");
-  gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), print_pg, label);
-    
-  /* Filter prefs */
-  if (w)
-    filter_te = gtk_object_get_data(GTK_OBJECT(w), E_FILT_TE_PTR_KEY);
-  filter_pg = filter_prefs_show(filter_te);
-
-  /* Pass along the entry widget pointer from the calling widget */
-  gtk_object_set_data(GTK_OBJECT(filter_pg), E_FILT_TE_PTR_KEY, filter_te);
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_FILTER_PAGE_KEY, filter_pg);
-  label = gtk_label_new ("Filters");
-  gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), filter_pg, label);
-  /* Column prefs */
-  column_pg = column_prefs_show();
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_COLUMN_PAGE_KEY, column_pg);
-  label = gtk_label_new ("Columns");
-  gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), column_pg, label);
-  
-  /* Jump to the specified page, if it was supplied */
-  if (start_page > E_PR_PG_NONE)
-    gtk_notebook_set_page(GTK_NOTEBOOK(prefs_nb), start_page);
-    
-  /* Button row: OK and cancel buttons */
-  bbox = gtk_hbutton_box_new();
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
-  gtk_container_add(GTK_CONTAINER(main_vb), bbox);
-  gtk_widget_show(bbox);
-  
-  ok_bt = gtk_button_new_with_label ("OK");
-  gtk_signal_connect_object(GTK_OBJECT(ok_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_main_ok_cb), GTK_OBJECT(prefs_w));
-  GTK_WIDGET_SET_FLAGS(ok_bt, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (bbox), ok_bt, TRUE, TRUE, 0);
-  gtk_widget_grab_default(ok_bt);
-  gtk_widget_show(ok_bt);
-
-  save_bt = gtk_button_new_with_label ("Save");
-  gtk_signal_connect_object(GTK_OBJECT(save_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_main_save_cb), GTK_OBJECT(prefs_w));
-  GTK_WIDGET_SET_FLAGS(save_bt, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (bbox), save_bt, TRUE, TRUE, 0);
-  gtk_widget_show(save_bt);
-  
-  cancel_bt = gtk_button_new_with_label ("Cancel");
-  gtk_signal_connect_object(GTK_OBJECT(cancel_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_main_cancel_cb), GTK_OBJECT(prefs_w));
-  GTK_WIDGET_SET_FLAGS(cancel_bt, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (bbox), cancel_bt, TRUE, TRUE, 0);
-  gtk_widget_show(cancel_bt);
-
-  gtk_widget_show(prefs_w);
-}
-
-void
-prefs_main_ok_cb(GtkWidget *w, gpointer win) {
-
-#ifdef GTK_HAVE_FEATURES_1_1_0
-  win = w;
-#endif
-  printer_prefs_ok(gtk_object_get_data(GTK_OBJECT(win), E_PRINT_PAGE_KEY));
-  filter_prefs_ok(gtk_object_get_data(GTK_OBJECT(win), E_FILTER_PAGE_KEY));
-  column_prefs_ok(gtk_object_get_data(GTK_OBJECT(win), E_COLUMN_PAGE_KEY));
-  gtk_widget_destroy(GTK_WIDGET(win));
-}
-
-void
-prefs_main_save_cb(GtkWidget *w, gpointer win) {
-#ifdef GTK_HAVE_FEATURES_1_1_0
-  win = w;
-#endif
-  printer_prefs_save(gtk_object_get_data(GTK_OBJECT(win), E_PRINT_PAGE_KEY));
-  filter_prefs_save(gtk_object_get_data(GTK_OBJECT(win), E_FILTER_PAGE_KEY));
-  column_prefs_save(gtk_object_get_data(GTK_OBJECT(win), E_COLUMN_PAGE_KEY));
-  write_prefs();
-}
-
-void
-prefs_main_cancel_cb(GtkWidget *w, gpointer win) {
-
-#ifdef GTK_HAVE_FEATURES_1_1_0
-  win = w;
-#endif
-  printer_prefs_cancel(gtk_object_get_data(GTK_OBJECT(win), E_PRINT_PAGE_KEY));
-  filter_prefs_cancel(gtk_object_get_data(GTK_OBJECT(win), E_FILTER_PAGE_KEY));
-  column_prefs_cancel(gtk_object_get_data(GTK_OBJECT(win), E_COLUMN_PAGE_KEY));
-  gtk_widget_destroy(GTK_WIDGET(win));
-}
-
 /* Parse through a list of comma-separated, quoted strings.  Return a
    list of the string data */
 static GList *
@@ -274,7 +139,7 @@ print.file: /a/very/long/path/
 #define MAX_VAL_LEN  1024
 #define DEF_NUM_COLS    6
 e_prefs *
-read_prefs() {
+read_prefs(char **pf_path_return) {
   enum { START, IN_VAR, PRE_VAL, IN_VAL, IN_SKIP };
   FILE     *pf;
   gchar     cur_var[MAX_VAR_LEN], cur_val[MAX_VAL_LEN];
@@ -311,11 +176,10 @@ read_prefs() {
     sprintf(pf_path, "%s/%s/%s", getenv("HOME"), PF_DIR, PF_NAME);
   }
     
+  *pf_path_return = NULL;
   if ((pf = fopen(pf_path, "r")) == NULL) {
-    if (errno != ENOENT) {
-      simple_dialog(ESD_TYPE_WARN, NULL,
-        "Can't open preferences file\n\"%s\".", pf_path);
-    }
+    if (errno != ENOENT)
+      *pf_path_return = pf_path;
     return &prefs;
   }
     
@@ -476,7 +340,7 @@ set_pref(gchar *pref, gchar *value) {
 }
 
 void
-write_prefs() {
+write_prefs(void) {
   FILE        *pf;
   struct stat  s_buf;
   
@@ -493,7 +357,11 @@ write_prefs() {
 
   sprintf(pf_path, "%s/%s", getenv("HOME"), PF_DIR);
   if (stat(pf_path, &s_buf) != 0)
+#ifdef WIN32
+    mkdir(pf_path);
+#else
     mkdir(pf_path, 0755);
+#endif
 
   sprintf(pf_path, "%s/%s/%s", getenv("HOME"), PF_DIR, PF_NAME);
   if ((pf = fopen(pf_path, "w")) == NULL) {
