@@ -1,7 +1,7 @@
 /* print.c
  * Routines for printing packet analysis trees.
  *
- * $Id: print.c,v 1.20 1999/09/12 20:23:33 guy Exp $
+ * $Id: print.c,v 1.24 1999/11/22 08:03:31 guy Exp $
  *
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
@@ -41,13 +41,16 @@
 #include "prefs.h"
 #include "print.h"
 #include "ps.h"
+#include "util.h"
 
 static void proto_tree_print_node_text(GNode *node, gpointer data);
-static void dumpit (FILE *fh, register const u_char *cp, register u_int length);
 static void proto_tree_print_node_ps(GNode *node, gpointer data);
 static void ps_clean_string(unsigned char *out, const unsigned char *in,
 			int outbuf_size);
-static void dumpit_ps (FILE *fh, register const u_char *cp, register u_int length);
+static void print_hex_data_ps(FILE *fh, register const u_char *cp,
+		register u_int length, char_enc encoding);
+static void print_ps_file(FILE* target_fh, FILE* source_fh);
+static void print_text_file(FILE* target_fh, FILE* source_fh);
 
 extern int proto_data; /* in packet-data.c */
 
@@ -57,6 +60,8 @@ typedef struct {
 	FILE		*fh;
 	const guint8	*pd;
 	gboolean	print_all_levels;
+	gboolean	print_hex_for_data;
+	char_enc	encoding;
 } print_data;
 
 FILE *open_print_dest(int to_file, const char *dest)
@@ -93,7 +98,21 @@ void print_finale(FILE *fh)
 		print_ps_finale(fh);
 }
 
-void proto_tree_print(gboolean print_one_packet, gboolean print_all_levels,
+void print_file(FILE* fh, const char* filename)
+{
+       FILE* fh2 = fopen(filename, "r");
+       if (fh2 == NULL) {
+               fprintf(stderr, "Could not open file %s for reading.\n", filename);
+               return;
+       }
+
+       if (prefs.pr_format == PR_FMT_PS)
+               print_ps_file(fh, fh2);
+       else
+               print_text_file(fh, fh2);
+}
+
+void proto_tree_print(gboolean print_one_packet, print_args_t *print_args,
     GNode *protocol_tree, const u_char *pd, frame_data *fd, FILE *fh)
 {
 	print_data data;
@@ -102,7 +121,11 @@ void proto_tree_print(gboolean print_one_packet, gboolean print_all_levels,
 	data.level = 0;
 	data.fh = fh;
 	data.pd = pd;
-	data.print_all_levels = print_all_levels;
+	data.encoding = fd->encoding;
+	data.print_all_levels = print_args->expand_all;
+	data.print_hex_for_data = !print_args->print_hex;
+	    /* If we're printing the entire packet in hex, don't
+	       print uninterpreted data fields in hex as well. */
 
 	/* XXX - printing multiple frames in PostScript looks as if it's
 	   tricky - you have to deal with page boundaries, I think -
@@ -157,9 +180,11 @@ void proto_tree_print_node_text(GNode *node, gpointer data)
 	/* Print the text */
 	fprintf(pdata->fh, "%s%s\n", space, label_ptr);
 
-	/* If it's uninterpreted data, dump it. */
-	if (fi->hfinfo->id == proto_data)
-		dumpit(pdata->fh, &pdata->pd[fi->start], fi->length);
+	/* If it's uninterpreted data, dump it (unless our caller will
+	   be printing the entire packet in hex). */
+	if (fi->hfinfo->id == proto_data && pdata->print_hex_for_data)
+		print_hex_data(pdata->fh, &pdata->pd[fi->start], fi->length,
+				pdata->encoding);
 
 	/* If we're printing all levels, or if this level is expanded,
 	   recurse into the subtree, if it exists. */
@@ -175,8 +200,8 @@ void proto_tree_print_node_text(GNode *node, gpointer data)
 
 /* This routine was created by Dan Lasley <DLASLEY@PROMUS.com>, and
 only slightly modified for ethereal by Gilbert Ramirez. */
-static
-void dumpit (FILE *fh, register const u_char *cp, register u_int length)
+void print_hex_data(FILE *fh, register const u_char *cp, register u_int length,
+		char_enc encoding)
 {
         register int ad, i, j, k;
         u_char c;
@@ -192,6 +217,9 @@ void dumpit (FILE *fh, register const u_char *cp, register u_int length)
                 line[j++] = binhex[c>>4];
                 line[j++] = binhex[c&0xf];
                 if (i&1) j++;
+		if (encoding == CHAR_EBCDIC) {
+			c = EBCDIC_to_ASCII1(c);
+		}
                 line[42+k++] = c >= ' ' && c < 0x7f ? c : '.';
                 if ((i & 15) == 15) {
                         fprintf (fh, "\n%4x  %s", ad, line);
@@ -239,7 +267,8 @@ void proto_tree_print_node_ps(GNode *node, gpointer data)
 	/* If it's uninterpreted data, dump it. */
 	if (fi->hfinfo->id == proto_data) {
 		print_ps_hex(pdata->fh);
-		dumpit_ps(pdata->fh, &pdata->pd[fi->start], fi->length);
+		print_hex_data_ps(pdata->fh, &pdata->pd[fi->start], fi->length,
+				pdata->encoding);
 	}
 
 	/* Recurse into the subtree, if it exists */
@@ -280,7 +309,8 @@ void ps_clean_string(unsigned char *out, const unsigned char *in,
 }
 
 static
-void dumpit_ps (FILE *fh, register const u_char *cp, register u_int length)
+void print_hex_data_ps(FILE *fh, register const u_char *cp, register u_int length,
+		char_enc encoding)
 {
         register int ad, i, j, k;
         u_char c;
@@ -297,9 +327,12 @@ void dumpit_ps (FILE *fh, register const u_char *cp, register u_int length)
                 line[j++] = binhex[c>>4];
                 line[j++] = binhex[c&0xf];
                 if (i&1) j++;
+		if (encoding == CHAR_EBCDIC) {
+			c = EBCDIC_to_ASCII1(c);
+		}
                 line[42+k++] = c >= ' ' && c < 0x7f ? c : '.';
                 if ((i & 15) == 15) {
-						ps_clean_string(psline, line, MAX_LINE_LENGTH);
+			ps_clean_string(psline, line, MAX_LINE_LENGTH);
                         fprintf (fh, "(%4x  %s) hexdump\n", ad, psline);
                         memset (line, ' ', sizeof line);
                         line[sizeof (line)-1] = j = k = 0;
@@ -314,3 +347,25 @@ void dumpit_ps (FILE *fh, register const u_char *cp, register u_int length)
         return;
 
 }
+
+static 
+void print_text_file(FILE* target_fh, FILE* source_fh)
+{
+       gchar buffer[MAX_LINE_LENGTH];
+       while (fgets(buffer, sizeof(buffer), source_fh) != NULL) {
+               fputs(buffer, target_fh);
+       }
+}
+
+static 
+void print_ps_file(FILE* target_fh, FILE* source_fh)
+{
+       gchar buffer[MAX_LINE_LENGTH];
+       gchar ps_buffer[MAX_LINE_LENGTH];
+
+       while (fgets(buffer, sizeof(buffer), source_fh) != NULL) {
+               ps_clean_string(ps_buffer, buffer, MAX_LINE_LENGTH);
+               fputs(ps_buffer, target_fh);
+       }
+}
+
