@@ -1,7 +1,7 @@
 /* proto.c
  * Routines for protocol tree
  *
- * $Id: proto.c,v 1.29 1999/09/18 15:44:40 deniel Exp $
+ * $Id: proto.c,v 1.50 1999/11/21 16:32:15 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -61,6 +61,12 @@
 #include "resolv.h"
 #endif
 
+#ifndef __REGISTER_H__
+#include "register.h"
+#endif
+
+#include "packet-ipv6.h"
+
 #define cVALS(x) (const value_string*)(x)
 
 #if defined(HAVE_UCD_SNMP_SNMP_H)
@@ -79,69 +85,25 @@ static proto_item *
 proto_tree_add_item_value(proto_tree *tree, int hfindex, gint start,
 	gint length, int include_format, int visible, va_list ap);
 
+static void fill_label_boolean(field_info *fi, gchar *label_str);
+static void fill_label_uint(field_info *fi, gchar *label_str);
+static void fill_label_enumerated_uint(field_info *fi, gchar *label_str);
+static void fill_label_enumerated_bitfield(field_info *fi, gchar *label_str);
+static void fill_label_numeric_bitfield(field_info *fi, gchar *label_str);
+static void fill_label_int(field_info *fi, gchar *label_str);
+static void fill_label_enumerated_int(field_info *fi, gchar *label_str);
+
+static int hfinfo_bitwidth(header_field_info *hfinfo);
+static char* hfinfo_uint_vals_format(header_field_info *hfinfo);
+static char* hfinfo_uint_format(header_field_info *hfinfo);
+static char* hfinfo_int_vals_format(header_field_info *hfinfo);
+static char* hfinfo_int_format(header_field_info *hfinfo);
+
 static gboolean check_for_protocol_or_field_id(GNode *node, gpointer data);
 static gboolean check_for_field_within_protocol(GNode *node, gpointer data);
 
 static int proto_register_field_init(header_field_info *hfinfo, int parent);
 
-/* centralization of registration functions */
-void proto_register_aarp(void);
-void proto_register_arp(void);
-void proto_register_ascend(void);
-void proto_register_atalk(void);
-void proto_register_atm(void);
-void proto_register_bootp(void);
-void proto_register_cdp(void);
-void proto_register_clnp(void);
-void proto_register_cotp(void);
-void proto_register_data(void);
-void proto_register_dns(void);
-void proto_register_eth(void);
-void proto_register_fddi(void);
-void proto_register_frame(void);
-void proto_register_ftp(void);
-void proto_register_giop(void);
-void proto_register_gre(void);
-void proto_register_http(void);
-void proto_register_icmp(void);
-void proto_register_icmpv6(void);
-void proto_register_icp(void);
-void proto_register_igmp(void);
-void proto_register_ip(void);
-void proto_register_ipp(void);
-void proto_register_ipsec(void);
-void proto_register_ipv6(void);
-void proto_register_ipx(void);
-void proto_register_isakmp(void);
-void proto_register_lapb(void);
-void proto_register_llc(void);
-void proto_register_mp(void);
-void proto_register_nbipx(void);
-void proto_register_nbt(void);
-void proto_register_ncp(void);
-void proto_register_netbios(void);
-void proto_register_nntp(void);
-void proto_register_null(void);
-void proto_register_ospf(void);
-void proto_register_pop(void);
-void proto_register_ppp(void);
-void proto_register_radius(void);
-void proto_register_rip(void);
-void proto_register_rsvp(void);
-void proto_register_rtsp(void);
-void proto_register_sdp(void);
-void proto_register_smb(void);
-#if defined(WITH_SNMP_CMU) || defined(WITH_SNMP_UCD)
-void proto_register_snmp(void);
-#endif
-void proto_register_telnet(void);
-void proto_register_tftp(void);
-void proto_register_tcp(void);
-void proto_register_tr(void);
-void proto_register_trmac(void);
-void proto_register_udp(void);
-void proto_register_x25(void);
-
 /* special-case header field used within proto.c */
 int hf_text_only = 1;
 
@@ -159,6 +121,14 @@ GMemChunk *gmc_item_labels = NULL;
 /* List which stores protocols and fields that have been registered */
 GPtrArray *gpa_hfinfo = NULL;
 
+/* Points to the first element of an array of Booleans, indexed by
+   a subtree item type; that array element is TRUE if subtrees of
+   an item of that type are to be expanded. */
+gboolean	*tree_is_expanded;
+
+/* Number of elements in that array. */
+int		num_tree_types;
+
 /* Is the parsing being done for a visible proto_tree or an invisible one?
  * By setting this correctly, the proto_tree creation is sped up by not
  * having to call vsnprintf and copy strings around.
@@ -169,6 +139,12 @@ gboolean proto_tree_is_visible = TRUE;
 void
 proto_init(void)
 {
+	static hf_register_info hf[] = {
+		{ &hf_text_only,
+		{ "Text",	"text", FT_TEXT_ONLY, BASE_NONE, NULL, 0x0,
+			"" }},
+	};
+
 	if (gmc_hfinfo)
 		g_mem_chunk_destroy(gmc_hfinfo);
 	if (gmc_field_info)
@@ -177,6 +153,8 @@ proto_init(void)
 		g_mem_chunk_destroy(gmc_item_labels);
 	if (gpa_hfinfo)
 		g_ptr_array_free(gpa_hfinfo, FALSE);
+	if (tree_is_expanded != NULL)
+		g_free(tree_is_expanded);
 
 	gmc_hfinfo = g_mem_chunk_new("gmc_hfinfo",
 		sizeof(struct header_field_info), 50 * sizeof(struct 
@@ -189,75 +167,24 @@ proto_init(void)
 		G_ALLOC_AND_FREE);
 	gpa_hfinfo = g_ptr_array_new();
 
-	/* Have each dissector register its protocols and fields. The
-	 * order doesn't matter. Put the calls in alphabetical order
-	 * just to make it easy. */
-	proto_register_aarp();
-	proto_register_arp();
-	proto_register_ascend();
-	proto_register_atalk();
-	proto_register_atm();
-	proto_register_bootp();
-	proto_register_cdp();
-	proto_register_clnp();
-	proto_register_cotp();
-	proto_register_data();
-	proto_register_dns();
-	proto_register_eth();
-	proto_register_fddi();
-	proto_register_frame();
-	proto_register_ftp();
-	proto_register_giop();
-	proto_register_gre();
-	proto_register_http();
-	proto_register_icmp();
-	proto_register_icmpv6();
-	proto_register_icp();
-	proto_register_igmp();
-	proto_register_ip();
-	proto_register_ipp();
-	proto_register_ipsec();
-	proto_register_ipv6();
-	proto_register_ipx();
-	proto_register_isakmp();
-	proto_register_lapb();
-	proto_register_llc();
-	proto_register_mp();
-	proto_register_nbipx();
-	proto_register_nbt();
-	proto_register_ncp();
-	proto_register_netbios();
-	proto_register_nntp();
-	proto_register_null();
-	proto_register_ospf();
-	proto_register_pop();
-	proto_register_ppp();
-	proto_register_radius();
-	proto_register_rip();
-	proto_register_rsvp();
-	proto_register_rtsp();
-	proto_register_sdp();
-	proto_register_smb();
-#if defined(WITH_SNMP_CMU) || defined(WITH_SNMP_UCD)
-	proto_register_snmp();
-#endif
-	proto_register_telnet();
-	proto_register_tftp();
-	proto_register_tcp();
-	proto_register_tr();
-	proto_register_trmac();
-	proto_register_udp();
-	proto_register_x25();
+	/* Allocate "tree_is_expanded", with one element for ETT_NONE,
+	   and initialize that element to FALSE. */
+	tree_is_expanded = g_malloc(sizeof (gint));
+	tree_is_expanded[0] = FALSE;
+	num_tree_types = 1;
+
+	/* Have each dissector register its protocols and fields. */
+	register_all_protocols();
 
 	/* Register one special-case FT_TEXT_ONLY field for use when
 		converting ethereal to new-style proto_tree. These fields
 		are merely strings on the GUI tree; they are not filterable */
-	hf_text_only = proto_register_field (
-		/* name */	"Text",
-		/* abbrev */	"text",
-		/* ftype */	FT_TEXT_ONLY,
-		/* parent */	-1,
-		/* vals[] */	NULL );
+	proto_register_field_array(-1, hf, array_length(hf));
+
+	/* We've assigned all the subtree type values; allocate the array
+	   for them, and zero it out. */
+	tree_is_expanded = g_malloc(num_tree_types*sizeof (gint *));
+	memset(tree_is_expanded, '\0', num_tree_types*sizeof (gint *));
 }
 
 void
@@ -366,6 +293,7 @@ proto_tree_add_item_value(proto_tree *tree, int hfindex, gint start,
 	proto_item	*pi;
 	field_info	*fi;
 	char		*junk, *format;
+	header_field_info *hfinfo;
 
 	if (!tree)
 		return(NULL);
@@ -382,6 +310,9 @@ proto_tree_add_item_value(proto_tree *tree, int hfindex, gint start,
 	fi->tree_type = ETT_NONE;
 	fi->visible = visible;
 
+	/* for convenience */
+
+	hfinfo = fi->hfinfo;
 /* from the stdarg man page on Solaris 2.6:
 NOTES
      It is up to the calling routine to specify  in  some  manner
@@ -396,15 +327,11 @@ NOTES
      converts  float arguments to double before passing them to a
      function.
 */
-	switch(fi->hfinfo->type) {
+	switch(hfinfo->type) {
 		case FT_NONE:
 			junk = va_arg(ap, guint8*);
 			break;
 
-		case FT_BOOLEAN:
-			fi->value.numeric = va_arg(ap, unsigned int) ? TRUE : FALSE;
-			break;
-
 		case FT_BYTES:
 			/* This g_malloc'ed memory is freed in
 			   proto_tree_free_node() */
@@ -412,24 +339,40 @@ NOTES
 			memcpy(fi->value.bytes, va_arg(ap, guint8*), length);
 			break;
 
+		case FT_BOOLEAN:
 		case FT_UINT8:
-		case FT_VALS_UINT8:
-			fi->value.numeric = va_arg(ap, unsigned int);
-			break;
-
 		case FT_UINT16:
-		case FT_VALS_UINT16:
+		case FT_UINT24:
+		case FT_UINT32:
+		case FT_INT8:
+		case FT_INT16:
+		case FT_INT24:
+		case FT_INT32:
 			fi->value.numeric = va_arg(ap, unsigned int);
+			if (hfinfo->bitmask) {
+				/* Mask out irrelevant portions */
+				fi->value.numeric &= hfinfo->bitmask;
+
+				/* Shift bits */
+				if (hfinfo->bitshift > 0) {
+					fi->value.numeric >>= hfinfo->bitshift;
+				}
+			}
 			break;
 
-		case FT_UINT32:
-		case FT_VALS_UINT24:
-		case FT_VALS_UINT32:
 		case FT_IPv4:
+			ipv4_addr_set_net_order_addr(&(fi->value.ipv4), va_arg(ap, unsigned int));
+			ipv4_addr_set_netmask_bits(&(fi->value.ipv4), 32);
+			break;
+
 		case FT_IPXNET:
 			fi->value.numeric = va_arg(ap, unsigned int);
 			break;
 
+		case FT_IPv6:
+			memcpy(fi->value.ipv6, va_arg(ap, guint8*), 16);
+			break;
+
 		case FT_DOUBLE:
 			fi->value.floating = va_arg(ap, double);
 			break;
@@ -454,7 +397,7 @@ NOTES
 			break;
 
 		default:
-			g_error("hfinfo->type %d not handled\n", fi->hfinfo->type);
+			g_error("hfinfo->type %d not handled\n", hfinfo->type);
 			break;
 	}
 
@@ -491,6 +434,7 @@ proto_tree_create_root(void)
 proto_tree*
 proto_item_add_subtree(proto_item *pi,  gint idx) {
 	field_info *fi = (field_info*) (((GNode*)pi)->data);
+	g_assert(idx >= 0 && idx < num_tree_types);
 	fi->tree_type = idx;
 	return (proto_tree*) pi;
 }
@@ -499,7 +443,20 @@ proto_item_add_subtree(proto_item *pi,  gint idx) {
 int
 proto_register_protocol(char *name, char *abbrev)
 {
-	return proto_register_field(name, abbrev, FT_NONE, -1, NULL);
+	struct header_field_info *hfinfo;
+
+	/* Here we do allocate a new header_field_info struct */
+	hfinfo = g_mem_chunk_alloc(gmc_hfinfo);
+	hfinfo->name = name;
+	hfinfo->abbrev = abbrev;
+	hfinfo->type = FT_NONE;
+	hfinfo->strings = NULL;
+	hfinfo->bitmask = 0;
+	hfinfo->bitshift = 0;
+	hfinfo->blurb = "";
+	hfinfo->parent = -1; /* this field differentiates protos and fields */
+
+	return proto_register_field_init(hfinfo, hfinfo->parent);
 }
 
 /* for use with static arrays only, since we don't allocate our own copies
@@ -516,29 +473,26 @@ proto_register_field_array(int parent, hf_register_info *hf, int num_records)
 	}
 }
 
-
-/* Here we do allocate a new header_field_info struct */
-int
-proto_register_field(char *name, char *abbrev, enum ftenum type, int parent,
-	struct value_string* vals)
-{
-	struct header_field_info *hfinfo;
-
-	hfinfo = g_mem_chunk_alloc(gmc_hfinfo);
-	hfinfo->name = name; /* should I g_strdup? */
-	hfinfo->abbrev = abbrev; /* should I g_strdup? */
-	hfinfo->type = type;
-	hfinfo->vals = vals;
-	hfinfo->parent = parent; /* this field differentiates protos and fields */
-
-	return proto_register_field_init(hfinfo, parent);
-}
-
 static int
 proto_register_field_init(header_field_info *hfinfo, int parent)
 {
-	g_assert((hfinfo->vals == NULL) || (hfinfo->type == FT_VALS_UINT8 || hfinfo->type == FT_VALS_UINT16 ||
-		hfinfo->type == FT_VALS_UINT24 || hfinfo->type == FT_VALS_UINT32));
+	/* These types of fields are allowed to have value_strings or true_false_strings */
+	g_assert((hfinfo->strings == NULL) || (
+			(hfinfo->type == FT_UINT8) ||
+			(hfinfo->type == FT_UINT16) ||
+			(hfinfo->type == FT_UINT24) ||
+			(hfinfo->type == FT_UINT32) ||
+			(hfinfo->type == FT_INT8) ||
+			(hfinfo->type == FT_INT16) ||
+			(hfinfo->type == FT_INT24) ||
+			(hfinfo->type == FT_INT32) ||
+			(hfinfo->type == FT_BOOLEAN) ));
+
+	/* if this is a bitfield, compure bitshift */
+	if (hfinfo->bitmask) {
+		while ((hfinfo->bitmask & (1 << hfinfo->bitshift)) == 0)
+			hfinfo->bitshift++;
+	}
 
 	hfinfo->parent = parent;
 
@@ -548,35 +502,92 @@ proto_register_field_init(header_field_info *hfinfo, int parent)
 	return hfinfo->id;
 }
 
+void
+proto_register_subtree_array(gint **indices, int num_indices)
+{
+	int	i;
+	gint	**ptr = indices;
+
+	/*
+	 * Add "num_indices" elements to "tree_is_expanded".
+	 */
+	tree_is_expanded = g_realloc(tree_is_expanded,
+	    (num_tree_types + num_indices)*sizeof (gint));
+
+	/*
+	 * Assign "num_indices" subtree numbers starting at "num_tree_types",
+	 * returning the indices through the pointers in the array whose
+	 * first element is pointed to by "indices", set to FALSE the
+	 * elements to which those subtree numbers refer, and update
+	 * "num_tree_types" appropriately.
+	 */
+	for (i = 0; i < num_indices; i++, ptr++, num_tree_types++) {
+		tree_is_expanded[num_tree_types] = FALSE;
+		**ptr = num_tree_types;
+	}
+}
+
 void
 proto_item_fill_label(field_info *fi, gchar *label_str)
 {
-	char *s;
+	struct header_field_info	*hfinfo = fi->hfinfo;
+	guint32				n_addr; /* network-order IPv4 address */
 
-	switch(fi->hfinfo->type) {
+	switch(hfinfo->type) {
 		case FT_NONE:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s", fi->hfinfo->name);
+				"%s", hfinfo->name);
 			break;
 
 		case FT_BOOLEAN:
-			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s", fi->hfinfo->name,
-				fi->value.numeric == TRUE ? "True" : "False");
+			fill_label_boolean(fi, label_str);
 			break;
 
 		case FT_BYTES:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s", fi->hfinfo->name, 
+				"%s: %s", hfinfo->name, 
 				 bytes_to_str(fi->value.bytes, fi->length));
 			break;
 
+		/* Four types of integers to take care of:
+		 * 	Bitfield, with val_string
+		 * 	Bitfield, w/o val_string
+		 * 	Non-bitfield, with val_string
+		 * 	Non-bitfield, w/o val_string
+		 */
 		case FT_UINT8:
 		case FT_UINT16:
+		case FT_UINT24:
 		case FT_UINT32:
-			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %u", fi->hfinfo->name,
-				fi->value.numeric);
+			if (hfinfo->bitmask) {
+				if (hfinfo->strings) {
+					fill_label_enumerated_bitfield(fi, label_str);
+				}
+				else {
+					fill_label_numeric_bitfield(fi, label_str);
+				}
+			}
+			else {
+				if (hfinfo->strings) {
+					fill_label_enumerated_uint(fi, label_str);
+				}
+				else {
+					fill_label_uint(fi, label_str);
+				}
+			}
+			break;
+
+		case FT_INT8:
+		case FT_INT16:
+		case FT_INT24:
+		case FT_INT32:
+			g_assert(!hfinfo->bitmask);
+			if (hfinfo->strings) {
+				fill_label_enumerated_int(fi, label_str);
+			}
+			else {
+				fill_label_int(fi, label_str);
+			}
 			break;
 
 		case FT_DOUBLE:
@@ -597,38 +608,10 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 				rel_time_to_str(&fi->value.time));
 			break;
 
-		case FT_VALS_UINT8:
-			s = match_strval(fi->value.numeric, cVALS(fi->hfinfo->vals));
-			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s (0x%02x)", fi->hfinfo->name,
-				(s ? s : "Unknown"), fi->value.numeric);
-			break;
-
-		case FT_VALS_UINT16:
-			s = match_strval(fi->value.numeric, cVALS(fi->hfinfo->vals));
-			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s (0x%04x)", fi->hfinfo->name,
-				(s ? s : "Unknown"), fi->value.numeric);
-			break;
-
-		case FT_VALS_UINT24:
-			s = match_strval(fi->value.numeric, cVALS(fi->hfinfo->vals));
-			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s (0x%06x)", fi->hfinfo->name,
-				(s ? s : "Unknown"), fi->value.numeric);
-			break;
-
-
-		case FT_VALS_UINT32:
-			s = match_strval(fi->value.numeric, cVALS(fi->hfinfo->vals));
-			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s (0x%08x)", fi->hfinfo->name,
-				(s ? s : "Unknown"), fi->value.numeric);
-			break;
-
 		case FT_IPXNET:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: 0x%08X", fi->hfinfo->name, fi->value.numeric);
+				"%s: 0x%08X (%s)", fi->hfinfo->name,
+				fi->value.numeric, get_ipxnet_name(fi->value.numeric));
 			break;
 
 		case FT_ETHER:
@@ -639,10 +622,18 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 			break;
 
 		case FT_IPv4:
+			n_addr = ipv4_get_net_order_addr(&fi->value.ipv4);
 			snprintf(label_str, ITEM_LABEL_LENGTH,
 				"%s: %s (%s)", fi->hfinfo->name,
-				get_hostname(fi->value.numeric),
-				ip_to_str((guint8*)&fi->value.numeric));
+				get_hostname(n_addr),
+				ip_to_str((guint8*)&n_addr));
+			break;
+
+		case FT_IPv6:
+			snprintf(label_str, ITEM_LABEL_LENGTH,
+				"%s: %s (%s)", fi->hfinfo->name,
+				get_hostname6((struct e_in6_addr *)fi->value.ipv6),
+				ip6_to_str((struct e_in6_addr*)fi->value.ipv6));
 			break;
 	
 		case FT_STRING:
@@ -656,12 +647,374 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 	}
 }
 
+static void
+fill_label_boolean(field_info *fi, gchar *label_str)
+{
+	char *p = label_str;
+	int bitfield_byte_length = 0, bitwidth;
+	guint32 unshifted_value;
+
+	struct header_field_info	*hfinfo = fi->hfinfo;
+	struct true_false_string	default_tf = { "True", "False" };
+	struct true_false_string	*tfstring = &default_tf;
+
+	if (hfinfo->strings) {
+		tfstring = (struct true_false_string*) hfinfo->strings;
+	}
+
+	if (hfinfo->bitmask) {
+		/* Figure out the bit width */
+		bitwidth = hfinfo_bitwidth(hfinfo);
+
+		/* Un-shift bits */
+		unshifted_value = fi->value.numeric;
+		if (hfinfo->bitshift > 0) {
+			unshifted_value <<= hfinfo->bitshift;
+		}
+
+		/* Create the bitfield first */
+		p = decode_bitfield_value(label_str, unshifted_value, hfinfo->bitmask, bitwidth);
+		bitfield_byte_length = p - label_str;
+	}
+
+	/* Fill in the textual info */
+	snprintf(p, ITEM_LABEL_LENGTH - bitfield_byte_length,
+		"%s: %s",  hfinfo->name,
+		fi->value.numeric ? tfstring->true_string : tfstring->false_string);
+}
+
+
+/* Fills data for bitfield ints with val_strings */
+static void
+fill_label_enumerated_bitfield(field_info *fi, gchar *label_str)
+{
+	char *format = NULL, *p;
+	int bitfield_byte_length, bitwidth;
+	guint32 unshifted_value;
+
+	struct header_field_info	*hfinfo = fi->hfinfo;
+
+	/* Figure out the bit width */
+	bitwidth = hfinfo_bitwidth(hfinfo);
+
+	/* Pick the proper format string */
+	format = hfinfo_uint_vals_format(hfinfo);
+
+	/* Un-shift bits */
+	unshifted_value = fi->value.numeric;
+	if (hfinfo->bitshift > 0) {
+		unshifted_value <<= hfinfo->bitshift;
+	}
+
+	/* Create the bitfield first */
+	p = decode_bitfield_value(label_str, unshifted_value, hfinfo->bitmask, bitwidth);
+	bitfield_byte_length = p - label_str;
+
+	/* Fill in the textual info using stored (shifted) value */
+	snprintf(p, ITEM_LABEL_LENGTH - bitfield_byte_length,
+			format,  hfinfo->name,
+			val_to_str(fi->value.numeric, cVALS(hfinfo->strings), "Unknown"),
+			fi->value.numeric);
+}
+
+static void
+fill_label_numeric_bitfield(field_info *fi, gchar *label_str)
+{
+	char *format = NULL, *p;
+	int bitfield_byte_length, bitwidth;
+	guint32 unshifted_value;
+
+	struct header_field_info	*hfinfo = fi->hfinfo;
+
+	/* Figure out the bit width */
+	bitwidth = hfinfo_bitwidth(hfinfo);
+
+	/* Pick the proper format string */
+	format = hfinfo_uint_format(hfinfo);
+
+	/* Un-shift bits */
+	unshifted_value = fi->value.numeric;
+	if (hfinfo->bitshift > 0) {
+		unshifted_value <<= hfinfo->bitshift;
+	}
+
+	/* Create the bitfield using */
+	p = decode_bitfield_value(label_str, unshifted_value, hfinfo->bitmask, bitwidth);
+	bitfield_byte_length = p - label_str;
+
+	/* Fill in the textual info using stored (shifted) value */
+	snprintf(p, ITEM_LABEL_LENGTH - bitfield_byte_length,
+			format,  hfinfo->name, fi->value.numeric);
+}
+
+static void
+fill_label_enumerated_uint(field_info *fi, gchar *label_str)
+{
+	char *format = NULL;
+	struct header_field_info	*hfinfo = fi->hfinfo;
+
+	/* Pick the proper format string */
+	format = hfinfo_uint_vals_format(hfinfo);
+
+	/* Fill in the textual info */
+	snprintf(label_str, ITEM_LABEL_LENGTH,
+			format,  hfinfo->name,
+			val_to_str(fi->value.numeric, cVALS(hfinfo->strings), "Unknown"),
+			fi->value.numeric);
+}
+
+static void
+fill_label_uint(field_info *fi, gchar *label_str)
+{
+	char *format = NULL;
+	struct header_field_info	*hfinfo = fi->hfinfo;
+
+	/* Pick the proper format string */
+	format = hfinfo_uint_format(hfinfo);
+
+	/* Fill in the textual info */
+	snprintf(label_str, ITEM_LABEL_LENGTH,
+			format,  hfinfo->name, fi->value.numeric);
+}
+
+static void
+fill_label_enumerated_int(field_info *fi, gchar *label_str)
+{
+	char *format = NULL;
+	struct header_field_info	*hfinfo = fi->hfinfo;
+
+	/* Pick the proper format string */
+	format = hfinfo_int_vals_format(hfinfo);
+
+	/* Fill in the textual info */
+	snprintf(label_str, ITEM_LABEL_LENGTH,
+			format,  hfinfo->name,
+			val_to_str(fi->value.numeric, cVALS(hfinfo->strings), "Unknown"),
+			fi->value.numeric);
+}
+
+static void
+fill_label_int(field_info *fi, gchar *label_str)
+{
+	char *format = NULL;
+	struct header_field_info	*hfinfo = fi->hfinfo;
+
+	/* Pick the proper format string */
+	format = hfinfo_int_format(hfinfo);
+
+	/* Fill in the textual info */
+	snprintf(label_str, ITEM_LABEL_LENGTH,
+			format,  hfinfo->name, fi->value.numeric);
+}
+
+static int
+hfinfo_bitwidth(header_field_info *hfinfo)
+{
+	int bitwidth = 0;
+
+	if (!hfinfo->bitmask) {
+		return 0;
+	}
+
+	switch(hfinfo->type) {
+		case FT_UINT8:
+		case FT_INT8:
+			bitwidth = 8;
+			break;
+		case FT_UINT16:
+		case FT_INT16:
+			bitwidth = 16;
+			break;
+		case FT_UINT24:
+		case FT_INT24:
+			bitwidth = 24;
+			break;
+		case FT_UINT32:
+		case FT_INT32:
+			bitwidth = 32;
+			break;
+		case FT_BOOLEAN:
+			bitwidth = hfinfo->display; /* hacky? :) */
+			break;
+		default:
+			g_assert_not_reached();
+			;
+	}
+	return bitwidth;
+}
+
+static char*
+hfinfo_uint_vals_format(header_field_info *hfinfo)
+{
+	char *format = NULL;
+
+	switch(hfinfo->display) {
+		case BASE_DEC:
+		case BASE_NONE:
+		case BASE_OCT: /* I'm lazy */
+		case BASE_BIN: /* I'm lazy */
+			format = "%s: %s (%u)";
+			break;
+		case BASE_HEX:
+			switch(hfinfo->type) {
+				case FT_UINT8:
+					format = "%s: %s (0x%02x)";
+					break;
+				case FT_UINT16:
+					format = "%s: %s (0x%04x)";
+					break;
+				case FT_UINT24:
+					format = "%s: %s (0x%06x)";
+					break;
+				case FT_UINT32:
+					format = "%s: %s (0x%08x)";
+					break;
+				default:
+					g_assert_not_reached();
+					;
+			}
+			break;
+		default:
+			g_assert_not_reached();
+			;
+	}
+	return format;
+}
+
+static char*
+hfinfo_uint_format(header_field_info *hfinfo)
+{
+	char *format = NULL;
+
+	/* Pick the proper format string */
+	switch(hfinfo->display) {
+		case BASE_DEC:
+		case BASE_NONE:
+		case BASE_OCT: /* I'm lazy */
+		case BASE_BIN: /* I'm lazy */
+			format = "%s: %u";
+			break;
+		case BASE_HEX:
+			switch(hfinfo->type) {
+				case FT_UINT8:
+					format = "%s: 0x%02x";
+					break;
+				case FT_UINT16:
+					format = "%s: 0x%04x";
+					break;
+				case FT_UINT24:
+					format = "%s: 0x%06x";
+					break;
+				case FT_UINT32:
+					format = "%s: 0x%08x";
+					break;
+				default:
+					g_assert_not_reached();
+					;
+			}
+			break;
+		default:
+			g_assert_not_reached();
+			;
+	}
+	return format;
+}
+
+static char*
+hfinfo_int_vals_format(header_field_info *hfinfo)
+{
+	char *format = NULL;
+
+	switch(hfinfo->display) {
+		case BASE_DEC:
+		case BASE_NONE:
+		case BASE_OCT: /* I'm lazy */
+		case BASE_BIN: /* I'm lazy */
+			format = "%s: %s (%d)";
+			break;
+		case BASE_HEX:
+			switch(hfinfo->type) {
+				case FT_INT8:
+					format = "%s: %s (0x%02x)";
+					break;
+				case FT_INT16:
+					format = "%s: %s (0x%04x)";
+					break;
+				case FT_INT24:
+					format = "%s: %s (0x%06x)";
+					break;
+				case FT_INT32:
+					format = "%s: %s (0x%08x)";
+					break;
+				default:
+					g_assert_not_reached();
+					;
+			}
+			break;
+		default:
+			g_assert_not_reached();
+			;
+	}
+	return format;
+}
+
+static char*
+hfinfo_int_format(header_field_info *hfinfo)
+{
+	char *format = NULL;
+
+	/* Pick the proper format string */
+	switch(hfinfo->display) {
+		case BASE_DEC:
+		case BASE_NONE:
+		case BASE_OCT: /* I'm lazy */
+		case BASE_BIN: /* I'm lazy */
+			format = "%s: %d";
+			break;
+		case BASE_HEX:
+			switch(hfinfo->type) {
+				case FT_INT8:
+					format = "%s: 0x%02x";
+					break;
+				case FT_INT16:
+					format = "%s: 0x%04x";
+					break;
+				case FT_INT24:
+					format = "%s: 0x%06x";
+					break;
+				case FT_INT32:
+					format = "%s: 0x%08x";
+					break;
+				default:
+					g_assert_not_reached();
+					;
+			}
+			break;
+		default:
+			g_assert_not_reached();
+			;
+	}
+	return format;
+}
+
+
+
 int
 proto_registrar_n(void)
 {
 	return gpa_hfinfo->len;
 }
 
+char*
+proto_registrar_get_name(int n)
+{
+    struct header_field_info *hfinfo;
+    hfinfo = find_hfinfo_record(n);
+    if (hfinfo)
+        return hfinfo->name;
+    else        return NULL;
+}
+
 char*
 proto_registrar_get_abbrev(int n)
 {
@@ -710,6 +1063,61 @@ proto_registrar_is_protocol(int n)
 		return FALSE;
 }
 
+/* Returns length of field in packet (not necessarily the length
+ * in our internal representation, as in the case of IPv4).
+ * 0 means undeterminable at time of registration
+ * -1 means the field is not registered. */
+gint
+proto_registrar_get_length(int n)
+{
+	struct header_field_info *hfinfo;
+
+	hfinfo = find_hfinfo_record(n);
+	if (!hfinfo)
+		return -1;
+
+	switch (hfinfo->type) {
+		case FT_TEXT_ONLY: /* not filterable */
+		case NUM_FIELD_TYPES: /* satisfy picky compilers */
+			return -1;
+
+		case FT_NONE:
+		case FT_BYTES:
+		case FT_BOOLEAN:
+		case FT_STRING:
+		case FT_DOUBLE:
+		case FT_ABSOLUTE_TIME:
+		case FT_RELATIVE_TIME:
+			return 0;
+
+		case FT_UINT8:
+		case FT_INT8:
+			return 1;
+
+		case FT_UINT16:
+		case FT_INT16:
+			return 2;
+
+		case FT_UINT24:
+		case FT_INT24:
+			return 3;
+
+		case FT_UINT32:
+		case FT_INT32:
+		case FT_IPXNET:
+		case FT_IPv4:
+			return 4;
+
+		case FT_ETHER:
+			return 6;
+
+		case FT_IPv6:
+			return 16;
+	}
+	g_assert_not_reached();
+	return -1;
+}
+
 /* Looks for a protocol or a field in a proto_tree. Returns TRUE if
  * it exists anywhere, or FALSE if it exists nowhere. */
 gboolean
@@ -855,9 +1263,24 @@ proto_registrar_dump(void)
 			case FT_UINT16:
 				enum_name = "FT_UINT16";
 				break;
+			case FT_UINT24:
+				enum_name = "FT_UINT24";
+				break;
 			case FT_UINT32:
 				enum_name = "FT_UINT32";
 				break;
+			case FT_INT8:
+				enum_name = "FT_INT8";
+				break;
+			case FT_INT16:
+				enum_name = "FT_INT16";
+				break;
+			case FT_INT24:
+				enum_name = "FT_INT24";
+				break;
+			case FT_INT32:
+				enum_name = "FT_INT32";
+				break;
 			case FT_DOUBLE:
 				enum_name = "FT_DOUBLE";
 				break;
@@ -885,18 +1308,6 @@ proto_registrar_dump(void)
 			case FT_IPXNET:
 				enum_name = "FT_IPXNET";
 				break;
-			case FT_VALS_UINT8:
-				enum_name = "FT_VALS_UINT8";
-				break;
-			case FT_VALS_UINT16:
-				enum_name = "FT_VALS_UINT16";
-				break;
-			case FT_VALS_UINT24:
-				enum_name = "FT_VALS_UINT24";
-				break;
-			case FT_VALS_UINT32:
-				enum_name = "FT_VALS_UINT32";
-				break;
 			case FT_TEXT_ONLY:
 				enum_name = "FT_TEXT_ONLY";
 				break;
