@@ -1,7 +1,6 @@
 
 /*  A Bison parser, made from ascend-grammar.y
- by  GNU Bison version 1.27
-  */
+    by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
 
@@ -20,11 +19,61 @@
 #define	HEXNUM	262
 #define	BYTE	263
 
-#line 51 "ascend-grammar.y"
+#line 1 "ascend-grammar.y"
 
+/* ascend-grammar.y
+ *
+ * $Id: ascend-grammar.y,v 1.8 1999/10/31 19:34:46 guy Exp $
+ *
+ * Wiretap Library
+ * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ */
+
+/*
+   Example 'wandsess' output data:
+   
+RECV-iguana:241:(task: B02614C0, time: 1975432.85) 49 octets @ 8003BD94
+  [0000]: FF 03 00 3D C0 06 CA 22 2F 45 00 00 28 6A 3B 40 
+  [0010]: 00 3F 03 D7 37 CE 41 62 12 CF 00 FB 08 20 27 00 
+  [0020]: 50 E4 08 DD D7 7C 4C 71 92 50 10 7D 78 67 C8 00 
+  [0030]: 00 
+XMIT-iguana:241:(task: B04E12C0, time: 1975432.85) 53 octets @ 8009EB16
+  [0000]: FF 03 00 3D C0 09 1E 31 21 45 00 00 2C 2D BD 40 
+  [0010]: 00 7A 06 D8 B1 CF 00 FB 08 CE 41 62 12 00 50 20 
+  [0020]: 29 7C 4C 71 9C 9A 6A 93 A4 60 12 22 38 3F 10 00 
+  [0030]: 00 02 04 05 B4 
+
+    Example 'wdd' output data:
+
+Date: 01/12/1990.  Time: 12:22:33
+Cause an attempt to place call to 14082750382
+WD_DIALOUT_DISP: chunk 2515EE type IP.
+(task: 251790, time: 994953.28) 44 octets @ 2782B8
+  [0000]: 00 C0 7B 71 45 6C 00 60 08 16 AA 51 08 00 45 00
+  [0010]: 00 2C 66 1C 40 00 80 06 53 F6 AC 14 00 18 CC 47
+  [0020]: C8 45 0A 31 00 50 3B D9 5B 75 00 00
  
+
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <time.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -32,6 +81,7 @@
 #include "wtap.h"
 #include "buffer.h"
 #include "ascend.h"
+#include "ascend-int.h"
 
 #define NFH_PATH "/dev/null"
 
@@ -50,7 +100,7 @@ FILE *nfh = NULL;
 struct tm wddt;
 
 
-#line 81 "ascend-grammar.y"
+#line 82 "ascend-grammar.y"
 typedef union {
 gchar  *s;
 guint32 d;
@@ -142,10 +192,10 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-    97,    98,    99,   102,   104,   106,   108,   111,   138,   166,
-   185,   186,   187,   188,   189,   190,   191,   192,   193,   194,
-   195,   196,   197,   198,   199,   200,   203,   205,   206,   207,
-   208,   209,   210,   211,   212
+    98,    99,   100,   103,   105,   107,   109,   112,   139,   167,
+   186,   187,   188,   189,   190,   191,   192,   193,   194,   195,
+   196,   197,   198,   199,   200,   201,   204,   206,   207,   208,
+   209,   210,   211,   212,   213
 };
 #endif
 
@@ -222,8 +272,8 @@ static const short yycheck[] = {     7,
     62,    59,     8,    61,     0,    63,    41,     5
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
-#line 3 "/usr/lib/bison.simple"
-/* This file comes from bison-1.27.  */
+#line 3 "/usr/share/misc/bison.simple"
+/* This file comes from bison-1.28.  */
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -436,7 +486,7 @@ __yy_memcpy (char *to, char *from, unsigned int count)
 #endif
 #endif
 
-#line 216 "/usr/lib/bison.simple"
+#line 217 "/usr/share/misc/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -765,7 +815,7 @@ yyreduce:
   switch (yyn) {
 
 case 8:
-#line 111 "ascend-grammar.y"
+#line 112 "ascend-grammar.y"
 {
   wirelen = yyvsp[-2].d;
   caplen = (yyvsp[-2].d < ASCEND_MAX_PKT_LEN) ? yyvsp[-2].d : ASCEND_MAX_PKT_LEN;
@@ -787,7 +837,7 @@ case 8:
 ;
     break;}
 case 9:
-#line 138 "ascend-grammar.y"
+#line 139 "ascend-grammar.y"
 {
   wddt.tm_sec  = yyvsp[-17].d;
   wddt.tm_min  = yyvsp[-18].d;
@@ -816,7 +866,7 @@ case 9:
 ;
     break;}
 case 10:
-#line 166 "ascend-grammar.y"
+#line 167 "ascend-grammar.y"
 {
   if (bcur < caplen) {
     pkt_data[bcur] = yyvsp[0].b;
@@ -836,7 +886,7 @@ case 10:
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 542 "/usr/lib/bison.simple"
+#line 543 "/usr/share/misc/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1056,7 +1106,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 215 "ascend-grammar.y"
+#line 216 "ascend-grammar.y"
 
 
 void
