@@ -1,6 +1,6 @@
 /* file.c
  *
- * $Id: file.c,v 1.24 1999/09/24 05:49:50 guy Exp $
+ * $Id: file.c,v 1.29 1999/11/10 19:47:56 gram Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -36,11 +36,13 @@
 #include "ngsniffer.h"
 #include "radcom.h"
 #include "ascend.h"
+#include "nettl.h"
 #include "libpcap.h"
 #include "snoop.h"
 #include "iptrace.h"
 #include "netmon.h"
 #include "netxray.h"
+#include "toshiba.h"
 
 /* The open_file_* routines should return:
  *
@@ -57,9 +59,17 @@
  * declare a "const" array of pointers to functions; putting "const"
  * right after "static" isn't the right answer, at least according
  * to GCC, which whines if I do that.
+ *
+ * Put the trace files that are merely saved telnet-sessions last, since it's
+ * possible that you could have captured someone a router telnet-session
+ * using another tool. So, a libpcap trace of an toshiba "snoop" session
+ * should be discovered as a libpcap file, not a toshiba file.
  */
 
 static int (*open_routines[])(wtap *, int *) = {
+	/* Files that have magic bytes in fixed locations. These
+	 * are easy to identify.
+	 */
 	libpcap_open,
 	lanalyzer_open,
 	ngsniffer_open,
@@ -68,7 +78,14 @@ static int (*open_routines[])(wtap *, int *) = {
 	netmon_open,
 	netxray_open,
 	radcom_open,
-	ascend_open
+	nettl_open,
+
+	/* Files whose magic headers are in text *somewhere* in the
+	 * file (usually because the trace is just a saved copy of
+	 * the telnet session). 
+	 */
+	ascend_open,
+	toshiba_open,
 };
 
 int wtap_def_seek_read (FILE *fh, int seek_off, guint8 *pd, int len)
@@ -99,7 +116,7 @@ wtap* wtap_open_offline(const char *filename, int *err)
 #endif
 
 	errno = ENOMEM;
-	wth = (wtap*)malloc(sizeof(wtap));
+	wth = g_malloc(sizeof(wtap));
 	if (wth == NULL) {
 		*err = errno;
 		return NULL;
@@ -109,12 +126,12 @@ wtap* wtap_open_offline(const char *filename, int *err)
 	errno = WTAP_ERR_CANT_OPEN;
 	if (!(wth->fd = open(filename, O_RDONLY))) {
 		*err = errno;
-		free(wth);
+		g_free(wth);
 		return NULL;
 	}
 	if (!(wth->fh = filed_open(wth->fd, "rb"))) {
 		*err = errno;
-		free(wth);
+		g_free(wth);
 		return NULL;
 	}
 
@@ -128,9 +145,8 @@ wtap* wtap_open_offline(const char *filename, int *err)
 
 		case -1:
 			/* I/O error - give up */
-			*err = errno;
 			file_close(wth->fh);
-			free(wth);
+			g_free(wth);
 			return NULL;
 
 		case 0:
@@ -145,7 +161,7 @@ wtap* wtap_open_offline(const char *filename, int *err)
 
 	/* Well, it's not one of the types of file we know about. */
 	file_close(wth->fh);
-	free(wth);
+	g_free(wth);
 	*err = WTAP_ERR_FILE_UNKNOWN_FORMAT;
 	return NULL;
 
@@ -196,7 +212,7 @@ static wtap_dumper* wtap_dump_open_common(FILE *fh, int filetype, int encap,
 {
 	wtap_dumper *wdh;
 
-	wdh = malloc(sizeof (wtap_dumper));
+	wdh = g_malloc(sizeof (wtap_dumper));
 	if (wdh == NULL) {
 		*err = errno;
 		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
@@ -224,7 +240,7 @@ static wtap_dumper* wtap_dump_open_common(FILE *fh, int filetype, int encap,
 	return wdh;
 
 fail:
-	free(wdh);
+	g_free(wdh);
 	fclose(fh);
 	return NULL;	/* XXX - provide a reason why we failed */
 }
@@ -257,7 +273,44 @@ int wtap_dump_close(wtap_dumper *wdh, int *err)
 		}
 		ret = 0;
 	}
-	free(wdh);
+	g_free(wdh);
 	return ret;
 }
 
+/*
+ * Routine to return a Wiretap error code (0 for no error, an errno
+ * for a file error, or a WTAP_ERR_ code for other errors) for an
+ * I/O stream.
+ */
+#ifdef HAVE_LIBZ
+int
+file_error(void *fh)
+{
+	int errnum;
+
+	gzerror(fh, &errnum);
+	switch (errnum) {
+
+	case Z_OK:		/* no error */
+		return 0;
+
+	case Z_STREAM_END:	/* EOF - not an error */
+		return 0;
+
+	case Z_ERRNO:		/* file I/O error */
+		return errno;
+
+	default:
+		return WTAP_ERR_ZLIB + errnum;
+	}
+}
+#else /* HAVE_LIBZ */
+int
+file_error(FILE *fh)
+{
+	if (ferror(fh))
+		return errno;
+	else
+		return 0;
+}
+#endif /* HAVE_LIBZ */
