@@ -1,6 +1,6 @@
 /* ngsniffer.c
  *
- * $Id: ngsniffer.c,v 1.23 1999/09/24 05:49:52 guy Exp $
+ * $Id: ngsniffer.c,v 1.27 1999/11/28 02:08:48 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -237,7 +237,7 @@ struct frame4_rec {
 
 /* values for V.timeunit */
 #define NUM_NGSNIFF_TIMEUNITS 7
-static double Usec[] = { 15.0, 0.838096, 15.0, 0.5, 2.0, 0.0, 0.1 };
+static double Usec[] = { 15.0, 0.838096, 15.0, 0.5, 2.0, 1.0, 0.1 };
 
 static int ngsniffer_read(wtap *wth, int *err);
 
@@ -260,7 +260,7 @@ int ngsniffer_open(wtap *wth, int *err)
 		WTAP_ENCAP_UNKNOWN,	/* PC Network broadband */
 		WTAP_ENCAP_UNKNOWN,	/* LocalTalk */
 		WTAP_ENCAP_UNKNOWN,	/* Znet */
-		WTAP_ENCAP_LAPB,	/* Internetwork analyzer */
+		WTAP_ENCAP_UNKNOWN,	/* Internetwork analyzer */
 		WTAP_ENCAP_UNKNOWN,	/* type 8 not defined in Sniffer */
 		WTAP_ENCAP_FDDI_BITSWAPPED,
 		WTAP_ENCAP_ATM_SNIFFER	/* ATM */
@@ -274,10 +274,9 @@ int ngsniffer_open(wtap *wth, int *err)
 	errno = WTAP_ERR_CANT_READ;
 	bytes_read = file_read(magic, 1, 17, wth->fh);
 	if (bytes_read != 17) {
-		if (file_error(wth->fh)) {
-			*err = errno;
+		*err = file_error(wth->fh);
+		if (*err != 0)
 			return -1;
-		}
 		return 0;
 	}
 	wth->data_offset += 17;
@@ -296,10 +295,9 @@ int ngsniffer_open(wtap *wth, int *err)
 	bytes_read = file_read(record_type, 1, 2, wth->fh);
 	bytes_read += file_read(record_length, 1, 4, wth->fh);
 	if (bytes_read != 6) {
-		if (file_error(wth->fh)) {
-			*err = errno;
+		*err = file_error(wth->fh);
+		if (*err != 0)
 			return -1;
-		}
 		return 0;
 	}
 	wth->data_offset += 6;
@@ -316,10 +314,9 @@ int ngsniffer_open(wtap *wth, int *err)
 	errno = WTAP_ERR_CANT_READ;
 	bytes_read = file_read(&version, 1, sizeof version, wth->fh);
 	if (bytes_read != sizeof version) {
-		if (file_error(wth->fh)) {
-			*err = errno;
+		*err = file_error(wth->fh);
+		if (*err != 0)
 			return -1;
-		}
 		return 0;
 	}
 	wth->data_offset += sizeof version;
@@ -331,9 +328,18 @@ int ngsniffer_open(wtap *wth, int *err)
 		return -1;
 	}
 
-	/* Check the data link type */
+	/* Check the data link type.
+	   If "version.network" is 7, that's "Internetwork analyzer";
+	   Sniffers appear to write out both LAPB and PPP captures
+	   (and perhaps other types of captures) in that fashion,
+	   and, so far, the only way we know of distinguishing them
+	   is to look at the first byte of the packet - if it's 0xFF,
+	   it's PPP, otherwise it's LAPB.
+	   Therefore, we treat it as WTAP_ENCAP_UNKNOWN for now, but
+	   don't treat that as an error. */
 	if (version.network >= NUM_NGSNIFF_ENCAPS
-	    || sniffer_encap[version.network] == WTAP_ENCAP_UNKNOWN) {
+	    || (sniffer_encap[version.network] == WTAP_ENCAP_UNKNOWN
+	       && version.network != 7)) {
 		g_message("ngsniffer: network type %u unknown or unsupported",
 		    version.network);
 		*err = WTAP_ERR_UNSUPPORTED;
@@ -403,6 +409,7 @@ static int ngsniffer_read(wtap *wth, int *err)
 	double	t;
 	guint16	time_low, time_med, time_high, true_size, size;
 	int	data_offset;
+	u_char	*pd;
 
 	for (;;) {
 		/*
@@ -411,10 +418,9 @@ static int ngsniffer_read(wtap *wth, int *err)
 		errno = WTAP_ERR_CANT_READ;
 		bytes_read = file_read(record_type, 1, 2, wth->fh);
 		if (bytes_read != 2) {
-			if (file_error(wth->fh)) {
-				*err = errno;
+			*err = file_error(wth->fh);
+			if (*err != 0)
 				return -1;
-			}
 			if (bytes_read != 0) {
 				*err = WTAP_ERR_SHORT_READ;
 				return -1;
@@ -425,9 +431,8 @@ static int ngsniffer_read(wtap *wth, int *err)
 		errno = WTAP_ERR_CANT_READ;
 		bytes_read = file_read(record_length, 1, 4, wth->fh);
 		if (bytes_read != 4) {
-			if (file_error(wth->fh))
-				*err = errno;
-			else
+			*err = file_error(wth->fh);
+			if (*err == 0)
 				*err = WTAP_ERR_SHORT_READ;
 			return -1;
 		}
@@ -453,9 +458,8 @@ static int ngsniffer_read(wtap *wth, int *err)
 			errno = WTAP_ERR_CANT_READ;
 			bytes_read = file_read(&frame2, 1, sizeof frame2, wth->fh);
 			if (bytes_read != sizeof frame2) {
-				if (file_error(wth->fh))
-					*err = errno;
-				else
+				*err = file_error(wth->fh);
+				if (*err == 0)
 					*err = WTAP_ERR_SHORT_READ;
 				return -1;
 			}
@@ -471,6 +475,21 @@ static int ngsniffer_read(wtap *wth, int *err)
 			t = (double)time_low+(double)(time_med)*65536.0 +
 			    (double)time_high*4294967296.0;
 
+			/*
+			 * In one PPP "Internetwork analyzer" capture,
+			 * the only bit seen in "fs" is the 0x80 bit,
+			 * which probably indicates the packet's
+			 * direction; all other bits were zero.
+			 * All bits in "frame2.flags" were zero.
+			 *
+			 * In one Ethernet capture, "fs" was always 0,
+			 * and "flags" was either 0 or 0x18, with no
+			 * obvious correlation with anything.
+			 *
+			 * In one Token Ring capture, "fs" was either 0
+			 * or 0xcc, and "flags" was either 0 or 0x18,
+			 * with no obvious correlation with anything.
+			 */
 			wth->phdr.pseudo_header.x25.flags = frame2.fs & 0x80;
 
 			goto found;
@@ -490,9 +509,8 @@ static int ngsniffer_read(wtap *wth, int *err)
 			errno = WTAP_ERR_CANT_READ;
 			bytes_read = file_read(&frame4, 1, sizeof frame4, wth->fh);
 			if (bytes_read != sizeof frame4) {
-				if (file_error(wth->fh))
-					*err = errno;
-				else
+				*err = file_error(wth->fh);
+				if (*err == 0)
 					*err = WTAP_ERR_SHORT_READ;
 				return -1;
 			}
@@ -561,18 +579,36 @@ found:
 	buffer_assure_space(wth->frame_buffer, length);
 	data_offset = wth->data_offset;
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(buffer_start_ptr(wth->frame_buffer), 1,
-			length, wth->fh);
+	pd = buffer_start_ptr(wth->frame_buffer);
+	bytes_read = file_read(pd, 1, length, wth->fh);
 
 	if (bytes_read != length) {
-		if (file_error(wth->fh))
-			*err = errno;
-		else
+		*err = file_error(wth->fh);
+		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
 		return -1;
 	}
 	wth->data_offset += length;
 
+	if (wth->file_encap == WTAP_ENCAP_UNKNOWN) {
+		/*
+		 * OK, this is from an "Internetwork analyzer"; let's
+		 * look at the first byte of the packet, and figure
+		 * out whether it's LAPB or PPP.
+		 */
+		if (pd[0] == 0xFF) {
+			/*
+			 * PPP.
+			 */
+			wth->file_encap = WTAP_ENCAP_PPP;
+		} else {
+			/*
+			 * LAPB.
+			 */
+			wth->file_encap = WTAP_ENCAP_LAPB;
+		}
+	}
+
 	t = t/1000000.0 * wth->capture.ngsniffer->timeunit; /* t = # of secs */
 	t += wth->capture.ngsniffer->start;
 	wth->phdr.ts.tv_sec = (long)t;
