@@ -1,6 +1,6 @@
 /* wtap.c
  *
- * $Id: wtap.c,v 1.22 1999/09/24 05:49:53 guy Exp $
+ * $Id: wtap.c,v 1.31 1999/11/26 17:57:14 gram Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -30,6 +30,7 @@
 #include "file.h"
 #include "buffer.h"
 #include "ascend.h"
+#include "toshiba.h"
 
 FILE* wtap_file(wtap *wth)
 {
@@ -51,6 +52,11 @@ int wtap_snapshot_length(wtap *wth)
 	return wth->snapshot_length;
 }
 
+int wtap_file_encap(wtap *wth)
+{
+	return wth->file_encap;
+}
+
 const char *wtap_file_type_string(wtap *wth)
 {
 	switch (wth->file_type) {
@@ -60,6 +66,9 @@ const char *wtap_file_type_string(wtap *wth)
 		case WTAP_FILE_PCAP:
 			return "pcap";
 
+		case WTAP_FILE_PCAP_MODIFIED:
+			return "pcap-modified";
+
 		case WTAP_FILE_LANALYZER:
 			return "Novell LANalyzer";
 
@@ -69,8 +78,11 @@ const char *wtap_file_type_string(wtap *wth)
 		case WTAP_FILE_SNOOP:
 			return "snoop";
 
-		case WTAP_FILE_IPTRACE:
-			return "iptrace";
+		case WTAP_FILE_IPTRACE_1_0:
+			return "AIX iptrace 1.0";
+
+		case WTAP_FILE_IPTRACE_2_0:
+			return "AIX iptrace 2.0";
 
 		case WTAP_FILE_NETMON_1_x:
 			return "Microsoft Network Monitor 1.x";
@@ -93,6 +105,12 @@ const char *wtap_file_type_string(wtap *wth)
 		case WTAP_FILE_ASCEND:
 			return "Lucent/Ascend access server trace";
 
+		case WTAP_FILE_NETTL:
+			return "HP-UX nettl trace";
+
+		case WTAP_FILE_TOSHIBA:
+			return "Toshiba Compact ISDN Router snoop trace";
+
 		default:
 			g_error("Unknown capture file type %d", wth->file_type);
 			return NULL;
@@ -121,6 +139,14 @@ const char *wtap_strerror(int err)
 	int wtap_errlist_index;
 
 	if (err < 0) {
+#ifdef HAVE_LIBZ
+		if (err >= WTAP_ERR_ZLIB_MIN && err <= WTAP_ERR_ZLIB_MAX) {
+			/* Assume it's a zlib error. */
+			sprintf(errbuf, "Uncompression error: %s",
+			    zError(err - WTAP_ERR_ZLIB));
+			return errbuf;
+		}
+#endif
 		wtap_errlist_index = -1 - err;
 		if (wtap_errlist_index >= WTAP_ERRLIST_SIZE) {
 			sprintf(errbuf, "Error %d", err);
@@ -141,6 +167,7 @@ void wtap_close(wtap *wth)
 	 * But for now this will work. */
 	switch(wth->file_type) {
 		case WTAP_FILE_PCAP:
+		case WTAP_FILE_PCAP_MODIFIED:
 			g_free(wth->capture.pcap);
 			break;
 
@@ -171,11 +198,22 @@ void wtap_close(wtap *wth)
 			g_free(wth->capture.ascend);
 			break;
 
+		case WTAP_FILE_NETTL:
+			g_free(wth->capture.nettl);
+			break;
+
 		/* default:
 			 nothing */
 	}
 
 	file_close(wth->fh);
+
+	if (wth->frame_buffer) {
+		buffer_free(wth->frame_buffer);
+		g_free(wth->frame_buffer);
+	}
+
+	g_free(wth);
 }
 
 int wtap_loop(wtap *wth, int count, wtap_handler callback, u_char* user,
@@ -195,13 +233,16 @@ int wtap_loop(wtap *wth, int count, wtap_handler callback, u_char* user,
 		return TRUE;	/* success */
 }
 
-int wtap_seek_read(int encaps, FILE *fh, int seek_off, guint8 *pd, int len)
+int wtap_seek_read(int file_type, FILE *fh, int seek_off, guint8 *pd, int len)
 {
-	switch (encaps) {
+	switch (file_type) {
 
-	case WTAP_ENCAP_ASCEND:
+	case WTAP_FILE_ASCEND:
 		return ascend_seek_read(fh, seek_off, pd, len);
 
+	case WTAP_FILE_TOSHIBA:
+		return toshiba_seek_read(fh, seek_off, pd, len);
+
 	default:
 		return wtap_def_seek_read(fh, seek_off, pd, len);
 	}
