@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h,v 1.41 1999/09/24 05:49:53 guy Exp $
+ * $Id: wtap.h,v 1.50 1999/11/26 17:57:14 gram Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -31,6 +31,8 @@
  * "wtap_dump_fd_open()" to indicate that there is no single encapsulation
  * type for all packets in the file; this may cause those routines to
  * fail if the capture file format being written can't support that.
+ * It's also returned by "wtap_file_encap()" for capture files that
+ * don't have a single encapsulation type for all packets in the file.
  *
  * WTAP_ENCAP_UNKNOWN is returned by "wtap_pcap_encap_to_wtap_encap()"
  * if it's handed an unknown encapsulation.
@@ -88,10 +90,11 @@
 #define WTAP_ENCAP_LAPB				11
 #define WTAP_ENCAP_ATM_SNIFFER			12
 #define WTAP_ENCAP_NULL				13
-#define WTAP_ENCAP_ASCEND				14
+#define WTAP_ENCAP_ASCEND			14
+#define WTAP_ENCAP_LAPD				15
 
 /* last WTAP_ENCAP_ value + 1 */
-#define WTAP_NUM_ENCAP_TYPES			15
+#define WTAP_NUM_ENCAP_TYPES			16
 
 /* File types that can be read by wiretap.
    We may eventually support writing some or all of these file types,
@@ -99,17 +102,21 @@
 #define WTAP_FILE_UNKNOWN			0
 #define WTAP_FILE_WTAP				1
 #define WTAP_FILE_PCAP				2
-#define WTAP_FILE_LANALYZER			3
-#define WTAP_FILE_NGSNIFFER			4
+#define WTAP_FILE_PCAP_MODIFIED			3
+#define WTAP_FILE_LANALYZER			4
+#define WTAP_FILE_NGSNIFFER			5
 #define WTAP_FILE_SNOOP				6
-#define WTAP_FILE_IPTRACE			7
-#define WTAP_FILE_NETMON_1_x			8
-#define WTAP_FILE_NETMON_2_x			9
-#define WTAP_FILE_NETXRAY_1_0			10
-#define WTAP_FILE_NETXRAY_1_1			11
-#define WTAP_FILE_NETXRAY_2_001			12
-#define WTAP_FILE_RADCOM			13
-#define WTAP_FILE_ASCEND			14
+#define WTAP_FILE_IPTRACE_1_0			7
+#define WTAP_FILE_IPTRACE_2_0			8
+#define WTAP_FILE_NETMON_1_x			9
+#define WTAP_FILE_NETMON_2_x			10
+#define WTAP_FILE_NETXRAY_1_0			11
+#define WTAP_FILE_NETXRAY_1_1			12
+#define WTAP_FILE_NETXRAY_2_001			13
+#define WTAP_FILE_RADCOM			14
+#define WTAP_FILE_ASCEND			15
+#define WTAP_FILE_NETTL				16
+#define WTAP_FILE_TOSHIBA			17
 
 /*
  * Maximum packet size we'll support.
@@ -146,12 +153,17 @@ typedef struct {
 	time_t	start;
 } radcom_t;
 
+typedef struct {
+	time_t	start;
+} nettl_t;
+
 typedef struct {
 	time_t	start;
 } lanalyzer_t;
 
 typedef struct {
-	int	byte_swapped;
+	gboolean byte_swapped;
+	gboolean modified;
 	guint16	version_major;
 	guint16	version_minor;
 } libpcap_t;
@@ -214,6 +226,11 @@ struct ascend_phdr {
 	guint32	task;			/* Task number */
 };
 
+/* Packet "pseudo-header" information for LAPD capture files. */
+struct lapd_phdr {
+	gboolean	from_network_to_user;
+};
+
 /*
  * Bits in AppTrafType.
  *
@@ -267,9 +284,10 @@ struct ascend_phdr {
 #define	AHLT_VCMX_BPDU		0xe	/* VCMX: BPDU */
 
 union pseudo_header {
-	struct x25_phdr x25;
-	struct ngsniffer_atm_phdr ngsniffer_atm;
-	struct ascend_phdr ascend;
+	struct x25_phdr			x25;
+	struct ngsniffer_atm_phdr	ngsniffer_atm;
+	struct ascend_phdr		ascend;
+	struct lapd_phdr		lapd;
 };
 
 struct wtap_pkthdr {
@@ -284,7 +302,6 @@ typedef void (*wtap_handler)(u_char*, const struct wtap_pkthdr*,
 		int, const u_char *);
 
 struct wtap;
-struct bpf_instruction;
 struct Buffer;
 
 typedef int (*subtype_read_func)(struct wtap*, int*);
@@ -303,6 +320,7 @@ typedef struct wtap {
 		lanalyzer_t		*lanalyzer;
 		ngsniffer_t		*ngsniffer;
 		radcom_t		*radcom;
+		nettl_t			*nettl;
 		netmon_t		*netmon;
 		netxray_t		*netxray;
 		ascend_t		*ascend;
@@ -345,10 +363,11 @@ FILE* wtap_file(wtap *wth);
 int wtap_fd(wtap *wth);
 int wtap_snapshot_length(wtap *wth); /* per file */
 int wtap_file_type(wtap *wth);
+int wtap_file_encap(wtap *wth);
 const char *wtap_file_type_string(wtap *wth);
 const char *wtap_strerror(int err);
 void wtap_close(wtap *wth);
-int wtap_seek_read (int encaps, FILE *fh, int seek_off, guint8 *pd, int len);
+int wtap_seek_read (int file_type, FILE *fh, int seek_off, guint8 *pd, int len);
 int wtap_def_seek_read (FILE *fh, int seek_off, guint8 *pd, int len);
 
 wtap_dumper* wtap_dump_open(const char *filename, int filetype, int encap,
@@ -393,6 +412,17 @@ int wtap_pcap_encap_to_wtap_encap(int encap);
 #define	WTAP_ERR_SHORT_WRITE			-12
 	/* An attempt to write wrote less data than it should have */
 
+/* Errors from zlib; zlib error Z_xxx turns into Wiretap error
+   WTAP_ERR_ZLIB + Z_xxx.
+
+   WTAP_ERR_ZLIB_MIN and WTAP_ERR_ZLIB_MAX bound the range of zlib
+   errors; we leave room for 100 positive and 100 negative error
+   codes. */
+
+#define	WTAP_ERR_ZLIB				-200
+#define	WTAP_ERR_ZLIB_MAX			-100
+#define	WTAP_ERR_ZLIB_MIN			-300
+
 /* Pointer versions of ntohs and ntohl.  Given a pointer to a member of a
  * byte array, returns the value of the two or four bytes at the pointer.
  * The pletoh[sl] versions return the little-endian representation.
